#!/usr/local/bin/ruby
# $Id: document_bench.rb,v 1.15.2.1 2002/12/27 15:18:09 toki Exp $
# -
# wwwsrv's document benchmark

require 'benchmark'		# -> http://www.notwork.org/~gotoken/ruby/p/benchmark/
require 'wwwsrv/logger'
require 'wwwsrv/request'
require 'wwwsrv/response'
require 'wwwsrv/document'
require 'wwwsrv/server'

include Benchmark
STDOUT.sync = true

trial = (ARGV.first && ARGV.first.to_i) || 1_000

def comma(num)
  num.to_s.reverse.scan(/\d\d?\d?/).join(',').reverse
end

pseudo_output = Object::new
def pseudo_output.write(msg); self; end
def pseudo_output.flush; self; end

def make_request(method='GET', path='/')
  request = WWWsrv::Request::new(method, path, WWWsrv::HTTP_1_1, WWWsrv::Header::new)
  request.host = host = WWWsrv::Host::new('localhost', 8088)
  request.header.accept = 'image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, */*'
  request.header.accept_language = 'ja'
  request.header.accept_encoding = 'gzip, deflate'
  request.header.user_agent = 'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0)'
  request.header.host = 'localhost:8088'
  request.header.connection = 'Keep-Alive'
  request.client = WWWsrv::Client::new('localhost', '129.21.64.206')
  response = WWWsrv::Response::new

  return request, response
end

prefix = ''
srv_opt = WWWsrv::ServerOption::new
srv_opt.sid = 'A.1-1'
srv_opt.log = WWWsrv::Logger::new(pseudo_output)

puts "[WWWsrv::Page * #{comma(trial)}]"
page = WWWsrv::Page::new("Hello world.\n", 'text/plain', Time::now)
puts(measure{
  trial.times do
    request, response = make_request
    page.respond(srv_opt, prefix, request, response) do |arg, *opts|
      # nothing to do.
    end
  end
})

puts "[WWWsrv::DocumentPart * #{comma(trial)}]"
part = WWWsrv::DocumentPart::new(page, '/prefix')
puts "1. path"
puts(measure{
  trial.times do
    request, response = make_request
    request.path = '/'
    part.respond(srv_opt, prefix, request, response) do |arg, *opts|
      # nothing to do.
    end
  end
})
puts "2. prefix"
prefix2 = "/prefix"
puts(measure{
  trial.times do
    request, response = make_request
    request.path = '/prefix/index.html'
    part.respond(srv_opt, prefix2, request, response) do |arg, *opts|
      # nothing to do.
    end
  end
})

puts "[WWWsrv::DocumentFolder * #{comma(trial)}]"
folder = WWWsrv::DocumentFolder::new
folder.mount(page, '/')
folder.mount(page, '/', :page => 'index.html')
folder.mount(page, '/first')
folder.mount(page, '/first', :page => 'index.html')
folder.mount(page, '/first/second')
folder.mount(page, '/first/second', :page => 'index.html')
%w[
  /
  /index.html
  /first/
  /first/index.html
  /first/second/
  /first/second/index.html
].each_with_index do |path, i|
  puts "#{i + 1}. #{path}"
  puts(measure{
    trial.times do
      request, response = make_request('GET', path)
      folder.respond(srv_opt, prefix, request, response) do |arg, *opts|
        # nothing to do.
      end
    end
  })
end

puts "[WWWsrv::Redirect * #{comma(trial)}]"
redirect = WWWsrv::Redirect::new{ |srv_opt, prefix, request| 'http://localhost/welcom.html' }
puts(measure{
  trial.times do
    request, response = make_request
    redirect.respond(srv_opt, prefix, request, response) do |arg, *opts|
      # nothing to do.
    end
  end
})

puts "[WWWsrv::DocumentModifier * #{comma(trial)}]"
modifier = WWWsrv::DocumentModifier::new(page)
puts(measure{
  trial.times do
    request, response = make_request
    modifier.respond(srv_opt, prefix, request, response) do |arg, *opts|
      # nothing to do.
    end
  end
})

puts "[WWWsrv::AccessControl * #{comma(trial)}]"
for i in 1..10
  puts "#{i}. #{i} step"
  access = WWWsrv::AccessControl::new(page)
  i.times do
    access.add_access_control(:allow) { |srv_opt, prefix, request|
      request.client.name != 'localhost'
    }
  end
  access.add_access_control(:allow) { |srv_opt, prefix, request|
    request.client.name == 'localhost'
  }
  puts(measure{
    trial.times do
      request, response = make_request
      access.respond(srv_opt, prefix, request, response) do |arg, *opts|
	# nothing to do.
      end
    end
  })
end

puts "[WWWsrv::PartialContent * #{comma(trial)}]"
partial = WWWsrv::PartialContent::new(page)
puts "1. throuth"
puts(measure{
  trial.times do
    request, response = make_request
    partial.respond(srv_opt, prefix, request, response) do |arg, *opts|
      # nothing to do.
    end
  end
})
puts "2. range"
puts(measure{
  trial.times do
    request, response = make_request
    request.header.range = 'bytes=8-'
    partial.respond(srv_opt, prefix, request, response) do |arg, *opts|
      # nothing to do.
    end
  end
})

puts "[standard document * #{trial}]"
document = WWWsrv::PartialContent::new(folder)
watcher = WWWsrv::ResponseWatcher::new
wbuf = WWWsrv::WriteBuffer::new(pseudo_output, WWWsrv::TIMEOUT, WWWsrv::BUFFER_SIZE)
puts(measure{
  trial.times do
    request, response = make_request
    res_writer = WWWsrv::ResponseWriter::new(wbuf, request, watcher)
    WWWsrv::Document::catch_status{
      document.respond(srv_opt, prefix, request, response) do |arg, *opts|
	res_writer.write(srv_opt.sid, arg, *opts)
      end
    }
    res_writer.terminate(srv_opt.sid)
  end
})
