#!/usr/local/bin/ruby
# $Id: header_bench.rb,v 1.4 2002/04/20 01:00:55 toki Exp $
# -
# wwwsrv's header accessor benchmark

require 'benchmark'		# -> http://www.notwork.org/~gotoken/ruby/p/benchmark/
require 'wwwsrv/header'

include Benchmark
STDOUT.sync = true

trial = (ARGV.first && ARGV.first.to_i) || 1_000

src = [
  [ 'Accept', 'image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, */*' ],
  [ 'Accept-Language', 'ja' ],
  [ 'Accept-Encoding', 'gzip, deflate' ],
  [ 'User-Agent', 'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0)' ],
  [ 'Host', 'localhost:8088' ],
  [ 'Connection', 'Keep-Alive' ]
]

def comma(num)
  num.to_s.reverse.scan(/\d\d?\d?/).join(',').reverse
end

puts "[WWWsrv::Header.normalize * #{comma(trial)}]"

names = src.map{ |name, value| name }
puts(measure{
  trial.times do
    for field_name in names
      WWWsrv::Header::normalize(field_name)
    end
  end
})

puts "[WWWsrv::Header\#[] * #{comma(trial)}]"

header = WWWsrv::Header::new
for field_name, field_value in src
  header[field_name] = field_value
end
names = src.map{ |name, value| name }

puts(measure{
  trial.times do
    for field_name in names
      header[field_name]
    end
  end
})

puts "[WWWsrv::Header\#[]= * #{comma(trial)}]"

header = WWWsrv::Header::new
puts(measure{
  trial.times do
    for field_name, field_value in src
      header[field_name] = field_value
    end
  end
})

puts "[WWWsrv::Header\#header_field * #{comma(trial)}]"

header = WWWsrv::Header::new
for field_name, field_value in src
  header[field_name] = field_value
end

puts(measure{
  trial.times do
    header.accept
    header.accept_language
    header.accept_encoding
    header.user_agent
    header.host
    header.connection
  end
})

puts "[WWWsrv::Header\#header_field= * #{comma(trial)}]"

header = WWWsrv::Header::new
value_1 = 'image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, */*'
value_2 = 'ja'
value_3 = 'gzip, deflate'
value_4 = 'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0)'
value_5 = 'localhost:8088'
value_6 = 'Keep-Alive'

puts(measure{
  trial.times do
    header.accept = value_1
    header.accept_language = value_2
    header.accept_encoding = value_3
    header.user_agent = value_4
    header.host = value_5
    header.connection = value_6
  end
})

class PseudoInput
  def initialize(src)
    @src = src.clone
  end

  def gets(rs=nil)
    @src.shift
  end

  def self::convCRNF(src)
    src.map!{ |line|
      line.chomp!("\n")
      line.chomp!("\r")
      line.concat("\r\n")
    }
  end
end

header_source = PseudoInput::convCRNF(<<'EOF'.to_a)
Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, */*
Accept-Language: ja
Accept-Encoding: gzip, deflate
User-Agent: Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0)
Host: localhost:8088
Connection: Keep-Alive

EOF

puts "[WWWsrv::Header#parse * #{comma(trial)}]"
header = WWWsrv::Header::new
puts(measure{
  trial.times do
    header.clear
    header.parse(PseudoInput::new(header_source))
  end
})

puts "[WWWsrv::Header.parse * #{comma(trial)}]"
puts(measure{
  trial.times do
    WWWsrv::Header::parse(PseudoInput::new(header_source))
  end
})
