#!/usr/local/bin/ruby
# $Id: htpasswd.rb,v 1.9 2002/07/07 16:51:43 toki Exp $
# -
# Encrypted password file manipulation for wwwsrv's basic authentication scheme.

require 'md5'

class Password
  # Note: salt -> 8 bytes random characters
  def self::crypt(password, salt=format('%08x', rand(2**32)))
    (salt.length > 8) and salt = salt[0, 8]
    (salt.length < 8) and salt = ' ' * (8 - salt.length) + salt
    hash = MD5::new
    hash.update(salt)
    hash.update("\0")
    hash.update(password)
    salt + hash.hexdigest
  end

  class Comparator
    def initialize(username, encrypted_password)
      @username = username
      @encrypted_password = encrypted_password
    end

    def ==(password)
      salt = @encrypted_password[0, 8]
      @encrypted_password == Password::crypt(@username + "\0" + password, salt) ||
	@encrypted_password == Password::crypt(password, salt) # for backward compatibility
    end
  end

  def initialize(htpasswd)
    @htpasswd = htpasswd
  end

  def [](username)
    if (File::exist? @htpasswd) then
      File::open(@htpasswd) do |input|
	for line in input
	  line.strip!
	  (line.empty? || line =~ /^#/) and next
	  ht_username, ht_encrypted_password = line.split(/:/, 2)
	  if (ht_username == username) then
	    return Comparator::new(ht_username, ht_encrypted_password)
	  end
	end
      end
    end

    nil
  end
end

if ($0 == __FILE__) then
  if (ARGV.length != 1) then
    puts "Usage: #{File::basename($0)} [username]"
    exit 1
  end

  stty = nil
  for path in %w[ /bin/stty /usr/bin/stty ]
    if (File::executable? path) then
      stty = path
      break
    end
  end

  username = ARGV.shift
  STDERR.print "Password: "
  begin
    system(stty, '-echo') if (stty && STDIN.tty?)
    password = gets or raise "required password!"
    STDERR.print "\n"
  ensure
    system(stty, 'echo') if (stty && STDIN.tty?)
  end
  password.chomp!

  puts(username + ':' + Password::crypt(username + "\0" + password))
end
