#!/usr/local/bin/ruby
# $Id: parser_bench.rb,v 1.4 2002/04/13 03:53:20 toki Exp $
# -
# wwwsrv's parser benchmark

require 'benchmark'		# -> http://www.notwork.org/~gotoken/ruby/p/benchmark/
require 'wwwsrv/obsolete'

include Benchmark
STDOUT.sync = true

trial = (ARGV.first && ARGV.first.to_i) || 1_000

class PseudoInput
  def initialize(src)
    @src = src.dup
  end

  def gets(rs=nil)
    @src.shift
  end
end

def comma(num)
  num.to_s.reverse.scan(/\d\d?\d?/).join(',').reverse
end

def conv_nl(src)
  src.map!{ |line|
    line.chomp!("\n")
    line.chomp!("\r")
    line.concat("\r\n")
  }
end

puts "[WWWsrv::Parser.parse_http_version * #{comma(trial)}]"
[ 'HTTP/1.0',
  'HTTP/1.1',
  'HTTP/2.0'
].each_with_index do |src, i|
  puts "#{i + 1}. #{src}"
  puts(measure{
    trial.times do
      WWWsrv::Parser::parse_http_version(src)
    end
  })
end

puts "[WWWsrv::Parser.parse_header * #{comma(trial)}]"
src = conv_nl(<<'EOF'.to_a)
Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, */*
Accept-Language: ja
Accept-Encoding: gzip, deflate
User-Agent: Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0)
Host: localhost:8088
Connection: Keep-Alive

EOF
puts(measure{
  trial.times do
    WWWsrv::Parser::parse_header(PseudoInput::new(src))
  end
})

puts "[WWWsrv::Parser.parse_request_line * #{comma(trial)}]"
src = [ "GET / HTTP/1.1\r\n" ]
puts(measure{
  trial.times do
    WWWsrv::Parser::parse_request_line(PseudoInput::new(src))
  end
})

puts "[WWWsrv::Parser.parse_request * #{comma(trial)}]"
src = conv_nl(<<'EOF'.to_a)
GET / HTTP/1.1
Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, */*
Accept-Language: ja
Accept-Encoding: gzip, deflate
User-Agent: Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0)
Host: localhost:8088
Connection: Keep-Alive

EOF
puts(measure{
  trial.times do
    WWWsrv::Parser::parse_request(PseudoInput::new(src))
  end
})
