# -*- mode: Ruby; -*-
# $Id: wwwsrv.conf.fcgi,v 1.9 2002/11/27 05:43:49 toki Exp $
# -
# Sample configuration of FastCGI

__HOST__

################################################################
#          Initial parameters of WWWsrv::Server class          #
################################################################

# <NOTE>
# Constants
#   BASE_DIR -> --base-dir option parameter or current working directory

HOSTNAME = 'cernobog.plutonian.ne.jp'      # => --hostname=cernobog.plutonian.ne.jp
PORT = 8088			           # => --port=8088
STRICT = false			           # => --strict
LOG_FILE = nil			           # => --no-log-file
LOGGING_LEVEL = WWWsrv::Logger::DEBUG      # => --logging-level=debug
DISPLAY_LEVEL = WWWsrv::Logger::MESSAGE    # => --display-level=message
EFFECTIVE_USER_ID = 'nobody'	           # => --effective-user-id=nobody
EFFECTIVE_GROUP_ID = 'nobody'	           # => --effective-group-id=nobody
QUIET = false                              # => --no-quiet
ALARM = WWWsrv::DebugAlertDocument::new    # => --debug

__SERVER__

################################################################
#              Setup code of WWWsrv::Server class              #
################################################################

# <NOTE>
# Constants
#   BASE_DIR -> --base-dir option parameter or current working directory
#   SERVER   -> initialized WWWsrv::Server object

#### Sample of FastCGI ####

require 'socket'
require 'wwwsrv/fastcgi'

# <NOTE>
# run FastCGI application as...
#   % run_fcgi unix $HOME/tmp/fcgi test_fastcgi.rb
#       or
#   % run_fcgi inet 9000 test_fastcgi.rb
# files...
#   run_fcgi -> misc/run_fcgi.rb
#   test_fastcgi.rb -> misc/test_fastcgi.rb

UNIX_NOT_INET = true
KEEP_CONN = true

fcgi_doc = WWWsrv::FastCGIDocument::new(:keep_conn => KEEP_CONN) {
  if (UNIX_NOT_INET) then
    # for UNIX domain socket
    UNIXsocket::open(File::join(ENV['HOME'], 'tmp', 'fcgi'))
  else
    # for TCP socket
    TCPsocket::new('localhost', 9000)
    # If you want throughput, then a socket option of TCP_NODELAY may
    # be enabled at both ends of TCP connection.
    #socket.setsockopt(Socket::IPPROTO_TCP, Socket::TCP_NODELAY, 1)
  end
}
SERVER.mount(fcgi_doc, '/')

# <NOTE>
# run FastCGI authorizer as...
#   % run_fcgi unix $HOME/tmp/fauth test_fcgi_auth.rb
#       or
#   % run_fcgi inet 9001 test_fcgi_auth.rb
# files...
#   run_fcgi -> misc/run_fcgi.rb
#   test_fcgi_auth.rb -> misc/test_fcgi_auth.rb

SERVER.attach('/auth') { |document|
  fcgi_auth = WWWsrv::FastCGIAuthorizer::new(document, :keep_conn => KEEP_CONN) {
    if (UNIX_NOT_INET) then
      # for UNIX domain socket
      UNIXsocket::open(File::join(ENV['HOME'], 'tmp', 'fauth'))
    else
      # for TCP socket
      TCPsocket::new('localhost', 9001)
      # If you want throughput, then a socket option of TCP_NODELAY may
      # be enabled at both ends of TCP connection.
      #socket.setsockopt(Socket::IPPROTO_TCP, Socket::TCP_NODELAY, 1)
    end
  }
}

# <NOTE>
# run FastCGI filter as...
#   % run_fcgi unix $HOME/tmp/filter test_fcgi_filter.rb
#       or
#   % run_fcgi inet 9002 test_fcgi_filter.rb
# files...
#  run_fcgi -> misc/run_fcgi.rb
#  test_fcgi_filter.rb -> misc/test_fcgi_filter.rb

SERVER.attach('/filter') { |document|
  WWWsrv::FastCGIFilter::new(document, :keep_conn => KEEP_CONN) {
    if (UNIX_NOT_INET) then
      # for UNIX domain socket
      UNIXsocket::open(File::join(ENV['HOME'], 'tmp', 'filter'))
    else
      # for TCP socket
      TCPsocket::new('localhost', 9002)
      # If you want throughput, then a socket option of TCP_NODELAY may
      # be enabled at both ends of TCP connection.
      #socket.setsockopt(Socket::IPPROTO_TCP, Socket::TCP_NODELAY, 1)
    end
  }
}
