# $Id: docutils.rb,v 1.13 2002/12/05 13:02:30 toki Exp $
# -
# Utilities for testing WWW server's documents

require 'pseudo_io'
require 'wwwsrv/const'
require 'wwwsrv/htmlconv'
require 'wwwsrv/logger'
require 'wwwsrv/reader'
require 'wwwsrv/srvopt'
require 'wwwsrv/request'
require 'wwwsrv/response'
require 'wwwsrv/document'

module TestWWWsrv
  module DocumentUtils
    def make_srv_opt(logging_level=WWWsrv::Logger::INTERNAL)
      log_output = PseudoIO::new
      logger = WWWsrv::Logger::new(log_output, logging_level)
      srv_opt = WWWsrv::ServerOption::new
      srv_opt.sid = '1'
      srv_opt.log = logger

      return srv_opt, log_output
    end
    module_function :make_srv_opt

    def make_request(method='GET', path='/', opts={})
      request = WWWsrv::Request::new(method, path, opts[:version] || WWWsrv::HTTP_1_1)
      request.header = WWWsrv::Header::new
      request.header.user_agent = opts[:user_agent] || "WWWsrvTest/#{WWWsrv::VERSION}"
      request.header.host = opts[:host] || 'localhost'
      request.host = WWWsrv::Host::fetch_cache(request.header.host.split(/:/, 2)[0],
					       (request.header.host.split(/:/, 2)[1] || '80').to_i)
      request.client = WWWsrv::Client::fetch_cache(opts[:client_name] || 'localhost',
						   opts[:client_address] || '127.0.0.1')

      body_input = PseudoIO::new
      body_input.write(opts[:body]) if (opts.include? :body)
      request.body = body_input

      return request, body_input
    end
    module_function :make_request

    def make_response(status=200, version=WWWsrv::HTTP_1_1)
      response = WWWsrv::Response::new(version, status)
      response.header.date = WWWsrv::HTMLconv::http_date(Time::now)
      response.header.server = WWWsrv::product_token

      response
    end
    module_function :make_response

    def try_to_respond(document, srv_opt, prefix, request, response)
      reply_response = nil
      body_spool = nil
      body_alist = []

      doc_status, err_args = WWWsrv::Document::catch_status{
	document.respond(srv_opt, prefix, request, response) do |arg, opts|
	  unless (reply_response) then
	    reply_response = arg
	  else
	    body_spool = '' unless body_spool
	    body_spool.concat(arg) if arg
	    body_alist.push([ arg, opts ])
	  end
	end
      }

      return {
	:doc_status    => doc_status,
	:err_args      => err_args,
	:response      => reply_response,
	:body          => body_spool,
	:body_alist    => body_alist
      }
    end
    module_function :try_to_respond
  end
end
