#!/usr/local/bin/ruby
# $Id: pseudo_doc.rb,v 1.10.2.1 2003/01/05 13:06:18 toki Exp $
# -
# Pseudo Document for WWW server test

require 'pseudo_io'
require 'docutils'
require 'wwwsrv/document'
require 'runit/testcase'
require 'runit/cui/testrunner'

module TestWWWsrv
  class PseudoDocument < WWWsrv::Document
    def initialize(content_type, content_fragments)
      @content_type = content_type
      @content = content_fragments
      @length = nil
    end

    attr_reader :content_type

    def set_content_length
      @length = 0
      for content_fragment in @content
	@length += content_fragment.length
      end

      self
    end

    def unset_content_length
      @length = nil
      self
    end

    def respond(srv_opt, prefix, request, response)
      srv_opt.doc_log_debug(self, 'enter')
      response.header.content_type = @content_type
      response.header.content_length = @length.to_s if @length
      yield(response)
      for content_fragment in @content
	yield(content_fragment)
      end

      self
    end
  end
end

if ($0 == __FILE__) then
  module TestWWWsrv
    class TestPseudoDocument < RUNIT::TestCase
      include DocumentUtils

      def setup
	@content = [
	  '01234',
	  '56789',
	  'ABCDE'
	]
	@document = PseudoDocument::new('text/plain', @content)
      end

      def test_respond
	srv_opt, log_output = make_srv_opt
	request, body_input = make_request('GET', '/')
	response = make_response
	respondent = try_to_respond(@document, srv_opt, '', request, response)
	assert_equal(:success, respondent[:doc_status])
	assert_equal(200, respondent[:response].status)
	assert_equal('text/plain', respondent[:response].header.content_type)
	assert_nil(respondent[:response].header.content_length)
	assert_equal(@content.join, respondent[:body])
	assert_equal(@content, respondent[:body_alist].map{ |fragment,*opts| fragment })
      end

      def test_respond_with_length
	@document.set_content_length
	srv_opt, log_output = make_srv_opt
	request, body_input = make_request('GET', '/')
	response = make_response
	respondent = try_to_respond(@document, srv_opt, '', request, response)
	assert_equal(:success, respondent[:doc_status])
	assert_equal(200, respondent[:response].status)
	assert_equal('text/plain', respondent[:response].header.content_type)
	assert_equal(@content.join.length, respondent[:response].header.content_length.to_i)
	assert_equal(@content.join, respondent[:body])
	assert_equal(@content, respondent[:body_alist].map{ |fragment,*opts| fragment })
      end
    end
  end

  if (ARGV.empty?) then
    TestWWWsrv::constants.map{ |name|
      TestWWWsrv::const_get(name)
    }.select{ |const|
      (const.kind_of? Class) && (const < RUNIT::TestCase)
    }.each do |test_case|
      RUNIT::CUI::TestRunner::run(test_case::suite)
    end
  else
    for class_name in ARGV
      test_case = TestWWWsrv.const_get("Test#{class_name}")
      RUNIT::CUI::TestRunner.run(test_case.suite)
    end
  end
end
