#!/usr/local/bin/ruby
# $Id: test_alert.rb,v 1.8.2.1 2003/01/05 13:06:20 toki Exp $
# -
# Test of WWWsrv::AlertDocument class
# Test of WWWsrv::DebugAlertDocument class

require 'docutils'
require 'pseudo_io'
require 'pseudo_doc'
require 'wwwsrv/srvopt'
require 'wwwsrv/request'
require 'wwwsrv/response'
require 'wwwsrv/logger'
require 'wwwsrv/alert'
require 'runit/testcase'
require 'runit/cui/testrunner'

module TestWWWsrv
  class TestAlertDocument < RUNIT::TestCase
    include DocumentUtils

    def setup
      @alert = WWWsrv::AlertDocument::new
    end

    def test_new
      assert_instance_of(WWWsrv::AlertDocument, @alert)
    end

    def test_respond
      srv_opt, log_output = make_srv_opt
      srv_opt[:error] = 'test'
      request, body_input = make_request('GET', '/')
      response = make_response(404)
      respondent = try_to_respond(@alert, srv_opt, '', request, response)

      assert_instance_of(WWWsrv::Response, respondent[:response])
      assert_equal('text/html', respondent[:response].header.content_type)
      assert(respondent[:body].length > 0)
    end
  end

  class TestDebugAlertDocument < RUNIT::TestCase
    include DocumentUtils

    def setup
      @alert = WWWsrv::DebugAlertDocument::new
    end

    def test_new
      assert_instance_of(WWWsrv::DebugAlertDocument, @alert)
    end

    def test_respond
      srv_opt, log_output = make_srv_opt
      srv_opt[:error] = 'test'
      request, body_input = make_request('GET', '/')
      response = make_response(404)
      respondent = try_to_respond(@alert, srv_opt, '', request, response)

      assert_instance_of(WWWsrv::Response, respondent[:response])
      assert_equal('text/html', respondent[:response].header.content_type)
      assert(respondent[:body].length > 0)
    end
  end
end

if ($0 == __FILE__) then
  if (ARGV.empty?) then
    TestWWWsrv::constants.map{ |name|
      TestWWWsrv::const_get(name)
    }.select{ |const|
      (const.kind_of? Class) && (const < RUNIT::TestCase)
    }.each do |test_case|
      RUNIT::CUI::TestRunner::run(test_case::suite)
    end
  else
    for class_name in ARGV
      test_case = TestWWWsrv.const_get("Test#{class_name}")
      RUNIT::CUI::TestRunner.run(test_case.suite)
    end
  end
end
