#!/usr/local/bin/ruby
# $Id: test_auth.rb,v 1.5.2.1 2003/01/05 13:06:21 toki Exp $
# -
# Test of Basic Authentication

require 'docutils'
require 'wwwsrv/request'
require 'wwwsrv/response'
require 'wwwsrv/auth'
require 'runit/testcase'
require 'runit/cui/testrunner'

module TestWWWsrv
  class TestBasicAuthentication < RUNIT::TestCase
    include DocumentUtils

    def setup
      @realm = 'Basic Authentication Test'
      @passwd = {
	'Alice' => 'foo',
	'Bob' => 'bar'
      }
      @page = WWWsrv::Page::new('Authenticated!', 'text/plain')
      @auth = WWWsrv::BasicAuthentication::new(@page, @realm, @passwd)
    end

    def test_new
      assert_instance_of(WWWsrv::BasicAuthentication, @auth)
    end

    def test_respond_allow
      [ %w[ Alice foo ],
	%w[ Bob   bar ]
      ].each do |username, password|
	srv_opt, log_output = make_srv_opt
	request, body_input = make_request('GET', '/')
	request.header.authorization = 'Basic ' + [ username + ':' + password ].pack('m')
	response = make_response
	respondent = try_to_respond(@auth, srv_opt, '', request, response)

	assert_equal(:success, respondent[:doc_status])
	assert_instance_of(WWWsrv::Response, respondent[:response])
	assert_equal(200, respondent[:response].status)
	assert_not_nil(respondent[:response].header.www_authenticate)
	assert_match(respondent[:response].header.www_authenticate, /#{Regexp::quote(@realm)}/)
      end
    end

    def test_respond_deny
      [ %w[ Alice bar ],
	%w[ Bob   foo ],
	%w[ Carol baz ]
      ].each do |username, password|
	srv_opt, log_output = make_srv_opt
	request, body_input = make_request('GET', '/')
	request.header.authorization = 'Basic ' + [ username + ':' + password ].pack('m')
	response = make_response
	respondent = try_to_respond(@auth, srv_opt, '', request, response)

	assert_equal(:success, respondent[:doc_status])
	assert_instance_of(WWWsrv::Response, respondent[:response])
	assert_equal(401, respondent[:response].status)
	assert_not_nil(respondent[:response].header.www_authenticate)
	assert_match(respondent[:response].header.www_authenticate, /#{Regexp::quote(@realm)}/)
      end
    end
  end
end

if ($0 == __FILE__) then
  if (ARGV.empty?) then
    TestWWWsrv::constants.map{ |name|
      TestWWWsrv::const_get(name)
    }.select{ |const|
      (const.kind_of? Class) && (const < RUNIT::TestCase)
    }.each do |test_case|
      RUNIT::CUI::TestRunner::run(test_case::suite)
    end
  else
    for class_name in ARGV
      test_case = TestWWWsrv.const_get("Test#{class_name}")
      RUNIT::CUI::TestRunner.run(test_case.suite)
    end
  end
end
