#!/usr/local/bin/ruby
# $Id: test_cgi.rb,v 1.15.2.1 2003/01/05 13:06:23 toki Exp $
# -
# Test of WWWsrv::CGIdocument class

require 'docutils'
require 'pseudo_io'
require 'pseudo_doc'
require 'wwwsrv/srvopt'
require 'wwwsrv/request'
require 'wwwsrv/response'
require 'wwwsrv/logger'
require 'wwwsrv/cgi'
require 'runit/testcase'
require 'runit/cui/testrunner'

module TestWWWsrv
  class TestCGIdocument < RUNIT::TestCase
    include DocumentUtils

    def setup
      @cgi_path = 'test.cgi'
      @cgi_doc = WWWsrv::CGIdocument::new(Dir::getwd, false, ENV['RUBY'] || 'ruby')
    end

    def test_new
      assert_instance_of(WWWsrv::CGIdocument, @cgi_doc)
    end

    def test_respond
      content = "Hello world.\n"
      open(@cgi_path, 'w') do |output|
	output.binmode
	output.write <<-EOF
#!/usr/local/bin/ruby
STDOUT.binmode
content = #{content.dump}
print "Content-Type: text/plain\\r\\n"
print "Content-Length: \#{content.length}\\r\\n"
print "\\r\\n"
print content
	EOF
      end

      begin
	srv_opt, log_output = make_srv_opt
	request, body_input = make_request('GET', '/test.cgi')
	response = make_response
	respondent = try_to_respond(@cgi_doc, srv_opt, '', request, response)
	assert_equal(:success, respondent[:doc_status])
	assert_instance_of(WWWsrv::Response, respondent[:response])
	assert_equal(200, respondent[:response].status)
	assert_equal('OK', respondent[:response].reason)
	assert_equal('text/plain', respondent[:response].header.content_type)
	assert_equal(content.length.to_s, respondent[:response].header.content_length)
	assert_equal(content, respondent[:body])
      ensure
	File::unlink(@cgi_path)
      end
    end

    def test_respond_with_status
      content = "Hello world.\n"
      open(@cgi_path, 'w') do |output|
	output.binmode
	output.write <<-EOF
#!/usr/local/bin/ruby
STDOUT.binmode
content = #{content.dump}
print "Status: 402 Forbidden\\r\\n"
print "Content-Type: text/plain\\r\\n"
print "Content-Length: \#{content.length}\\r\\n"
print "\\r\\n"
print content
	EOF
      end

      begin
	srv_opt, log_output = make_srv_opt
	request, body_input = make_request('GET', '/test.cgi')
	response = make_response
	respondent = try_to_respond(@cgi_doc, srv_opt, '', request, response)
	assert_equal(:success, respondent[:doc_status])
	assert_instance_of(WWWsrv::Response, respondent[:response])
	assert_equal(402, respondent[:response].status)
	assert_equal('Forbidden', respondent[:response].reason)
	assert_equal('text/plain', respondent[:response].header.content_type)
	assert_equal(content.length.to_s, respondent[:response].header.content_length)
	assert_equal(content, respondent[:body])
      ensure
	File::unlink(@cgi_path)
      end
    end

    def test_respond_with_location_curr_path
      content = "Hello world.\n"
      open(@cgi_path, 'w') do |output|
	output.binmode
	output.write <<-EOF
#!/usr/local/bin/ruby
STDOUT.binmode
content = #{content.dump}
print "Location: index.html\\r\\n"
print "Content-Type: text/plain\\r\\n"
print "Content-Length: \#{content.length}\\r\\n"
print "\\r\\n"
print content
	EOF
      end

      begin
	srv_opt, log_output = make_srv_opt
	request, body_input = make_request('GET', '/test.cgi')
	response = make_response
	respondent = try_to_respond(@cgi_doc, srv_opt, '', request, response)
	assert_equal(:success, respondent[:doc_status])
	assert_instance_of(WWWsrv::Response, respondent[:response])
	assert_equal(302, respondent[:response].status)
	assert_equal('http://localhost:80/index.html', respondent[:response].header.location)
	assert_equal('Found', respondent[:response].reason)
	assert_equal(content.length.to_s, respondent[:response].header.content_length)
	assert_equal('text/plain', respondent[:response].header.content_type)
	assert_equal(content, respondent[:body])
      ensure
	File::unlink(@cgi_path)
      end
    end

    def test_respond_with_location_rel_path
      content = "Hello world.\n"
      open(@cgi_path, 'w') do |output|
	output.binmode
	output.write <<-EOF
#!/usr/local/bin/ruby
STDOUT.binmode
content = #{content.dump}
print "Location: /index.html\\r\\n"
print "Content-Type: text/plain\\r\\n"
print "Content-Length: \#{content.length}\\r\\n"
print "\\r\\n"
print content
	EOF
      end

      begin
	srv_opt, log_output = make_srv_opt
	request, body_input = make_request('GET', '/test.cgi')
	response = make_response
	respondent = try_to_respond(@cgi_doc, srv_opt, '', request, response)
	assert_equal(:success, respondent[:doc_status])
	assert_instance_of(WWWsrv::Response, respondent[:response])
	assert_equal(302, respondent[:response].status)
	assert_equal('http://localhost:80/index.html', respondent[:response].header.location)
	assert_equal('Found', respondent[:response].reason)
	assert_equal(content.length.to_s, respondent[:response].header.content_length)
	assert_equal('text/plain', respondent[:response].header.content_type)
	assert_equal(content, respondent[:body])
      ensure
	File::unlink(@cgi_path)
      end
    end

    def test_respond_with_location_abs_path
      content = "Hello world.\n"
      open(@cgi_path, 'w') do |output|
	output.binmode
	output.write <<-EOF
#!/usr/local/bin/ruby
STDOUT.binmode
content = #{content.dump}
print "Location: http://localhost/index.html\\r\\n"
print "Content-Type: text/plain\\r\\n"
print "Content-Length: \#{content.length}\\r\\n"
print "\\r\\n"
print content
	EOF
      end

      begin
	srv_opt, log_output = make_srv_opt
	request, body_input = make_request('GET', '/test.cgi')
	response = make_response
	respondent = try_to_respond(@cgi_doc, srv_opt, '', request, response)
	assert_equal(:success, respondent[:doc_status])
	assert_instance_of(WWWsrv::Response, respondent[:response])
	assert_equal(302, respondent[:response].status)
	assert_equal('http://localhost/index.html', respondent[:response].header.location)
	assert_equal('Found', respondent[:response].reason)
	assert_equal(content.length.to_s, respondent[:response].header.content_length)
	assert_equal('text/plain', respondent[:response].header.content_type)
	assert_equal(content, respondent[:body])
      ensure
	File::unlink(@cgi_path)
      end
    end

    def test_respond_none_parsed_header
      @cgi_doc = WWWsrv::CGIdocument::new(Dir::getwd, true, ENV['RUBY'] || 'ruby')
      content = "Hello world.\n"
      open(@cgi_path, 'w') do |output|
	output.binmode
	output.write <<-EOF
#!/usr/local/bin/ruby
STDOUT.binmode
content = #{content.dump}
print "HTTP/1.0 200 OK\\r\\n"
print "Content-Type: text/plain\\r\\n"
print "Content-Length: #{content.length}\\r\\n"
print "\\r\\n"
print content
	EOF
      end

      begin
	srv_opt, log_output = make_srv_opt
	request, body_input = make_request('GET', '/test.cgi')
	response = make_response
	respondent = try_to_respond(@cgi_doc, srv_opt, '', request, response)
	assert_equal(:success, respondent[:doc_status])
	assert_instance_of(WWWsrv::Response, respondent[:response])
	assert_equal(WWWsrv::HTTP_1_0, respondent[:response].version)
	assert_equal(200, respondent[:response].status)
	assert_equal('OK', respondent[:response].reason)
	assert_equal('text/plain', respondent[:response].header.content_type)
	assert_equal(content.length.to_s, respondent[:response].header.content_length)
	assert_equal(content, respondent[:body])
      ensure
	File::unlink(@cgi_path)
      end
    end

    def test_empty_response
      open(@cgi_path, 'w') do |output|
	output.binmode
	output.write <<-EOF
#!/usr/local/bin/ruby
# nothing to do.
	EOF
      end

      begin
	srv_opt, log_output = make_srv_opt
	request, body_input = make_request('GET', '/test.cgi')
	response = make_response
	respondent = try_to_respond(@cgi_doc, srv_opt, '', request, response)
	assert_equal(:error, respondent[:doc_status])
	status, message, header, version = respondent[:err_args]
	assert_equal(500, status)
      ensure
	File::unlink(@cgi_path)
      end
    end

    WWWsrv::Header::define_field('X-Test-Append')
    WWWsrv::Header::define_field('X-Test-Not-Append')

    def test_append_header_field
      open(@cgi_path, 'w') do |output|
	output.binmode
	output.write <<-EOF
#!/usr/local/bin/ruby
STDOUT.binmode
print "X-Test-Append: append\\r\\n"
print "X-Test-Not-Append: first\\r\\n"
print "X-Test-Not-Append: second\\r\\n"
print "\\r\\n"
	EOF
      end

      begin
	srv_opt, log_output = make_srv_opt
	request, body_input = make_request('GET', '/test.cgi')
	response = make_response
	response.header.x_test_append = 'first'
	respondent = try_to_respond(@cgi_doc, srv_opt, '', request, response)
	assert_equal(:success, respondent[:doc_status])
	assert_equal('first, append', respondent[:response].header.x_test_append)
	assert_equal('first, second', respondent[:response].header.x_test_not_append)
      ensure
	File::unlink(@cgi_path)
      end
    end
  end
end

if ($0 == __FILE__) then
  if (ARGV.empty?) then
    TestWWWsrv::constants.map{ |name|
      TestWWWsrv::const_get(name)
    }.select{ |const|
      (const.kind_of? Class) && (const < RUNIT::TestCase)
    }.each do |test_case|
      RUNIT::CUI::TestRunner::run(test_case::suite)
    end
  else
    for class_name in ARGV
      test_case = TestWWWsrv.const_get("Test#{class_name}")
      RUNIT::CUI::TestRunner.run(test_case.suite)
    end
  end
end
