#!/usr/local/bin/ruby
# $Id: test_delegate.rb,v 1.2.2.2 2003/01/05 13:06:25 toki Exp $
# -
# Test of WWWsrv::Delegatable module

require 'wwwsrv/delegate'
require 'runit/testcase'
require 'runit/cui/testrunner'

module TestWWWsrv
  class TestDelegatable < RUNIT::TestCase
    # alias for ruby-1.6
    unless (defined? NoMethodError) then
      NoMethodError = NameError
    end

    def setup
      @src_class = Class::new
      @src_class.class_eval %q{
	def foo
	  'foo of src_obj'
	end

	def bar
	  'bar of src_obj'
	end

	def baz
	  'baz of src_obj'
	end
	protected :baz

	def qux
	  'qux of src_obj'
	end
	private :qux
      }

      @deleg_class = Class::new
      @deleg_class.class_eval %q{
	def initialize(src_obj)
	  @src_obj = src_obj
	end

	def __src_obj__
	  @src_obj
	end
	private :__src_obj__

	def bar
	  'bar of deleg_obj'
	end
      }
    end

    def test_source
      src_obj = @src_class.new
      assert_equal('foo of src_obj', src_obj.foo)
      assert_equal('bar of src_obj', src_obj.bar)
      assert_exception(NoMethodError) { src_obj.baz }
      assert_exception(NoMethodError) { src_obj.qux }

      deleg_obj = @deleg_class.new(src_obj)
      assert_exception(NoMethodError) { deleg_obj.foo }
      assert_equal('bar of deleg_obj', deleg_obj.bar)
      assert_exception(NoMethodError) { deleg_obj.baz }
      assert_exception(NoMethodError) { deleg_obj.qux }
    end

    def test_delegation
      src_class = @src_class
      @deleg_class.class_eval %q{
        include WWWsrv::Delegatable
        delegate(src_class, :__src_obj__)
      }

      deleg_obj = @deleg_class.new(@src_class.new)
      assert_equal('foo of src_obj', deleg_obj.foo)
      assert_equal('bar of deleg_obj', deleg_obj.bar)
      #assert_exception(NoMethodError) { deleg_obj.baz }
      assert_exception(NoMethodError) { deleg_obj.qux }
    end

    def test_delegation_with_wrapper
      src_class = @src_class
      @deleg_class.class_eval %q{
        include WWWsrv::Delegatable
        delegate(src_class, :__src_obj__, :__wrap__)

        def __wrap__(id, opt)
	  "[#{id} with #{opt}: " + yield([]) + ']'
        end
	private :__wrap__
      }

      deleg_obj = @deleg_class.new(@src_class.new)
      assert_equal('[foo with option: foo of src_obj]', deleg_obj.foo(:option))
      assert_equal('bar of deleg_obj', deleg_obj.bar)
      #assert_exception(NoMethodError) { deleg_obj.baz(:option) }
      assert_exception(NoMethodError) { deleg_obj.qux }
    end
  end
end

if ($0 == __FILE__) then
  if (ARGV.empty?) then
    TestWWWsrv::constants.map{ |name|
      TestWWWsrv::const_get(name)
    }.select{ |const|
      (const.kind_of? Class) && (const < RUNIT::TestCase)
    }.each do |test_case|
      RUNIT::CUI::TestRunner::run(test_case::suite)
    end
  else
    for class_name in ARGV
      test_case = TestWWWsrv.const_get("Test#{class_name}")
      RUNIT::CUI::TestRunner.run(test_case.suite)
    end
  end
end
