#!/usr/local/bin/ruby
# $Id: test_fastcgi.rb,v 1.22.2.2 2003/01/05 13:06:30 toki Exp $
# -
# Test of WWWsrv::FastCGIdocument class

require 'docutils'
require 'pseudo_io'
require 'wwwsrv/fastcgi'
require 'runit/testcase'
require 'runit/cui/testrunner'

module TestWWWsrv
  class TestFastCGIRecord < RUNIT::TestCase
    include WWWsrv::FastCGIconst

    def setup
      @record = WWWsrv::FastCGIRecord::new
    end

    def test_receive_from
      [ [ FCGI_VERSION, FCGI_BEGIN_REQUEST,     0,  '',         0  ],
	[ FCGI_VERSION, FCGI_ABORT_REQUEST,     1,  'first',    1  ],
	[ FCGI_VERSION, FCGI_END_REQUEST,       2,  'second',   2  ],
	[ FCGI_VERSION, FCGI_PARAMS,            3,  'third',    3  ],
	[ FCGI_VERSION, FCGI_STDIN,             4,  'fourth',   4  ],
	[ FCGI_VERSION, FCGI_STDOUT,            5,  'fifth',    5  ],
	[ FCGI_VERSION, FCGI_STDERR,            6,  'sixth',    6  ],
	[ FCGI_VERSION, FCGI_DATA,              7,  'seventh',  7  ],
	[ FCGI_VERSION, FCGI_GET_VALUES,        8,  'eighth',   8  ],
	[ FCGI_VERSION, FCGI_GET_VALUES_RESULT, 9,  'ninth',    9  ],
	[ FCGI_VERSION, FCGI_UNKNOWN_TYPE,      10, 'tenth',    10 ],
	[ FCGI_VERSION, FCGI_MAXTYPE,           11, 'eleventh', 11 ]
      ].each do |version, record_type, request_id, content, reserved|
	padding_length = content.length % 8

	input = PseudoIO::new
	input.write(version.chr)
	input.write(record_type.chr)
	input.write(((request_id & 0xFF00) >> 8).chr)
	input.write((request_id & 0x00FF).chr)
	input.write(((content.length & 0xFF00) >> 8).chr)
	input.write((content.length & 0x00FF).chr)
	input.write(padding_length.chr)
	input.write(reserved.chr)
	input.write(content)
	input.write("\0" * padding_length)

	@record.clear
	assert_nil(@record.record_type)
	assert_nil(@record.request_id)
	assert_nil(@record.content)
	assert_equal(0, @record.padding_length)
	assert_equal(0, @record.reserved)
	assert_equal(FCGI_VERSION, @record.version)

	@record.receive_from(input)
	assert_equal(record_type, @record.record_type)
	assert_equal(request_id, @record.request_id)
	assert_equal((! content.empty?) ? content : nil , @record.content)
	assert_equal(padding_length, @record.padding_length)
	assert_equal(reserved, @record.reserved)

	assert_nil(input.data)
      end
    end

    def test_send_to
      [ [ FCGI_VERSION, FCGI_BEGIN_REQUEST,     0,  '',         0  ],
	[ FCGI_VERSION, FCGI_ABORT_REQUEST,     1,  'first',    1  ],
	[ FCGI_VERSION, FCGI_END_REQUEST,       2,  'second',   2  ],
	[ FCGI_VERSION, FCGI_PARAMS,            3,  'third',    3  ],
	[ FCGI_VERSION, FCGI_STDIN,             4,  'fourth',   4  ],
	[ FCGI_VERSION, FCGI_STDOUT,            5,  'fifth',    5  ],
	[ FCGI_VERSION, FCGI_STDERR,            6,  'sixth',    6  ],
	[ FCGI_VERSION, FCGI_DATA,              7,  'seventh',  7  ],
	[ FCGI_VERSION, FCGI_GET_VALUES,        8,  'eighth',   8  ],
	[ FCGI_VERSION, FCGI_GET_VALUES_RESULT, 9,  'ninth',    9  ],
	[ FCGI_VERSION, FCGI_UNKNOWN_TYPE,      10, 'tenth',    10 ],
	[ FCGI_VERSION, FCGI_MAXTYPE,           11, 'eleventh', 11 ]
      ].each do |version, record_type, request_id, content, reserved|
	padding_length = content.length % 8
	output = PseudoIO::new

	@record.clear
	assert_nil(@record.record_type)
	assert_nil(@record.request_id)
	assert_nil(@record.content)
	assert_equal(0, @record.padding_length)
	assert_equal(0, @record.reserved)
	assert_equal(FCGI_VERSION, @record.version)

	@record.record_type = record_type
	@record.request_id = request_id
	@record.content = (! content.empty?) ? content : nil
	@record.padding_length = padding_length
	@record.reserved = reserved
	@record.version = version
	@record.send_to(output)

	encoded_record = version.chr
	encoded_record += record_type.chr
	encoded_record += ((request_id & 0xFF00) >> 8).chr
	encoded_record += (request_id & 0x00FF).chr
	encoded_record += ((content.length & 0xFF00) >> 8).chr
	encoded_record += (content.length & 0x00FF).chr
	encoded_record += padding_length.chr
	encoded_record += reserved.chr
	encoded_record += content
	encoded_record += "\0" * padding_length
	assert_equal(encoded_record, output.read(8 + content.length + padding_length))
	assert_nil(output.data)
      end
    end

    def test_encode_name_value_pairs
      name_value_pairs = [
	[ 'first',  'foo' ],
	[ 'second', 'bar' ],
	[ 'third',  'baz' ]
      ]
      encoded_name_value_pairs = name_value_pairs.map{ |name, value|
	[ name.length, value.length ].pack('CC') + name + value
      }.join
      assert_equal(encoded_name_value_pairs, WWWsrv::FastCGIRecord::encode_name_value_pairs(name_value_pairs))

      long_name_value_pairs = [
	[ 'a' * 0b0111_1111, 'b' * 0b1000_0000 ],
	[ 'c' * 0b1000_0000, 'd' * 0b0111_1111 ],
	[ 'd' * 0b1000_0000, 'e' * 0b1000_0000 ]
      ]
      long_encoded_name_value_pairs =
	[ 0b0111_1111 ].pack('C') +
	[ 0b1000_0000, 0b0000_0000, 0b0000_0000, 0b1000_0000 ].pack('CCCC') +
	'a' * 0b0111_1111 + 'b' * 0b1000_0000 +
	[ 0b1000_0000, 0b0000_0000, 0b0000_0000, 0b1000_0000 ].pack('CCCC') +
	[ 0b0111_1111 ].pack('C') +
	'c' * 0b1000_0000 + 'd' * 0b0111_1111 +
	[ 0b1000_0000, 0b0000_0000, 0b0000_0000, 0b1000_0000 ].pack('CCCC') +
	[ 0b1000_0000, 0b0000_0000, 0b0000_0000, 0b1000_0000 ].pack('CCCC') +
	'd' * 0b1000_0000 + 'e' * 0b1000_0000
      assert_equal(long_encoded_name_value_pairs, WWWsrv::FastCGIRecord::encode_name_value_pairs(long_name_value_pairs))
    end

    def test_decode_name_value_pairs
      WWWsrv::FastCGIRecord::decode_name_value_pairs('') do |name, value|
	assert_fail('empty string is decoded.')
      end

      name_value_pairs = [
	[ 'first',  'foo' ],
	[ 'second', 'bar' ],
	[ 'third',  'baz' ],
	[ 'a' * 0b0111_1111, 'b' * 0b1000_0000 ],
	[ 'c' * 0b1000_0000, 'd' * 0b0111_1111 ],
	[ 'd' * 0b1000_0000, 'e' * 0b1000_0000 ]
      ]
      encoded_name_value_pairs = WWWsrv::FastCGIRecord::encode_name_value_pairs(name_value_pairs)
      WWWsrv::FastCGIRecord::decode_name_value_pairs(encoded_name_value_pairs) do |name, value|
	src_name, src_value = name_value_pairs.shift
	assert_equal(src_name, name)
	assert_equal(src_value, value)
      end
      assert(name_value_pairs.empty?)

      for one_pair in name_value_pairs
	encoded_one_pair = WWWsrv::FastCGIRecord::encode_name_value_pairs([ one_pair ])
	for len in 1...encoded_one_pair.length
	  assert_exception(WWWsrv::FastCGIParseError) {
	    WWWsrv::FastCGIRecord::encode_name_value_pairs(encoded_one_pair) do |name, value|
	      assert_fail('an invalid name value pair is parsed.')
	    end
	  }
	end
      end
    end

    def test_encode_begin_request
      [ [ FCGI_RESPONDER, FCGI_KEEP_CONN, "\0" * 5 ],
	[ FCGI_AUTHORIZER, FCGI_KEEP_CONN, "\0" * 4 + "\1" ],
	[ FCGI_FILTER, 0, "\0" * 4 + "\2" ]
      ].each do |role, flags, reserved|
	encoded_begin_request = ((role & 0xFF00) >> 8).chr + (role & 0x00FF).chr
	encoded_begin_request += flags.chr
	encoded_begin_request += reserved
	assert_equal(encoded_begin_request, WWWsrv::FastCGIRecord::encode_begin_request(role, flags, reserved))
      end
    end

    def test_decode_begin_request
      [ [ FCGI_RESPONDER, FCGI_KEEP_CONN, "\0" * 5 ],
	[ FCGI_AUTHORIZER, FCGI_KEEP_CONN, "\0" * 4 + "\1" ],
	[ FCGI_FILTER, 0, "\0" * 4 + "\2" ]
      ].each do |role, flags, reserved|
	encoded_begin_request = ((role & 0xFF00) >> 8).chr + (role & 0x00FF).chr
	encoded_begin_request += flags.chr
	encoded_begin_request += reserved
	decoded_begin_request = WWWsrv::FastCGIRecord::decode_begin_request(encoded_begin_request)
	assert_equal(3, decoded_begin_request.length)
	assert_equal(role, decoded_begin_request[0])
	assert_equal(flags, decoded_begin_request[1])
	assert_equal(reserved, decoded_begin_request[2])
      end
    end

    def test_encode_end_request
      [ [ 0, FCGI_REQUEST_COMPLETE, "\0\0\0" ],
	[ 1, FCGI_CANT_MPX_CONN, "\0\0\1" ],
	[ 2, FCGI_OVERLOADED, "\0\0\2" ],
	[ 3, FCGI_UNKNOWN_ROLE, "\0\0\3" ]
      ].each do |app_status, protocol_status, reserved|
	encoded_end_request = ((app_status & 0xFF000000) >> 24).chr
	encoded_end_request += ((app_status & 0x00FF0000) >> 16).chr
	encoded_end_request += ((app_status & 0x0000FF00) >> 8).chr
	encoded_end_request += (app_status & 0x000000FF).chr
	encoded_end_request += protocol_status.chr
	encoded_end_request += reserved
	assert_equal(encoded_end_request, WWWsrv::FastCGIRecord::encode_end_request(app_status, protocol_status, reserved))
      end
    end

    def test_decode_end_request
      [ [ 0, FCGI_REQUEST_COMPLETE, "\0\0\0" ],
	[ 1, FCGI_CANT_MPX_CONN, "\0\0\1" ],
	[ 2, FCGI_OVERLOADED, "\0\0\2" ],
	[ 3, FCGI_UNKNOWN_ROLE, "\0\0\3" ]
      ].each do |app_status, protocol_status, reserved|
	encoded_end_request = ((app_status & 0xFF000000) >> 24).chr
	encoded_end_request += ((app_status & 0x00FF0000) >> 16).chr
	encoded_end_request += ((app_status & 0x0000FF00) >> 8).chr
	encoded_end_request += (app_status & 0x000000FF).chr
	encoded_end_request += protocol_status.chr
	encoded_end_request += reserved
	decoded_end_request = WWWsrv::FastCGIRecord::decode_end_request(encoded_end_request)
	assert_equal(3, decoded_end_request.length)
	assert_equal(app_status, decoded_end_request[0])
	assert_equal(protocol_status, decoded_end_request[1])
	assert_equal(reserved, decoded_end_request[2])
      end
    end

    def test_encode_unknown_type
      [ [ 0, "\0" * 7 ],
	[ 1, "\0" * 6 + "\1" ],
	[ 2, "\0" * 6 + "\2" ]
      ].each do |unknown_type, reserved|
	encoded_unknown_type = unknown_type.chr
	encoded_unknown_type += reserved
	assert_equal(encoded_unknown_type, WWWsrv::FastCGIRecord::encode_unknown_type(unknown_type, reserved))
      end
    end

    def test_decode_unknown_type
      [ [ 0, "\0" * 7 ],
	[ 1, "\0" * 6 + "\1" ],
	[ 2, "\0" * 6 + "\2" ]
      ].each do |unknown_type, reserved|
	encoded_unknown_type = unknown_type.chr
	encoded_unknown_type += reserved
	decoded_unknown_type = WWWsrv::FastCGIRecord::decode_unknown_type(encoded_unknown_type)
	assert_equal(2, decoded_unknown_type.length)
	assert_equal(unknown_type, decoded_unknown_type[0])
	assert_equal(reserved, decoded_unknown_type[1])
      end
    end
  end

  class TestFastCGIResponder < RUNIT::TestCase
    include DocumentUtils
    include WWWsrv::FastCGIconst

    def setup
      @fcgi_stream = PseudoIO::new
      @responder = WWWsrv::FastCGIResponder::new{ @fcgi_stream }
      @req_id = 1

      def @fcgi_stream.shutdown(how=2)
	case (how)
	when 0, 1, 2
	  # right.
	else
	  raise "invalid parameter of shutdown for socket: #{how.inspect}"
	end
	nil
      end
    end

    def test_respond_get
      message = "Hello world.\n"
      record = WWWsrv::FastCGIRecord::new
      record.record_type = FCGI_STDOUT
      record.request_id = @req_id
      record.content = "Content-Type: text/plain\r\n"
      record.content += "Content-Length: #{message.length}\r\n"
      record.content += "\r\n"
      record.content += message
      record.send_to(@fcgi_stream)
      record.content = nil
      record.send_to(@fcgi_stream)

      record.clear
      record.record_type = FCGI_STDERR
      record.request_id = @req_id
      record.content = nil
      record.send_to(@fcgi_stream)

      record.clear
      record.record_type = FCGI_END_REQUEST
      record.request_id = @req_id
      record.content = WWWsrv::FastCGIRecord::encode_end_request(0, FCGI_REQUEST_COMPLETE)
      record.send_to(@fcgi_stream)

      srv_opt, log_output = make_srv_opt
      request, body_input = make_request('GET', '/fastcgi/hello/world')
      response = make_response
      respondent = try_to_respond(@responder, srv_opt, '/fastcgi', request, response)
      assert_equal(:success, respondent[:doc_status])
      assert_equal(200, respondent[:response].status)
      assert_equal('text/plain', respondent[:response].header.content_type)
      assert_equal(message.length.to_s, respondent[:response].header.content_length)
      assert_equal(message, respondent[:body])

      record.clear
      record.receive_from(@fcgi_stream)
      assert_equal(FCGI_BEGIN_REQUEST, record.record_type)
      assert_equal(@req_id, record.request_id)
      assert_not_nil(record.content)
      assert_equal(0, record.reserved)
      assert_equal(FCGI_VERSION, record.version)
      role, flags, reserved = WWWsrv::FastCGIRecord::decode_begin_request(record.content)
      assert_equal(FCGI_RESPONDER, role)
      assert_equal(@responder.keep_conn ? FCGI_KEEP_CONN : 0, flags & FCGI_KEEP_CONN)
      assert_equal("\0" * 5, reserved)

      encoded_name_value_pairs = ''
      loop do
	record.clear
	record.receive_from(@fcgi_stream)
	assert_equal(FCGI_PARAMS, record.record_type)
	assert_equal(@req_id, record.request_id)
	record.content or break
	assert_equal(0, record.reserved)
	assert_equal(FCGI_VERSION, record.version)
	encoded_name_value_pairs.concat(record.content)
      end

      fcgi_var = WWWsrv::FastCGIRecord::decode_name_value_pairs(encoded_name_value_pairs)
      assert_equal('CGI/1.1', fcgi_var['GATEWAY_INTERFACE'])
      assert_equal('HTTP/1.1', fcgi_var['SERVER_PROTOCOL'])
      assert_equal('GET', fcgi_var['REQUEST_METHOD'])
      assert_equal('/fastcgi', fcgi_var['SCRIPT_NAME'])
      assert_equal('/hello/world', fcgi_var['PATH_INFO'])

      record.clear
      record.receive_from(@fcgi_stream)
      assert_equal(FCGI_STDIN, record.record_type)
      assert_equal(@req_id, record.request_id)
      assert_nil(record.content)
      assert_equal(0, record.reserved)
      assert_equal(FCGI_VERSION, record.version)

      assert_nil(@fcgi_stream.data)
    end

    def test_respond_post
      message = "Hello world.\n"
      record = WWWsrv::FastCGIRecord::new
      record.record_type = FCGI_STDOUT
      record.request_id = @req_id
      record.content = "Content-Type: text/plain\r\n"
      record.content += "Content-Length: #{message.length}\r\n"
      record.content += "\r\n"
      record.content += message
      record.send_to(@fcgi_stream)
      record.content = nil
      record.send_to(@fcgi_stream)

      record.clear
      record.record_type = FCGI_STDERR
      record.request_id = @req_id
      record.content = nil
      record.send_to(@fcgi_stream)

      record.clear
      record.record_type = FCGI_END_REQUEST
      record.request_id = @req_id
      record.content = WWWsrv::FastCGIRecord::encode_end_request(0, FCGI_REQUEST_COMPLETE)
      record.send_to(@fcgi_stream)

      post_message = "This is a pen."
      srv_opt, log_output = make_srv_opt
      request, body_input = make_request('POST', '/fastcgi/hello/world', :body => post_message)
      request.header.content_type = "text/plain"
      request.header.content_length = post_message.length.to_s
      response = make_response
      respondent = try_to_respond(@responder, srv_opt, '/fastcgi', request, response)
      assert_equal(:success, respondent[:doc_status])
      assert_equal(200, respondent[:response].status)
      assert_equal('text/plain', respondent[:response].header.content_type)
      assert_equal(message.length.to_s, respondent[:response].header.content_length)
      assert_equal(message, respondent[:body])

      record.clear
      record.receive_from(@fcgi_stream)
      assert_equal(FCGI_BEGIN_REQUEST, record.record_type)
      assert_equal(1, record.request_id)
      assert_not_nil(record.content)
      assert_equal(0, record.reserved)
      assert_equal(FCGI_VERSION, record.version)
      role, flags, reserved = WWWsrv::FastCGIRecord::decode_begin_request(record.content)
      assert_equal(FCGI_RESPONDER, role)
      assert_equal(@responder.keep_conn ? FCGI_KEEP_CONN : 0, flags & FCGI_KEEP_CONN)
      assert_equal("\0" * 5, reserved)

      encoded_name_value_pairs = ''
      loop do
	record.clear
	record.receive_from(@fcgi_stream)
	assert_equal(FCGI_PARAMS, record.record_type)
	assert_equal(1, record.request_id)
	assert_equal(0, record.reserved)
	assert_equal(FCGI_VERSION, record.version)
	record.content or break
	encoded_name_value_pairs.concat(record.content)
      end

      fcgi_var = WWWsrv::FastCGIRecord::decode_name_value_pairs(encoded_name_value_pairs)
      assert_equal('CGI/1.1', fcgi_var['GATEWAY_INTERFACE'])
      assert_equal('HTTP/1.1', fcgi_var['SERVER_PROTOCOL'])
      assert_equal('POST', fcgi_var['REQUEST_METHOD'])
      assert_equal('/fastcgi', fcgi_var['SCRIPT_NAME'])
      assert_equal('/hello/world', fcgi_var['PATH_INFO'])
      assert_equal('text/plain', fcgi_var['CONTENT_TYPE'])
      assert_equal(post_message.length.to_s, fcgi_var['CONTENT_LENGTH'])

      stdin_message = ''
      loop do
	record.clear
	record.receive_from(@fcgi_stream)
	assert_equal(FCGI_STDIN, record.record_type)
	assert_equal(1, record.request_id)
	assert_equal(0, record.reserved)
	assert_equal(FCGI_VERSION, record.version)
	record.content or break
	stdin_message.concat(record.content)
      end

      assert_equal(post_message, stdin_message)
      assert_nil(@fcgi_stream.data)
    end
  end

  class TestFastCGIAuthorizer < RUNIT::TestCase
    include DocumentUtils
    include WWWsrv::FastCGIconst

    def setup
      @fcgi_stream = PseudoIO::new
      @msg_type = 'text/plain'
      @message = "Hello world.\n"
      @src_doc = WWWsrv::Page::new(@message, @msg_type)
      @authorizer = WWWsrv::FastCGIAuthorizer::new(@src_doc) { @fcgi_stream }
      @req_id = 1

      def @fcgi_stream.shutdown(how=2)
	case (how)
	when 0, 1, 2
	  # right.
	else
	  raise "invalid parameter of shutdown for socket: #{how.inspect}"
	end
	nil
      end
    end

    def test_respond_allow
      message = "Allow\n"
      record = WWWsrv::FastCGIRecord::new
      record.record_type = FCGI_STDOUT
      record.request_id = @req_id
      record.content = "Status: 200 OK\r\n"
      record.content += "Content-Type: text/plain\r\n"
      record.content += "Content-Length: #{message.length}\r\n"
      record.content += "Variable-AUTH_METHOD: debug\r\n"
      record.content += "\r\n"
      record.content += message
      record.send_to(@fcgi_stream)
      record.content = nil
      record.send_to(@fcgi_stream)

      record.clear
      record.record_type = FCGI_STDERR
      record.request_id = @req_id
      record.content = nil
      record.send_to(@fcgi_stream)

      record.clear
      record.record_type = FCGI_END_REQUEST
      record.request_id = @req_id
      record.content = WWWsrv::FastCGIRecord::encode_end_request(0, FCGI_REQUEST_COMPLETE)
      record.send_to(@fcgi_stream)

      srv_opt, log_output = make_srv_opt
      request, body_input =make_request('GET', '/fastcgi/hello/world')
      response = make_response
      respondent = try_to_respond(@authorizer, srv_opt, '/fastcgi', request, response)
      assert_equal('debug', request[:FCGI_AUTH_METHOD])
      assert_equal(:success, respondent[:doc_status])
      assert_equal(200, respondent[:response].status)
      assert_equal(@msg_type, respondent[:response].header.content_type)
      assert_equal(@message.length.to_s, respondent[:response].header.content_length)
      assert_equal(@message, respondent[:body])

      record.clear
      record.receive_from(@fcgi_stream)
      assert_equal(FCGI_BEGIN_REQUEST, record.record_type)
      assert_equal(@req_id, record.request_id)
      assert_not_nil(record.content)
      assert_equal(0, record.reserved)
      assert_equal(FCGI_VERSION, record.version)
      role, flags, reserved = WWWsrv::FastCGIRecord::decode_begin_request(record.content)
      assert_equal(FCGI_AUTHORIZER, role)
      assert_equal(@authorizer.keep_conn ? FCGI_KEEP_CONN : 0, flags & FCGI_KEEP_CONN)
      assert_equal("\0" * 5, reserved)

      encoded_name_value_pairs = ''
      loop do
	record.clear
	record.receive_from(@fcgi_stream)
	assert_equal(FCGI_PARAMS, record.record_type)
	assert_equal(@req_id, record.request_id)
	record.content or break
	assert_equal(0, record.reserved)
	assert_equal(FCGI_VERSION, record.version)
	encoded_name_value_pairs.concat(record.content)
      end

      fcgi_var = WWWsrv::FastCGIRecord::decode_name_value_pairs(encoded_name_value_pairs)
      assert_equal('CGI/1.1', fcgi_var['GATEWAY_INTERFACE'])
      assert_equal('HTTP/1.1', fcgi_var['SERVER_PROTOCOL'])
      assert_equal('GET', fcgi_var['REQUEST_METHOD'])
      assert_nil(fcgi_var['CONTENT_LENGTH'])
      assert_nil(fcgi_var['SCRIPT_NAME'])
      assert_nil(fcgi_var['PATH_INFO'])

      assert_nil(@fcgi_stream.data)
    end

    def test_respond_deny
      message = "Deny\n"
      record = WWWsrv::FastCGIRecord::new
      record.record_type = FCGI_STDOUT
      record.request_id = @req_id
      record.content = "Status: 403 Forbidden\r\n"
      record.content += "Content-Type: text/plain\r\n"
      record.content += "Content-Length: #{message.length}\r\n"
      record.content += "Variable-AUTH_METHOD: debug\r\n"
      record.content += "\r\n"
      record.content += message
      record.send_to(@fcgi_stream)
      record.content = nil
      record.send_to(@fcgi_stream)

      record.clear
      record.record_type = FCGI_STDERR
      record.request_id = @req_id
      record.content = nil
      record.send_to(@fcgi_stream)

      record.clear
      record.record_type = FCGI_END_REQUEST
      record.request_id = @req_id
      record.content = WWWsrv::FastCGIRecord::encode_end_request(0, FCGI_REQUEST_COMPLETE)
      record.send_to(@fcgi_stream)

      srv_opt, log_output = make_srv_opt
      request, body_input =make_request('GET', '/fastcgi/hello/world')
      response = make_response
      respondent = try_to_respond(@authorizer, srv_opt, '/fastcgi', request, response)
      assert_nil(request[:FCGI_AUTH_METHOD])
      assert_equal(:success, respondent[:doc_status])
      assert_equal(403, respondent[:response].status)
      assert_equal('text/plain', respondent[:response].header.content_type)
      assert_equal(message.length.to_s, respondent[:response].header.content_length)
      assert_equal(message, respondent[:body])

      record.clear
      record.receive_from(@fcgi_stream)
      assert_equal(FCGI_BEGIN_REQUEST, record.record_type)
      assert_equal(@req_id, record.request_id)
      assert_not_nil(record.content)
      assert_equal(0, record.reserved)
      assert_equal(FCGI_VERSION, record.version)
      role, flags, reserved = WWWsrv::FastCGIRecord::decode_begin_request(record.content)
      assert_equal(FCGI_AUTHORIZER, role)
      assert_equal(@authorizer.keep_conn ? FCGI_KEEP_CONN : 0, flags & FCGI_KEEP_CONN)
      assert_equal("\0" * 5, reserved)

      encoded_name_value_pairs = ''
      loop do
	record.clear
	record.receive_from(@fcgi_stream)
	assert_equal(FCGI_PARAMS, record.record_type)
	assert_equal(@req_id, record.request_id)
	record.content or break
	assert_equal(0, record.reserved)
	assert_equal(FCGI_VERSION, record.version)
	encoded_name_value_pairs.concat(record.content)
      end

      fcgi_var = WWWsrv::FastCGIRecord::decode_name_value_pairs(encoded_name_value_pairs)
      assert_equal('CGI/1.1', fcgi_var['GATEWAY_INTERFACE'])
      assert_equal('HTTP/1.1', fcgi_var['SERVER_PROTOCOL'])
      assert_equal('GET', fcgi_var['REQUEST_METHOD'])
      assert_nil(fcgi_var['CONTENT_LENGTH'])
      assert_nil(fcgi_var['SCRIPT_NAME'])
      assert_nil(fcgi_var['PATH_INFO'])

      assert_nil(@fcgi_stream.data)
    end
  end

  class TestFastCGIFilter < RUNIT::TestCase
    include DocumentUtils
    include WWWsrv::FastCGIconst

    def setup
      @fcgi_stream = PseudoIO::new
      @msg_type = 'text/plain'
      @message = "Hello world.\n"
      @mtime = Time::now
      @src_doc = WWWsrv::Page::new(@message, @msg_type, @mtime)
      @filter = WWWsrv::FastCGIFilter::new(@src_doc) { @fcgi_stream }
      @req_id = 1

      def @fcgi_stream.shutdown(how=2)
	case (how)
	when 0, 1, 2
	  # right.
	else
	  raise "invalid parameter of shutdown for socket: #{how.inspect}"
	end
	nil
      end
    end

    def test_respond
      message = "This message is filtered.\n"
      record = WWWsrv::FastCGIRecord::new
      record.record_type = FCGI_STDOUT
      record.request_id = @req_id
      record.content = "Status: 200 OK\r\n"
      record.content += "Content-Type: text/plain\r\n"
      record.content += "Content-Length: #{message.length}\r\n"
      record.content += "\r\n"
      record.content += message
      record.send_to(@fcgi_stream)
      record.content = nil
      record.send_to(@fcgi_stream)

      record.clear
      record.record_type = FCGI_STDERR
      record.request_id = @req_id
      record.content = nil
      record.send_to(@fcgi_stream)

      record.clear
      record.record_type = FCGI_END_REQUEST
      record.request_id = @req_id
      record.content = WWWsrv::FastCGIRecord::encode_end_request(0, FCGI_REQUEST_COMPLETE)
      record.send_to(@fcgi_stream)

      srv_opt, log_output = make_srv_opt
      request, body_input = make_request('GET', '/fastcgi/hello/world')
      response = make_response
      respondent = try_to_respond(@filter, srv_opt, '/fastcgi', request, response)
      assert_equal(:success, respondent[:doc_status])
      assert_equal(200, respondent[:response].status)
      assert_equal('text/plain', respondent[:response].header.content_type)
      assert_equal(message.length.to_s, respondent[:response].header.content_length)
      assert_equal(message, respondent[:body])

      record.clear
      record.receive_from(@fcgi_stream)
      assert_equal(FCGI_BEGIN_REQUEST, record.record_type)
      assert_equal(@req_id, record.request_id)
      assert_not_nil(record.content)
      assert_equal(0, record.reserved)
      assert_equal(FCGI_VERSION, record.version)
      role, flags, reserved = WWWsrv::FastCGIRecord::decode_begin_request(record.content)
      assert_equal(FCGI_FILTER, role)
      assert_equal(@filter.keep_conn ? FCGI_KEEP_CONN : 0, flags & FCGI_KEEP_CONN)
      assert_equal("\0" * 5, reserved)

      encoded_name_value_pairs = ''
      loop do
	record.clear
	record.receive_from(@fcgi_stream)
	assert_equal(FCGI_PARAMS, record.record_type)
	assert_equal(@req_id, record.request_id)
	record.content or break
	assert_equal(0, record.reserved)
	assert_equal(FCGI_VERSION, record.version)
	encoded_name_value_pairs.concat(record.content)
      end

      fcgi_var = WWWsrv::FastCGIRecord::decode_name_value_pairs(encoded_name_value_pairs)
      assert_equal('CGI/1.1', fcgi_var['GATEWAY_INTERFACE'])
      assert_equal('HTTP/1.1', fcgi_var['SERVER_PROTOCOL'])
      assert_equal('GET', fcgi_var['REQUEST_METHOD'])
      assert_equal('/fastcgi', fcgi_var['SCRIPT_NAME'])
      assert_equal('/hello/world', fcgi_var['PATH_INFO'])
      assert_equal(@mtime.to_i.to_s, fcgi_var['FCGI_DATA_LAST_MOD'])
      assert_equal(@message.length.to_s, fcgi_var['FCGI_DATA_LENGTH'])

      stdin_message = ''; stdin_closed = false
      data_message = ''; data_closed = false
      begin
	record.clear
	record.receive_from(@fcgi_stream)
	case (record.record_type)
	when FCGI_STDIN
	  assert_equal(1, record.request_id)
	  assert_equal(0, record.reserved)
	  assert_equal(FCGI_VERSION, record.version)
	  if (record.content) then
	    stdin_message.concat(record.content)
	  else
	    stdin_closed = true
	  end
	when FCGI_DATA
	  assert_equal(1, record.request_id)
	  assert_equal(0, record.reserved)
	  assert_equal(FCGI_VERSION, record.version)
	  if (record.content) then
	    data_message.concat(record.content)
	  else
	    data_closed = true
	  end
	else
	  assert_fail("Unknown record type: #{record.inspect}")
	end
      end until (stdin_closed && data_closed)

      assert(stdin_message.empty?)
      assert_equal(@message, data_message)
      assert_nil(@fcgi_stream.data)
    end
  end
end

if ($0 == __FILE__) then
  if (ARGV.empty?) then
    TestWWWsrv::constants.map{ |name|
      TestWWWsrv::const_get(name)
    }.select{ |const|
      (const.kind_of? Class) && (const < RUNIT::TestCase)
    }.each do |test_case|
      RUNIT::CUI::TestRunner::run(test_case::suite)
    end
  else
    for class_name in ARGV
      test_case = TestWWWsrv.const_get("Test#{class_name}")
      RUNIT::CUI::TestRunner.run(test_case.suite)
    end
  end
end
