#!/usr/local/bin/ruby
# $Id: test_filter.rb,v 1.13.2.1 2003/01/05 13:06:31 toki Exp $
# -
# Test of document filter for WWW server

require 'docutils'
require 'pseudo_io'
require 'wwwsrv/srvopt'
require 'wwwsrv/logger'
require 'wwwsrv/request'
require 'wwwsrv/response'
require 'wwwsrv/filter'
require 'runit/testcase'
require 'runit/cui/testrunner'

module TestWWWsrv
  class TestLineFilterDocument < RUNIT::TestCase
    include DocumentUtils

    def setup
      @name = 'index.html'
      @content = <<-EOF
<html>
<head>
<title>Test</title>
</head>
<body>
<h1>Test</h1>
<p>Hello world.</p>
</body>
</html>
      EOF

      @root = WWWsrv::DocumentFolder::new
      @root.mount(WWWsrv::Page::new(@content), '/', :page => @name)
    end

    def test_through
      doc_filter = WWWsrv::LineFilterDocument::new(@root) { |line| line }
      srv_opt, log_output = make_srv_opt
      request, body_input = make_request('GET', '/index.html')
      response = make_response
      respondent = try_to_respond(doc_filter, srv_opt, '', request, response)
      assert_equal(:success, respondent[:doc_status])
      assert_nil(respondent[:response].header.content_length)
      assert_equal(@content, respondent[:body])
      assert(! respondent[:body_alist].empty?)
    end

    def test_modify
      doc_filter = WWWsrv::LineFilterDocument::new(@root) { |line|
	line.gsub(/Hello/, 'Bye bye')
      }
      srv_opt, log_output = make_srv_opt
      request, body_input = make_request('GET', '/index.html')
      response = make_response
      respondent = try_to_respond(doc_filter, srv_opt, '', request, response)
      assert_equal(:success, respondent[:doc_status])
      assert_nil(respondent[:response].header.content_length)
      assert_equal(@content.gsub(/Hello/, 'Bye bye'), respondent[:body])
      assert(! respondent[:body_alist].empty?)
    end

    def test_no_new_line
      @content = <<-EOF
<html>
<head>
<title>Test</title>
</head>
<body>
<h1>Test</h1>
      EOF
      @content.concat("<p>Hello world.</p>\n" * 1024)
      @content.concat(<<-EOF)
</body>
</html>
      EOF
      @content.chomp!
      @root.mount(WWWsrv::Page::new(@content), '/', :page => @name)

      doc_filter = WWWsrv::LineFilterDocument::new(@root) { |line|
	line.gsub(/Hello/, 'Bye bye')
      }
      srv_opt, log_output = make_srv_opt
      request, body_input = make_request('GET', '/index.html')
      response = make_response
      respondent = try_to_respond(doc_filter, srv_opt, '', request, response)
      assert_equal(:success, respondent[:doc_status])
      assert_nil(respondent[:response].header.content_length)
      assert_equal(@content.gsub(/Hello/, 'Bye bye'), respondent[:body])
      assert(! respondent[:body_alist].empty?)
    end
  end
end

if ($0 == __FILE__) then
  if (ARGV.empty?) then
    TestWWWsrv::constants.map{ |name|
      TestWWWsrv::const_get(name)
    }.select{ |const|
      (const.kind_of? Class) && (const < RUNIT::TestCase)
    }.each do |test_case|
      RUNIT::CUI::TestRunner::run(test_case::suite)
    end
  else
    for class_name in ARGV
      test_case = TestWWWsrv.const_get("Test#{class_name}")
      RUNIT::CUI::TestRunner.run(test_case.suite)
    end
  end
end
