#!/usr/local/bin/ruby
# $Id: test_form.rb,v 1.6.2.2 2003/04/11 14:08:56 toki Exp $
# -
# Test of WWWsrv::FormData module

require 'pseudo_io'
require 'wwwsrv/htmlconv'
require 'wwwsrv/form'
require 'runit/testcase'
require 'runit/cui/testrunner'

module TestWWWsrv
  class TestFormData < RUNIT::TestCase
    def setup
      @form_data = WWWsrv::FormData.new

      head = WWWsrv::Header.build('Content-Disposition' => 'form-data; name="submit"')
      @form_data.add_form_data('submit', head)
      @form_data.add_form_data_body('submit', head, 'OK')
      # -
      head = WWWsrv::Header.build('Content-Disposition' => 'form-data; name="date"')
      @form_data.add_form_data('date', head)
      @form_data.add_form_data_body('date', head, '1975-11-19')
      # -
      head = WWWsrv::Header.build(
	'Content-Disposition' => 'form-data; name="text"; filename="hello.txt"',
	'Content-Type' => 'text/html'
      )
      @form_data.add_form_data('text', head)
      @form_data.add_form_data_body('text', head, "Hello world.\n", 'hello.txt')
      # -
      head = WWWsrv::Header.build(
	'Content-Disposition' => 'form-data; name="files"',
	'Content-Type' => 'multipart/mixed; boundary="Bb:C04y"'
      )
      @form_data.add_form_data('files', head)
      # --
      head = WWWsrv::Header.build(
	'Content-Disposition' => 'file; filename="hello.html"',
	'Content-Type' => 'text/html'
      )
      body = "<html>\n"
      body += "<body><p>Hello world.</p></body>\n"
      body += "</html>\n"
      @form_data.add_form_data_body('files', head, body, 'hello.html')
      # --
      head = WWWsrv::Header.build(
	'Content-Disposition' => 'file; filename="hello.css"',
	'Content-Type' => 'text/css'
      )
      body = "p { font-weight: bold; font-size: large }\n"
      @form_data.add_form_data_body('files', head, body, 'hello.css')
      # -
      head = WWWsrv::Header.build(
	'Content-Disposition' => 'form-data; name="option"',
	'X-Position' => '1'
      )
      @form_data.add_form_data('option', head)
      @form_data.add_form_data_body('option', head, '1')
      # -
      head = WWWsrv::Header.build(
	'Content-Disposition' => 'form-data; name="option"',
	'X-Position' => '2'
      )
      @form_data.add_form_data('option', head)
      @form_data.add_form_data_body('option', head, '2')
    end

    def test_square_brackets
      assert_equal('OK', @form_data['submit'])
      assert_equal('OK', @form_data['submit', 0])
      assert_equal('OK', @form_data['submit', 0, 0])
      assert_nil(@form_data['submit', 0, 1])
      assert_nil(@form_data['submit', 1])
      assert_equal([ 'OK' ], @form_data['submit', 0..-1])
      assert_equal([ [ 'OK' ] ], @form_data['submit', 0..-1, 0..-1])

      assert_equal('1975-11-19', @form_data['date'])
      assert_equal('1975-11-19', @form_data['date', 0])
      assert_equal('1975-11-19', @form_data['date', 0, 0])
      assert_nil(@form_data['date', 0, 1])
      assert_nil(@form_data['date', 1])
      assert_equal([ '1975-11-19' ], @form_data['date', 0..-1])
      assert_equal([ [ '1975-11-19' ] ], @form_data['date', 0..-1, 0..-1])

      assert_equal("Hello world.\n", @form_data['text'])
      assert_equal("Hello world.\n", @form_data['text', 0])
      assert_equal("Hello world.\n", @form_data['text', 0, 0])
      assert_nil(@form_data['text', 0, 1])
      assert_nil(@form_data['text', 1])
      assert_equal([ "Hello world.\n" ], @form_data['text', 0..-1])
      assert_equal([ [ "Hello world.\n" ] ], @form_data['text', 0..-1, 0..-1])

      body1 = "<html>\n"
      body1 += "<body><p>Hello world.</p></body>\n"
      body1 += "</html>\n"
      body2 = "p { font-weight: bold; font-size: large }\n"
      assert_equal(body1, @form_data['files'])
      assert_equal(body1, @form_data['files', 0])
      assert_equal(body1, @form_data['files', 0, 0])
      assert_equal(body2, @form_data['files', 0, 1])
      assert_nil(@form_data['files', 0, 2])
      assert_nil(@form_data['files', 1])
      assert_equal([ body1 ], @form_data['files', 0..-1])
      assert_equal([ [ body1, body2 ] ], @form_data['files', 0..-1, 0..-1])

      assert_equal('1', @form_data['option'])
      assert_equal('1', @form_data['option', 0])
      assert_equal('1', @form_data['option', 0, 0])
      assert_nil(@form_data['option', 0, 1])
      assert_equal('2', @form_data['option', 1])
      assert_equal('2', @form_data['option', 1, 0])
      assert_nil(@form_data['option', 1, 1])
      assert_nil(@form_data['option', 2])
      assert_equal([ '1', '2' ], @form_data['option', 0..-1])
      assert_equal([ [ '1' ], [ '2' ] ], @form_data['option', 0..-1, 0..-1])

      assert_nil(@form_data['nothing'])
      assert_nil(@form_data['nothing', 0])
      assert_nil(@form_data['nothing', 0, 0])
      assert_nil(@form_data['nothing', 0..-1])
      assert_nil(@form_data['nothing', 0..-1, 0..-1])
    end

    def test_head
      head = WWWsrv::Header.build('Content-Disposition' => 'form-data; name="submit"')
      assert_equal(head, @form_data.head('submit'))
      assert_equal(head, @form_data.head('submit', 0))
      assert_nil(@form_data.head('submit', 1))
      assert_equal([ head ], @form_data.head('submit', 0..-1))

      head = WWWsrv::Header.build('Content-Disposition' => 'form-data; name="date"')
      assert_equal(head, @form_data.head('date'))
      assert_equal(head, @form_data.head('date', 0))
      assert_nil(@form_data.head('date', 1))
      assert_equal([ head ], @form_data.head('date', 0..-1))

      head = WWWsrv::Header.build(
	'Content-Disposition' => 'form-data; name="text"; filename="hello.txt"',
	'Content-Type' => 'text/html'
      )
      assert_equal(head, @form_data.head('text'))
      assert_equal(head, @form_data.head('text', 0))
      assert_nil(@form_data.head('text', 1))
      assert_equal([ head ], @form_data.head('text', 0..-1))

      head = WWWsrv::Header.build(
	'Content-Disposition' => 'form-data; name="files"',
	'Content-Type' => 'multipart/mixed; boundary="Bb:C04y"'
      )
      assert_equal(head, @form_data.head('files'))
      assert_equal(head, @form_data.head('files', 0))
      assert_nil(@form_data.head('files', 1))
      assert_equal([ head ], @form_data.head('files', 0..-1))

      head1 = WWWsrv::Header.build(
	'Content-Disposition' => 'form-data; name="option"',
	'X-Position' => '1'
      )
      head2 = WWWsrv::Header.build(
	'Content-Disposition' => 'form-data; name="option"',
	'X-Position' => '2'
      )
      assert_equal(head1, @form_data.head('option'))
      assert_equal(head1, @form_data.head('option', 0))
      assert_equal(head2, @form_data.head('option', 1))
      assert_nil(@form_data.head('option', 2))
      assert_equal([ head1, head2 ], @form_data.head('option', 0..-1))

      assert_nil(@form_data.head('nothing'))
      assert_nil(@form_data.head('nothing', 0))
      assert_nil(@form_data.head('nothing', 0..-1))
    end

    def test_body_head
      head = WWWsrv::Header.build('Content-Disposition' => 'form-data; name="submit"')
      assert_equal(head, @form_data.body_head('submit'))
      assert_equal(head, @form_data.body_head('submit', 0))
      assert_equal(head, @form_data.body_head('submit', 0, 0))
      assert_nil(@form_data.body_head('submit', 0, 1))
      assert_nil(@form_data.body_head('submit', 1))
      assert_equal([ head ], @form_data.body_head('submit', 0..-1))
      assert_equal([ [ head ] ], @form_data.body_head('submit', 0..-1, 0..-1))

      head = WWWsrv::Header.build('Content-Disposition' => 'form-data; name="date"')
      assert_equal(head, @form_data.body_head('date'))
      assert_equal(head, @form_data.body_head('date', 0))
      assert_equal(head, @form_data.body_head('date', 0, 0))
      assert_nil(@form_data.body_head('date', 0, 1))
      assert_nil(@form_data.body_head('date', 1))
      assert_equal([ head ], @form_data.body_head('date', 0..-1))
      assert_equal([ [ head ] ], @form_data.body_head('date', 0..-1, 0..-1))

      head = WWWsrv::Header.build(
	'Content-Disposition' => 'form-data; name="text"; filename="hello.txt"',
	'Content-Type' => 'text/html'
      )
      assert_equal(head, @form_data.body_head('text'))
      assert_equal(head, @form_data.body_head('text', 0))
      assert_equal(head, @form_data.body_head('text', 0, 0))
      assert_nil(@form_data.body_head('text', 0, 1))
      assert_nil(@form_data.body_head('text', 1))
      assert_equal([ head ], @form_data.body_head('text', 0..-1))
      assert_equal([ [ head ] ], @form_data.body_head('text', 0..-1, 0..-1))

      head1 = WWWsrv::Header.build(
	'Content-Disposition' => 'file; filename="hello.html"',
	'Content-Type' => 'text/html'
      )
      head2 = WWWsrv::Header.build(
	'Content-Disposition' => 'file; filename="hello.css"',
	'Content-Type' => 'text/css'
      )
      assert_equal(head1, @form_data.body_head('files'))
      assert_equal(head1, @form_data.body_head('files', 0))
      assert_equal(head1, @form_data.body_head('files', 0, 0))
      assert_equal(head2, @form_data.body_head('files', 0, 1))
      assert_nil(@form_data.body_head('files', 0, 2))
      assert_nil(@form_data.body_head('files', 1))
      assert_equal([ head1 ], @form_data.body_head('files', 0..-1))
      assert_equal([ [ head1, head2 ] ], @form_data.body_head('files', 0..-1, 0..-1))

      head1 = WWWsrv::Header.build(
	'Content-Disposition' => 'form-data; name="option"',
	'X-Position' => '1'
      )
      head2 = WWWsrv::Header.build(
	'Content-Disposition' => 'form-data; name="option"',
	'X-Position' => '2'
      )
      assert_equal(head1, @form_data.body_head('option'))
      assert_equal(head1, @form_data.body_head('option', 0))
      assert_equal(head1, @form_data.body_head('option', 0, 0))
      assert_nil(@form_data.body_head('option', 0, 1))
      assert_equal(head2, @form_data.body_head('option', 1))
      assert_equal(head2, @form_data.body_head('option', 1, 0))
      assert_nil(@form_data.body_head('option', 1, 1))
      assert_nil(@form_data.body_head('option', 2))
      assert_equal([ head1, head2 ], @form_data.body_head('option', 0..-1))
      assert_equal([ [ head1 ], [ head2 ] ], @form_data.body_head('option', 0..-1, 0..-1))

      assert_nil(@form_data.body_head('nothing'))
      assert_nil(@form_data.body_head('nothing', 0))
      assert_nil(@form_data.body_head('nothing', 0, 0))
      assert_nil(@form_data.body_head('nothing', 0..-1))
      assert_nil(@form_data.body_head('nothing', 0..-1, 0..-1))
    end

    def test_filename
      assert_nil(@form_data.filename('submit'))
      assert_nil(@form_data.filename('submit', 0))
      assert_nil(@form_data.filename('submit', 0, 0))
      assert_nil(@form_data.filename('submit', 0, 1))
      assert_nil(@form_data.filename('submit', 1))
      assert_equal([ nil ], @form_data.filename('submit', 0..-1))
      assert_equal([ [ nil ] ], @form_data.filename('submit', 0..-1, 0..-1))

      assert_nil(@form_data.filename('date'))
      assert_nil(@form_data.filename('date', 0))
      assert_nil(@form_data.filename('date', 0, 0))
      assert_nil(@form_data.filename('date', 0, 1))
      assert_nil(@form_data.filename('date', 1))
      assert_equal([ nil ], @form_data.filename('date', 0..-1))
      assert_equal([ [ nil ] ], @form_data.filename('date', 0..-1, 0..-1))

      assert_equal('hello.txt', @form_data.filename('text'))
      assert_equal('hello.txt', @form_data.filename('text', 0))
      assert_equal('hello.txt', @form_data.filename('text', 0, 0))
      assert_nil(@form_data.filename('text', 0, 1))
      assert_nil(@form_data.filename('text', 1))
      assert_equal([ 'hello.txt' ], @form_data.filename('text', 0..-1))
      assert_equal([ [ 'hello.txt' ] ], @form_data.filename('text', 0..-1, 0..-1))

      assert_equal('hello.html', @form_data.filename('files'))
      assert_equal('hello.html', @form_data.filename('files', 0))
      assert_equal('hello.html', @form_data.filename('files', 0, 0))
      assert_equal('hello.css', @form_data.filename('files', 0, 1))
      assert_nil(@form_data.filename('files', 0, 2))
      assert_nil(@form_data.filename('files', 1))
      assert_equal([ 'hello.html' ], @form_data.filename('files', 0..-1))
      assert_equal([ [ 'hello.html', 'hello.css' ] ], @form_data.filename('files', 0..-1, 0..-1))

      assert_nil(@form_data.filename('option'))
      assert_nil(@form_data.filename('option', 0))
      assert_nil(@form_data.filename('option', 0, 0))
      assert_nil(@form_data.filename('option', 0, 1))
      assert_nil(@form_data.filename('option', 1))
      assert_nil(@form_data.filename('option', 1, 0))
      assert_nil(@form_data.filename('option', 1, 1))
      assert_nil(@form_data.filename('option', 2))
      assert_equal([ nil, nil ], @form_data.filename('option', 0..-1))
      assert_equal([ [ nil ], [ nil ] ], @form_data.filename('option', 0..-1, 0..-1))

      assert_nil(@form_data.filename('nothing'))
      assert_nil(@form_data.filename('nothing', 0))
      assert_nil(@form_data.filename('nothing', 0, 0))
      assert_nil(@form_data.filename('nothing', 0..-1))
      assert_nil(@form_data.filename('nothing', 0..-1, 0..-1))
    end

    def test_count_name
      assert_equal(5, @form_data.count_name)
      form_data = WWWsrv::FormData.new
      assert_equal(0, form_data.count_name)
    end

    def test_count_form_data
      assert_equal(1, @form_data.count_form_data('submit'))
      assert_equal(1, @form_data.count_form_data('date'))
      assert_equal(1, @form_data.count_form_data('text'))
      assert_equal(1, @form_data.count_form_data('files'))
      assert_equal(2, @form_data.count_form_data('option'))
      assert_equal(0, @form_data.count_form_data('nothing'))
    end

    def test_count_form_data_body
      assert_equal(1, @form_data.count_form_data_body('submit'))
      assert_equal(1, @form_data.count_form_data_body('submit', 0))
      assert_equal(0, @form_data.count_form_data_body('submit', 1))

      assert_equal(1, @form_data.count_form_data_body('date'))
      assert_equal(1, @form_data.count_form_data_body('date', 0))
      assert_equal(0, @form_data.count_form_data_body('date', 1))

      assert_equal(1, @form_data.count_form_data_body('text'))
      assert_equal(1, @form_data.count_form_data_body('text', 0))
      assert_equal(0, @form_data.count_form_data_body('text', 1))

      assert_equal(2, @form_data.count_form_data_body('files'))
      assert_equal(2, @form_data.count_form_data_body('files', 0))
      assert_equal(0, @form_data.count_form_data_body('files', 1))

      assert_equal(1, @form_data.count_form_data_body('option'))
      assert_equal(1, @form_data.count_form_data_body('option', 0))
      assert_equal(1, @form_data.count_form_data_body('option', 1))
      assert_equal(0, @form_data.count_form_data_body('option', 2))

      assert_equal(0, @form_data.count_form_data_body('nothing'))
      assert_equal(0, @form_data.count_form_data_body('nothing', 0))
    end

    def test_names
      assert_equal(%w[ submit date text files option ].sort, @form_data.names.sort)
      form_data = WWWsrv::FormData.new
      assert(form_data.empty?)
    end

    def test_empty?
      assert(! @form_data.empty?)
      form_data = WWWsrv::FormData.new
      assert(form_data.empty?)
    end

    def test_include?
      assert((@form_data.include? 'submit'))
      assert((@form_data.include? 'date'))
      assert((@form_data.include? 'text'))
      assert((@form_data.include? 'files'))
      assert((@form_data.include? 'option'))
      assert(! (@form_data.include? 'nothing'))
    end

    def test_each
      form_data_alist = [
	[ 'submit', 'OK' ],
	[ 'date', '1975-11-19' ],
	[ 'text', "Hello world.\n" ],
	[ 'files',
	  [ "<html>\n" +
	    "<body><p>Hello world.</p></body>\n" +
	    "</html>\n",
	    "p { font-weight: bold; font-size: large }\n"
	  ]
	],
	[ 'option', '1' ],
	[ 'option', '2' ]
      ]

      form_each_alist = Array.new
      @form_data.each do |name, body|
	form_each_alist.push([ name, body ])
      end

      assert_equal(form_data_alist.sort, form_each_alist.sort)
    end

    def test_parse_query
      form_data_parsed_alist = [
	[ :form_data_name, 'user' ],
	[ :form_data_head, WWWsrv::Header.build('Content-Disposition' => 'form-data; name=user') ],
	[ :form_data_body, 'toki' ],
	[ :form_data_end, nil ],
	# -
	[ :form_data_name, 'path' ],
	[ :form_data_head, WWWsrv::Header.build('Content-Disposition' => 'form-data; name=path') ],
	[ :form_data_body, '/~toki' ],
	[ :form_data_end, nil ],
	# -
	[ :form_data_name, 'option' ],
	[ :form_data_head, WWWsrv::Header.build('Content-Disposition' => 'form-data; name=option') ],
	[ :form_data_body, '1' ],
	[ :form_data_end, nil ],
	# -
	[ :form_data_name, 'option' ],
	[ :form_data_head, WWWsrv::Header.build('Content-Disposition' => 'form-data; name=option') ],
	[ :form_data_body, '2' ],
	[ :form_data_end, nil ],
	# -
	[ :form_data_name, 'submit' ],
	[ :form_data_head, WWWsrv::Header.build('Content-Disposition' => 'form-data; name=submit') ],
	[ :form_data_end, nil ]
      ]

      form_data_scan_alist = Array.new
      WWWsrv::FormData.parse_query('user=toki&path=%2F%7Etoki&option=1&option=2&submit') do |tag, value|
	form_data_scan_alist.push([ tag, value ])
      end

      assert_equal(form_data_parsed_alist, form_data_scan_alist)
    end

    def test_scan_query
      padding = '_' * 1024*256
      url_encoded_form_data_string = "user=toki&path=%2F%7Etoki&option=1&option=2&submit&padding=#{padding}"
      form_data_parsed_alist = [
	[ :form_data_name, 'user' ],
	[ :form_data_head, WWWsrv::Header.build('Content-Disposition' => 'form-data; name=user') ],
	[ :form_data_body, 'toki' ],
	[ :form_data_end, nil ],
	# -
	[ :form_data_name, 'path' ],
	[ :form_data_head, WWWsrv::Header.build('Content-Disposition' => 'form-data; name=path') ],
	[ :form_data_body, '/~toki' ],
	[ :form_data_end, nil ],
	# -
	[ :form_data_name, 'option' ],
	[ :form_data_head, WWWsrv::Header.build('Content-Disposition' => 'form-data; name=option') ],
	[ :form_data_body, '1' ],
	[ :form_data_end, nil ],
	# -
	[ :form_data_name, 'option' ],
	[ :form_data_head, WWWsrv::Header.build('Content-Disposition' => 'form-data; name=option') ],
	[ :form_data_body, '2' ],
	[ :form_data_end, nil ],
	# -
	[ :form_data_name, 'submit' ],
	[ :form_data_head, WWWsrv::Header.build('Content-Disposition' => 'form-data; name=submit') ],
	[ :form_data_end, nil ],
	# -
	[ :form_data_name, 'padding' ],
	[ :form_data_head, WWWsrv::Header.build('Content-Disposition' => 'form-data; name=padding') ],
	[ :form_data_body, padding ],
	[ :form_data_end, nil ]
      ]

      input = PseudoIO.new
      input.write(url_encoded_form_data_string)
      form_data_scan_alist = Array.new
      WWWsrv::FormData.scan_query(input) do |tag, value|
	form_data_scan_alist.push([ tag, value ])
      end

      assert_equal(form_data_parsed_alist, form_data_scan_alist)
      assert_nil(input.data)
    end

    def test_scan_form_date
      content_type = 'multipart/form-data; boundary=AaB03x'

      input = PseudoIO.new
      input.print "--AaB03x\r\n"
      input.print 'Content-Disposition: form-data; name="submit"' + "\r\n"
      input.print "\r\n"
      input.print "OK\r\n"
      input.print "--AaB03x\r\n"
      input.print 'Content-Disposition: form-data; name="date"' + "\r\n"
      input.print "\r\n"
      input.print "1975-11-19\r\n"
      input.print "--AaB03x\r\n"
      input.print 'Content-Disposition: form-data; name="text"; filename="hello.txt"' + "\r\n"
      input.print "Content-Type: text/plain\r\n"
      input.print "\r\n"
      input.print "Hello world.\n"
      input.print "\r\n"
      input.print "--AaB03x\r\n"
      input.print 'Content-Disposition: form-data; name="html"' + "\r\n"
      input.print 'Content-Type: multipart/mixed; boundary="Bb:C04y"' + "\r\n"
      input.print "\r\n"
      input.print "--Bb:C04y\r\n"
      input.print 'Content-Disposition: file; filename="hello.html"' + "\r\n"
      input.print "Content-Type: text/html\r\n"
      input.print "\r\n"
      input.print "<html>\n"
      input.print "<body><p>Hello world.</p></body>\n"
      input.print "</html>\n"
      input.print "\r\n"
      input.print "--Bb:C04y\r\n"
      input.print 'Content-Disposition: file; filename="hello.css"' + "\r\n"
      input.print "Content-Type: text/css\r\n"
      input.print "\r\n"
      input.print "p { font-weight: bold; font-size: large }\n"
      input.print "\r\n"
      input.print "--Bb:C04y--\r\n"
      input.print "--AaB03x--\r\n"

      form_data_parsed_alist = [
	[ :form_data_name, 'submit' ],
	[ :form_data_head, WWWsrv::Header.build(
	    'Content-Disposition' => 'form-data; name="submit"'
	  )
	],
	[ :form_data_body, 'OK' ],
	[ :form_data_end, nil ],
	# -
	[ :form_data_name, 'date' ],
	[ :form_data_head, WWWsrv::Header.build(
	    'Content-Disposition' => 'form-data; name="date"'
	  )
	],
	[ :form_data_body, '1975-11-19' ],
	[ :form_data_end, nil ],
	# -
	[ :form_data_name, 'text' ],
	[ :form_data_file, 'hello.txt' ],
	[ :form_data_head, WWWsrv::Header.build(
	    'Content-Disposition' => 'form-data; name="text"; filename="hello.txt"',
	    'Content-Type' => 'text/plain'
	  )
	],
	[ :form_data_body, "Hello world.\n" ],
	[ :form_data_end, nil ],
	# -
	[ :form_data_name, 'html' ],
	[ :form_data_head, WWWsrv::Header.build(
	    'Content-Disposition' => 'form-data; name="html"',
	    'Content-Type' => 'multipart/mixed; boundary="Bb:C04y"'
	  )
	],
	[ :begin_mixed, nil ],
	# --
	[ :form_data_file, 'hello.html' ],
	[ :form_data_head, WWWsrv::Header.build(
	    'Content-Disposition' => 'file; filename="hello.html"',
	    'Content-Type' => 'text/html'
	  )
	],
	[ :form_data_body, "<html>\n" ],
	[ :form_data_body, "<body><p>Hello world.</p></body>\n" ],
	[ :form_data_body, "</html>\n" ],
	[ :form_data_end, nil ],
	# --
	[ :form_data_file, 'hello.css' ],
	[ :form_data_head, WWWsrv::Header.build(
	    'Content-Disposition' => 'file; filename="hello.css"',
	    'Content-Type' => 'text/css'
	  )
	],
	[ :form_data_body, "p { font-weight: bold; font-size: large }\n" ],
	[ :form_data_end, nil ],
	# --
	[ :end_mixed, nil ]
      ]

      form_data_scan_alist = Array.new
      WWWsrv::FormData.scan_form_data(input, content_type) do |tag, value|
	form_data_scan_alist.push([ tag, value ])
      end

      assert_equal(form_data_parsed_alist, form_data_scan_alist)
      assert_nil(input.data)
    end
  end
end

if ($0 == __FILE__) then
  if (ARGV.empty?) then
    TestWWWsrv::constants.map{ |name|
      TestWWWsrv::const_get(name)
    }.select{ |const|
      (const.kind_of? Class) && (const < RUNIT::TestCase)
    }.each do |test_case|
      RUNIT::CUI::TestRunner::run(test_case::suite)
    end
  else
    for class_name in ARGV
      test_case = TestWWWsrv.const_get("Test#{class_name}")
      RUNIT::CUI::TestRunner.run(test_case.suite)
    end
  end
end
