#!/usr/local/bin/ruby
# $Id: test_htmlconv.rb,v 1.3.2.1 2003/01/05 13:06:36 toki Exp $
# -
# Test of WWWsrv::HTMLconv module

require 'wwwsrv/htmlconv'
require 'runit/testcase'
require 'runit/cui/testrunner'

module TestWWWsrv
  class TestHTMLconv < RUNIT::TestCase
    def test_http_date
      time = Time::gm(2001, 11, 19, 12, 12, 12)
      assert_equal('Mon, 19 Nov 2001 12:12:12 GMT', WWWsrv::HTMLconv::http_date(time))
    end

    def test_quoteHTML
      assert_equal('&quot;&lt;&gt;&amp;', WWWsrv::HTMLconv::quoteHTML('"<>&'))
    end

    def test_unquoteHTML
      assert_equal('"<>&', WWWsrv::HTMLconv::unquoteHTML('&quot;&lt;&gt;&amp;'))
    end

    def test_quoteSHELL
      assert_equal('"\"\$\`\\\\"', WWWsrv::HTMLconv::quoteSHELL('"$`\\'))
    end

    def test_encodeURL
      for c in 0x00..0xFF
	char = c.chr
	encode_url = format('%%%02X', c)
	if (char !~ /[ _0-9A-Za-z.-]/) then
	  if (char == ' ') then
	    assert_equal('+', WWWsrv::HTMLconv::encodeURL(char))
	  else
	    assert_equal(encode_url, WWWsrv::HTMLconv::encodeURL(char))
	  end
	end
      end
    end

    def test_decodeURL
      for c in 0x00..0xFF
	char = c.chr
	encode_url = format('%%%02X', c)
	assert_equal(char, WWWsrv::HTMLconv::decodeURL(encode_url))
	encode_url = format('%%%02x', c)
	assert_equal(char, WWWsrv::HTMLconv::decodeURL(encode_url))
      end
    end

    def test_parse_query
      param_data = [
	%w[ double first ],
	%w[ double second ],
	%w[ debug true ]
      ].map{ |param_name, param_value|
	WWWsrv::HTMLconv::encodeURL(param_name) + '=' +
	  WWWsrv::HTMLconv::encodeURL(param_value)
      }.join('&')

      params = WWWsrv::HTMLconv::decode_query(param_data)
      assert_equal('first', params['double'][0])
      assert_equal('second', params['double'][1])
      assert_equal('true', params['debug'][0])
    end
  end
end

if ($0 == __FILE__) then
  if (ARGV.empty?) then
    TestWWWsrv::constants.map{ |name|
      TestWWWsrv::const_get(name)
    }.select{ |const|
      (const.kind_of? Class) && (const < RUNIT::TestCase)
    }.each do |test_case|
      RUNIT::CUI::TestRunner::run(test_case::suite)
    end
  else
    for class_name in ARGV
      test_case = TestWWWsrv.const_get("Test#{class_name}")
      RUNIT::CUI::TestRunner.run(test_case.suite)
    end
  end
end
