#!/usr/local/bin/ruby
# $Id: test_logger.rb,v 1.5.2.1 2003/01/05 13:06:38 toki Exp $
# -
# Test of WWWsrv::Logger class
# Test of WWWsrv::MultiLogger class

require 'pseudo_io'
require 'wwwsrv/logger'
require 'runit/testcase'
require 'runit/cui/testrunner'

module TestWWWsrv
  class TestLogger < RUNIT::TestCase
    def setup
      @int_iobuf = PseudoIO::new
      @int_logger = WWWsrv::Logger::new(@int_iobuf, WWWsrv::Logger::INTERNAL)
      @dbg_iobuf = PseudoIO::new
      @dbg_logger = WWWsrv::Logger::new(@dbg_iobuf, WWWsrv::Logger::DEBUG)
      @sys_iobuf = PseudoIO::new
      @sys_logger = WWWsrv::Logger::new(@sys_iobuf, WWWsrv::Logger::SYSTEM)
      @msg_iobuf = PseudoIO::new
      @msg_logger = WWWsrv::Logger::new(@msg_iobuf, WWWsrv::Logger::MESSAGE)
      @wrn_iobuf = PseudoIO::new
      @wrn_logger = WWWsrv::Logger::new(@wrn_iobuf, WWWsrv::Logger::WARNING)
      @ntc_iobuf = PseudoIO::new
      @ntc_logger = WWWsrv::Logger::new(@ntc_iobuf, WWWsrv::Logger::NOTICE)
      @alr_iobuf = PseudoIO::new
      @alr_logger = WWWsrv::Logger::new(@alr_iobuf, WWWsrv::Logger::ALERT)
      @err_iobuf = PseudoIO::new
      @err_logger = WWWsrv::Logger::new(@err_iobuf, WWWsrv::Logger::ERROR)

      @multi_logger = WWWsrv::MultiLogger::new
      @multi_logger.add_logger(@int_logger)
      @multi_logger.add_logger(@dbg_logger)
      @multi_logger.add_logger(@sys_logger)
      @multi_logger.add_logger(@msg_logger)
      @multi_logger.add_logger(@wrn_logger)
      @multi_logger.add_logger(@ntc_logger)
      @multi_logger.add_logger(@alr_logger)
      @multi_logger.add_logger(@err_logger)
    end

    def test_new
      assert_instance_of(WWWsrv::Logger, @int_logger)
      assert_instance_of(WWWsrv::Logger, @dbg_logger)
      assert_instance_of(WWWsrv::Logger, @sys_logger)
      assert_instance_of(WWWsrv::Logger, @msg_logger)
      assert_instance_of(WWWsrv::Logger, @wrn_logger)
      assert_instance_of(WWWsrv::Logger, @ntc_logger)
      assert_instance_of(WWWsrv::Logger, @alr_logger)
      assert_instance_of(WWWsrv::Logger, @err_logger)
    end

    def test_internal
      @multi_logger.print(WWWsrv::Logger::INTERNAL, 'hello')
      assert_match(/hello/, @int_iobuf.data)
      assert_nil(@dbg_iobuf.data)
      assert_nil(@sys_iobuf.data)
      assert_nil(@msg_iobuf.data)
      assert_nil(@wrn_iobuf.data)
      assert_nil(@ntc_iobuf.data)
      assert_nil(@alr_iobuf.data)
      assert_nil(@err_iobuf.data)

      @multi_logger.internal('world')
      assert_match(/world/, @int_iobuf.data)
      assert_nil(@dbg_iobuf.data)
      assert_nil(@sys_iobuf.data)
      assert_nil(@msg_iobuf.data)
      assert_nil(@wrn_iobuf.data)
      assert_nil(@ntc_iobuf.data)
      assert_nil(@alr_iobuf.data)
      assert_nil(@err_iobuf.data)
    end

    def test_debug
      @multi_logger.print(WWWsrv::Logger::DEBUG, 'hello')
      assert_match(/hello/, @int_iobuf.data)
      assert_match(/hello/, @dbg_iobuf.data)
      assert_nil(@sys_iobuf.data)
      assert_nil(@msg_iobuf.data)
      assert_nil(@wrn_iobuf.data)
      assert_nil(@ntc_iobuf.data)
      assert_nil(@alr_iobuf.data)
      assert_nil(@err_iobuf.data)

      @multi_logger.debug('world')
      assert_match(/world/, @int_iobuf.data)
      assert_match(/world/, @dbg_iobuf.data)
      assert_nil(@sys_iobuf.data)
      assert_nil(@msg_iobuf.data)
      assert_nil(@wrn_iobuf.data)
      assert_nil(@ntc_iobuf.data)
      assert_nil(@alr_iobuf.data)
      assert_nil(@err_iobuf.data)
    end

    def test_system
      @multi_logger.print(WWWsrv::Logger::SYSTEM, 'hello')
      assert_match(/hello/, @int_iobuf.data)
      assert_match(/hello/, @dbg_iobuf.data)
      assert_match(/hello/, @sys_iobuf.data)
      assert_nil(@msg_iobuf.data)
      assert_nil(@wrn_iobuf.data)
      assert_nil(@ntc_iobuf.data)
      assert_nil(@alr_iobuf.data)
      assert_nil(@err_iobuf.data)

      @multi_logger.system('world')
      assert_match(/world/, @int_iobuf.data)
      assert_match(/world/, @dbg_iobuf.data)
      assert_match(/world/, @sys_iobuf.data)
      assert_nil(@msg_iobuf.data)
      assert_nil(@wrn_iobuf.data)
      assert_nil(@ntc_iobuf.data)
      assert_nil(@alr_iobuf.data)
      assert_nil(@err_iobuf.data)
    end

    def test_message
      @multi_logger.print(WWWsrv::Logger::MESSAGE, 'hello')
      assert_match(/hello/, @int_iobuf.data)
      assert_match(/hello/, @dbg_iobuf.data)
      assert_match(/hello/, @sys_iobuf.data)
      assert_match(/hello/, @msg_iobuf.data)
      assert_nil(@wrn_iobuf.data)
      assert_nil(@ntc_iobuf.data)
      assert_nil(@alr_iobuf.data)
      assert_nil(@err_iobuf.data)

      @multi_logger.message('world')
      assert_match(/world/, @int_iobuf.data)
      assert_match(/world/, @dbg_iobuf.data)
      assert_match(/world/, @sys_iobuf.data)
      assert_match(/world/, @msg_iobuf.data)
      assert_nil(@wrn_iobuf.data)
      assert_nil(@ntc_iobuf.data)
      assert_nil(@alr_iobuf.data)
      assert_nil(@err_iobuf.data)
    end

    def test_warning
      @multi_logger.print(WWWsrv::Logger::WARNING, 'hello')
      assert_match(/hello/, @int_iobuf.data)
      assert_match(/hello/, @dbg_iobuf.data)
      assert_match(/hello/, @sys_iobuf.data)
      assert_match(/hello/, @msg_iobuf.data)
      assert_match(/hello/, @wrn_iobuf.data)
      assert_nil(@ntc_iobuf.data)
      assert_nil(@alr_iobuf.data)
      assert_nil(@err_iobuf.data)

      @multi_logger.warning('world')
      assert_match(/world/, @int_iobuf.data)
      assert_match(/world/, @dbg_iobuf.data)
      assert_match(/world/, @sys_iobuf.data)
      assert_match(/world/, @msg_iobuf.data)
      assert_match(/world/, @wrn_iobuf.data)
      assert_nil(@ntc_iobuf.data)
      assert_nil(@alr_iobuf.data)
      assert_nil(@err_iobuf.data)
    end

    def test_notice
      @multi_logger.print(WWWsrv::Logger::NOTICE, 'hello')
      assert_match(/hello/, @int_iobuf.data)
      assert_match(/hello/, @dbg_iobuf.data)
      assert_match(/hello/, @sys_iobuf.data)
      assert_match(/hello/, @msg_iobuf.data)
      assert_match(/hello/, @wrn_iobuf.data)
      assert_match(/hello/, @ntc_iobuf.data)
      assert_nil(@alr_iobuf.data)
      assert_nil(@err_iobuf.data)

      @multi_logger.notice('world')
      assert_match(/world/, @int_iobuf.data)
      assert_match(/world/, @dbg_iobuf.data)
      assert_match(/world/, @sys_iobuf.data)
      assert_match(/world/, @msg_iobuf.data)
      assert_match(/world/, @wrn_iobuf.data)
      assert_match(/world/, @ntc_iobuf.data)
      assert_nil(@alr_iobuf.data)
      assert_nil(@err_iobuf.data)
    end

    def test_alert
      @multi_logger.print(WWWsrv::Logger::ALERT, 'hello')
      assert_match(/hello/, @int_iobuf.data)
      assert_match(/hello/, @dbg_iobuf.data)
      assert_match(/hello/, @sys_iobuf.data)
      assert_match(/hello/, @msg_iobuf.data)
      assert_match(/hello/, @wrn_iobuf.data)
      assert_match(/hello/, @ntc_iobuf.data)
      assert_match(/hello/, @alr_iobuf.data)
      assert_nil(@err_iobuf.data)

      @multi_logger.alert('world')
      assert_match(/world/, @int_iobuf.data)
      assert_match(/world/, @dbg_iobuf.data)
      assert_match(/world/, @sys_iobuf.data)
      assert_match(/world/, @msg_iobuf.data)
      assert_match(/world/, @wrn_iobuf.data)
      assert_match(/world/, @ntc_iobuf.data)
      assert_match(/world/, @alr_iobuf.data)
      assert_nil(@err_iobuf.data)
    end

    def test_error
      @multi_logger.print(WWWsrv::Logger::ERROR, 'hello')
      assert_match(/hello/, @int_iobuf.data)
      assert_match(/hello/, @dbg_iobuf.data)
      assert_match(/hello/, @sys_iobuf.data)
      assert_match(/hello/, @msg_iobuf.data)
      assert_match(/hello/, @wrn_iobuf.data)
      assert_match(/hello/, @ntc_iobuf.data)
      assert_match(/hello/, @alr_iobuf.data)
      assert_match(/hello/, @err_iobuf.data)

      @multi_logger.error('world')
      assert_match(/world/, @int_iobuf.data)
      assert_match(/world/, @dbg_iobuf.data)
      assert_match(/world/, @sys_iobuf.data)
      assert_match(/world/, @msg_iobuf.data)
      assert_match(/world/, @wrn_iobuf.data)
      assert_match(/world/, @ntc_iobuf.data)
      assert_match(/world/, @alr_iobuf.data)
      assert_match(/world/, @err_iobuf.data)
    end
  end
end

if ($0 == __FILE__) then
  if (ARGV.empty?) then
    TestWWWsrv::constants.map{ |name|
      TestWWWsrv::const_get(name)
    }.select{ |const|
      (const.kind_of? Class) && (const < RUNIT::TestCase)
    }.each do |test_case|
      RUNIT::CUI::TestRunner::run(test_case::suite)
    end
  else
    for class_name in ARGV
      test_case = TestWWWsrv.const_get("Test#{class_name}")
      RUNIT::CUI::TestRunner.run(test_case.suite)
    end
  end
end
