#!/usr/local/bin/ruby
# $Id: test_multipart.rb,v 1.5.4.1 2003/01/05 13:06:40 toki Exp $
# -
# Test of WWWsrv::Multipart module

require 'pseudo_io'
require 'wwwsrv/htmlconv'
require 'wwwsrv/multipart'
require 'runit/testcase'
require 'runit/cui/testrunner'

# for debug
module WWWsrv
  class Header
    def inspect
      self.to_a.sort.inspect
    end
  end
end

module TestWWWsrv
  class TestMultipart < RUNIT::TestCase
    def setup
      @input = PseudoIO.new
      @boundary = 'AaB03x'
      @content_type = "multipart/form-data; boundary=#{@boundary}"
    end

    def test_scan_multipart
      @input.print "--AaB03x\r\n"
      @input.print 'Content-Disposition: form-data; name="submit"' + "\r\n"
      @input.print "\r\n"
      @input.print "OK\r\n"
      @input.print "--AaB03x\r\n"
      @input.print 'Content-Disposition: form-data; name="date"' + "\r\n"
      @input.print "\r\n"
      @input.print "1975-11-19\r\n"
      @input.print "--AaB03x\r\n"
      @input.print 'Content-Disposition: form-data; name="text"; filename="hello.txt"' + "\r\n"
      @input.print "Content-Type: text/html\r\n"
      @input.print "\r\n"
      @input.print "Hello world.\n"
      @input.print "\r\n"
      @input.print "--AaB03x\r\n"
      @input.print 'Content-Disposition: form-data; name="files"' + "\r\n"
      @input.print 'Content-Type: multipart/mixed; boundary="Bb:C04y"' + "\r\n"
      @input.print "\r\n"
      @input.print "--Bb:C04y\r\n"
      @input.print 'Content-Disposition: file; filename="hello.html"' + "\r\n"
      @input.print "Content-Type: text/html\r\n"
      @input.print "\r\n"
      @input.print "<html>\n"
      @input.print "<body><p>Hello world.</p></body>\n"
      @input.print "</html>\n"
      @input.print "\r\n"
      @input.print "--Bb:C04y\r\n"
      @input.print 'Content-Disposition: file; filename="hello.css"' + "\r\n"
      @input.print "Content-Type: text/css\r\n"
      @input.print "\r\n"
      @input.print "p { font-weight: bold; font-size: large }\n"
      @input.print "\r\n"
      @input.print "--Bb:C04y--\r\n"
      @input.print "--AaB03x--\r\n"

      multipart_parsed_alist = [
	[ :begin_multipart, nil ],
	# -
	[ :part_head, WWWsrv::Header.build('Content-Disposition' => 'form-data; name="submit"') ],
	[ :part_body, 'OK' ],
	[ :part_end, nil ],
	# -
	[ :part_head, WWWsrv::Header.build('Content-Disposition' => 'form-data; name="date"') ],
	[ :part_body, '1975-11-19' ],
	[ :part_end, nil ],
	# -
	[ :part_head, WWWsrv::Header.build(
	    'Content-Disposition' => 'form-data; name="text"; filename="hello.txt"',
	    'Content-Type' => 'text/html'
	  )
	],
	[ :part_body, "Hello world.\n" ],
	[ :part_end, nil ],
	# -
	[ :part_head, WWWsrv::Header.build(
	    'Content-Disposition' => 'form-data; name="files"',
	    'Content-Type' => 'multipart/mixed; boundary="Bb:C04y"'
	  )
	],
	# -
	[ :begin_multipart, nil ],
	# --
	[ :part_head, WWWsrv::Header.build(
	    'Content-Disposition' => 'file; filename="hello.html"',
	    'Content-Type' => 'text/html'
	  )
	],
	[ :part_body, "<html>\n" ],
	[ :part_body, "<body><p>Hello world.</p></body>\n" ],
	[ :part_body, "</html>\n" ],
	[ :part_end, nil ],
	# --
	[ :part_head, WWWsrv::Header.build(
	    'Content-Disposition' => 'file; filename="hello.css"',
	    'Content-Type' => 'text/css'
	  )
	],
	[ :part_body, "p { font-weight: bold; font-size: large }\n" ],
	[ :part_end, nil ],
	# --
	[ :end_multipart, nil ],
	# -
	[ :end_multipart, nil ]
      ]

      multipart_scan_alist = Array.new
      WWWsrv::Multipart.scan_multipart(@input, @content_type) do |tag, value|
	multipart_scan_alist.push([ tag, value ])
      end

      assert_equal(multipart_parsed_alist, multipart_scan_alist)
      assert_nil(@input.data)
    end

    def test_scan_multipart_with_ln
      @input.print "--AaB03x\n"
      @input.print 'Content-Disposition: form-data; name="submit"' + "\n"
      @input.print "\n"
      @input.print "OK\n"
      @input.print "--AaB03x\n"
      @input.print 'Content-Disposition: form-data; name="date"' + "\n"
      @input.print "\n"
      @input.print "1975-11-19\n"
      @input.print "--AaB03x\r\n"
      @input.print 'Content-Disposition: form-data; name="text"; filename="hello.txt"' + "\r\n"
      @input.print "Content-Type: text/html\r\n"
      @input.print "\r\n"
      @input.print "Hello world.\n"
      @input.print "\r\n"
      @input.print "--AaB03x\n"
      @input.print 'Content-Disposition: form-data; name="files"' + "\n"
      @input.print 'Content-Type: multipart/mixed; boundary="Bb:C04y"' + "\n"
      @input.print "\n"
      @input.print "--Bb:C04y\n"
      @input.print 'Content-Disposition: file; filename="hello.html"' + "\n"
      @input.print 'Content-Type: text/html' + "\n"
      @input.print "\n"
      @input.print "<html>\n"
      @input.print "<body><p>Hello world.</p></body>\n"
      @input.print "</html>\n"
      @input.print "\n"
      @input.print "--Bb:C04y\n"
      @input.print 'Content-Disposition: file; filename="hello.css"' + "\n"
      @input.print 'Content-Type: text/css' + "\n"
      @input.print "\n"
      @input.print "p { font-weight: bold; font-size: large }\n"
      @input.print "\n"
      @input.print "--Bb:C04y--\n"
      @input.print "--AaB03x--\n"

      multipart_parsed_alist = [
	[ :begin_multipart, nil ],
	# -
	[ :part_head, WWWsrv::Header.build('Content-Disposition' => 'form-data; name="submit"') ],
	[ :part_body, 'OK' ],
	[ :part_end, nil ],
	# -
	[ :part_head, WWWsrv::Header.build('Content-Disposition' => 'form-data; name="date"') ],
	[ :part_body, '1975-11-19' ],
	[ :part_end, nil ],
	# -
	[ :part_head, WWWsrv::Header.build(
	    'Content-Disposition' => 'form-data; name="text"; filename="hello.txt"',
	    'Content-Type' => 'text/html'
	  )
	],
	[ :part_body, "Hello world.\n" ],
	[ :part_end, nil ],
	# -
	[ :part_head, WWWsrv::Header.build(
	    'Content-Disposition' => 'form-data; name="files"',
	    'Content-Type' => 'multipart/mixed; boundary="Bb:C04y"'
	  )
	],
	# -
	[ :begin_multipart, nil ],
	# --
	[ :part_head, WWWsrv::Header.build(
	    'Content-Disposition' => 'file; filename="hello.html"',
	    'Content-Type' => 'text/html'
	  )
	],
	[ :part_body, "<html>\n" ],
	[ :part_body, "<body><p>Hello world.</p></body>\n" ],
	[ :part_body, "</html>\n" ],
	[ :part_end, nil ],
	# --
	[ :part_head, WWWsrv::Header.build(
	    'Content-Disposition' => 'file; filename="hello.css"',
	    'Content-Type' => 'text/css'
	  )
	],
	[ :part_body, "p { font-weight: bold; font-size: large }\n" ],
	[ :part_end, nil ],
	# --
	[ :end_multipart, nil ],
	# -
	[ :end_multipart, nil ]
      ]

      multipart_scan_alist = Array.new
      WWWsrv::Multipart.scan_multipart(@input, @content_type) do |tag, value|
	multipart_scan_alist.push([ tag, value ])
      end

      assert_equal(multipart_parsed_alist, multipart_scan_alist)
      assert_nil(@input.data)
    end

    def test_scan_multipart_with_preamble_and_epilogue
      @input.print "This is outer preamble.\n"
      @input.print "--AaB03x\r\n"
      @input.print 'Content-Disposition: form-data; name="submit"' + "\r\n"
      @input.print "\r\n"
      @input.print "OK\r\n"
      @input.print "--AaB03x\r\n"
      @input.print 'Content-Disposition: form-data; name="date"' + "\r\n"
      @input.print "\r\n"
      @input.print "1975-11-19\r\n"
      @input.print "--AaB03x\r\n"
      @input.print 'Content-Disposition: form-data; name="text"; filename="hello.txt"' + "\r\n"
      @input.print "Content-Type: text/html\r\n"
      @input.print "\r\n"
      @input.print "Hello world.\n"
      @input.print "\r\n"
      @input.print "--AaB03x\r\n"
      @input.print 'Content-Disposition: form-data; name="files"' + "\r\n"
      @input.print 'Content-Type: multipart/mixed; boundary="Bb:C04y"' + "\r\n"
      @input.print "\r\n"
      @input.print "This is inner preamble.\n"
      @input.print "--Bb:C04y\r\n"
      @input.print 'Content-Disposition: file; filename="hello.html"' + "\r\n"
      @input.print "Content-Type: text/html\r\n"
      @input.print "\r\n"
      @input.print "<html>\n"
      @input.print "<body><p>Hello world.</p></body>\n"
      @input.print "</html>\n"
      @input.print "\r\n"
      @input.print "--Bb:C04y\r\n"
      @input.print 'Content-Disposition: file; filename="hello.css"' + "\r\n"
      @input.print "Content-Type: text/css\r\n"
      @input.print "\r\n"
      @input.print "p { font-weight: bold; font-size: large }\n"
      @input.print "\r\n"
      @input.print "--Bb:C04y--\r\n"
      @input.print "This is inner epilogue.\n"
      @input.print "--AaB03x--\r\n"
      @input.print "This is outer epilogue.\n"

      multipart_parsed_alist = [
	[ :begin_multipart, nil ],
	# -
	[ :part_head, WWWsrv::Header.build('Content-Disposition' => 'form-data; name="submit"') ],
	[ :part_body, 'OK' ],
	[ :part_end, nil ],
	# -
	[ :part_head, WWWsrv::Header.build('Content-Disposition' => 'form-data; name="date"') ],
	[ :part_body, '1975-11-19' ],
	[ :part_end, nil ],
	# -
	[ :part_head, WWWsrv::Header.build(
	    'Content-Disposition' => 'form-data; name="text"; filename="hello.txt"',
	    'Content-Type' => 'text/html'
	  )
	],
	[ :part_body, "Hello world.\n" ],
	[ :part_end, nil ],
	# -
	[ :part_head, WWWsrv::Header.build(
	    'Content-Disposition' => 'form-data; name="files"',
	    'Content-Type' => 'multipart/mixed; boundary="Bb:C04y"'
	  )
	],
	[ :begin_multipart, nil ],
	# --
	[ :part_head, WWWsrv::Header.build(
	    'Content-Disposition' => 'file; filename="hello.html"',
	    'Content-Type' => 'text/html'
	  )
	],
	[ :part_body, "<html>\n" ],
	[ :part_body, "<body><p>Hello world.</p></body>\n" ],
	[ :part_body, "</html>\n" ],
	[ :part_end, nil ],
	# --
	[ :part_head, WWWsrv::Header.build(
	    'Content-Disposition' => 'file; filename="hello.css"',
	    'Content-Type' => 'text/css'
	  )
	],
	[ :part_body, "p { font-weight: bold; font-size: large }\n" ],
	[ :part_end, nil ],
	# --
	[ :end_multipart, nil ],
	# -
	[ :end_multipart, nil ]
      ]

      multipart_scan_alist = Array.new
      WWWsrv::Multipart.scan_multipart(@input, @content_type) do |tag, value|
	multipart_scan_alist.push([ tag, value ])
      end

      assert_equal(multipart_parsed_alist, multipart_scan_alist)
      assert_equal("This is outer epilogue.\n", @input.data)
    end

    def test_scan_multipart_incomplete
      @input.print "--AaB03x\r\n"
      @input.print 'Content-Disposition: form-data; name="submit"' + "\r\n"
      @input.print "\r\n"
      @input.print "OK\r\n"
      @input.print "--AaB03x\r\n"
      @input.print 'Content-Disposition: form-data; name="date"' + "\r\n"
      @input.print "\r\n"
      @input.print "1975-11-19\r\n"
      @input.print "--AaB03x\r\n"
      @input.print 'Content-Disposition: form-data; name="text"; filename="hello.txt"' + "\r\n"
      @input.print "Content-Type: text/html\r\n"
      @input.print "\r\n"
      @input.print "Hello world.\n"
      @input.print "\r\n"
      @input.print "--AaB03x\r\n"
      @input.print 'Content-Disposition: form-data; name="files"' + "\r\n"
      @input.print 'Content-Type: multipart/mixed; boundary="Bb:C04y"' + "\r\n"
      @input.print "\r\n"
      @input.print "--Bb:C04y\r\n"
      @input.print 'Content-Disposition: file; filename="hello.html"' + "\r\n"
      @input.print "Content-Type: text/html\r\n"
      @input.print "\r\n"
      @input.print "<html>\n"
      @input.print "<body><p>Hello world.</p></body>\n"
      @input.print "</html>\n"
      @input.print "\r\n"
      @input.print "--Bb:C04y\r\n"
      @input.print 'Content-Disposition: file; filename="hello.css"' + "\r\n"
      @input.print "Content-Type: text/css\r\n"
      @input.print "\r\n"
      @input.print "p { font-weight: bold; font-size: large }\n"
      @input.print "\r\n"
      @input.print #"--Bb:C04y--\r\n" # not terminated
      @input.print "--AaB03x--\r\n"

      assert_exception(WWWsrv::MultipartParseError) {
	multipart_scan_alist = Array.new
	# raise CGIw::MultipartParseError exception
	WWWsrv::Multipart.scan_multipart(@input, @content_type) do |tag, value|
	  multipart_scan_alist.push([ tag, value ]) 
	end
      }
    end

    def test_scan_multipart_without_boundary
      @input.print "--AaB03x\r\n"
      @input.print 'Content-Disposition: form-data; name="submit"' + "\r\n"
      @input.print "\r\n"
      @input.print "OK\r\n"
      @input.print "--AaB03x\r\n"
      @input.print 'Content-Disposition: form-data; name="date"' + "\r\n"
      @input.print "\r\n"
      @input.print "1975-11-19\r\n"
      @input.print "--AaB03x\r\n"
      @input.print 'Content-Disposition: form-data; name="text"; filename="hello.txt"' + "\r\n"
      @input.print "Content-Type: text/html\r\n"
      @input.print "\r\n"
      @input.print "Hello world.\n"
      @input.print "\r\n"
      @input.print "--AaB03x\r\n"
      @input.print 'Content-Disposition: form-data; name="files"' + "\r\n"
      @input.print 'Content-Type: multipart/mixed' + "\r\n" # without boundary
      @input.print "\r\n"
      @input.print "--Bb:C04y\r\n"
      @input.print 'Content-Disposition: file; filename="hello.html"' + "\r\n"
      @input.print "Content-Type: text/html\r\n"
      @input.print "\r\n"
      @input.print "<html>\n"
      @input.print "<body><p>Hello world.</p></body>\n"
      @input.print "</html>\n"
      @input.print "\r\n"
      @input.print "--Bb:C04y\r\n"
      @input.print 'Content-Disposition: file; filename="hello.css"' + "\r\n"
      @input.print "Content-Type: text/css\r\n"
      @input.print "\r\n"
      @input.print "p { font-weight: bold; font-size: large }\n"
      @input.print "\r\n"
      @input.print "--Bb:C04y--\r\n"
      @input.print "--AaB03x--\r\n"

      assert_exception(WWWsrv::MultipartParseError) {
	multipart_scan_alist = Array.new
	# raise CGIw::MultipartParseError exception
	WWWsrv::Multipart.scan_multipart(@input, @content_type) do |tag, value|
	    multipart_scan_alist.push([ tag, value ])
	end
      }
    end

    def test_scan_multipart_empty
      @input.print "--AaB03x--\r\n"

      multipart_parsed_alist = [
	[ :begin_multipart, nil ],
	[ :end_multipart, nil ]
      ]

      multipart_scan_alist = Array.new
      WWWsrv::Multipart.scan_multipart(@input, @content_type) do |tag, value|
	multipart_scan_alist.push([ tag, value ])
      end

      assert_equal(multipart_parsed_alist, multipart_scan_alist)
      assert_nil(@input.data)
    end
  end
end

if ($0 == __FILE__) then
  if (ARGV.empty?) then
    TestWWWsrv::constants.map{ |name|
      TestWWWsrv::const_get(name)
    }.select{ |const|
      (const.kind_of? Class) && (const < RUNIT::TestCase)
    }.each do |test_case|
      RUNIT::CUI::TestRunner::run(test_case::suite)
    end
  else
    for class_name in ARGV
      test_case = TestWWWsrv.const_get("Test#{class_name}")
      RUNIT::CUI::TestRunner.run(test_case.suite)
    end
  end
end
