#!/usr/local/bin/ruby
# $Id: test_path.rb,v 1.6.2.1 2003/01/05 13:06:41 toki Exp $
# -
# Test of WWWsrv::Path module

require 'wwwsrv/path'
require 'runit/testcase'
require 'runit/cui/testrunner'

module TestWWWsrv
  class TestPath < RUNIT::TestCase
    def test_subpath
      #   prefix     path             subpath
      [ [ "",      "/foo",      "/foo"  ],
	[ "",      "/foo/",     "/foo/" ],
	[ "/",     "/foo",      "/foo"  ],
	[ "/foo",  "/foo",      ""      ],
	[ "/foo",  "/foo/",     "/"     ],
	[ "/foo/", "/foo/",     "/"     ],
	[ "/foo",  "/foo/bar",  "/bar"  ],
	[ "/foo",  "/foo/bar/", "/bar/" ],
	[ "/foo/", "/foo/bar",  "/bar"  ],
	[ "",      "foo",       "foo"   ],
	[ "",      "foo/",      "foo/"  ],
	[ "foo",   "foo/bar",   "/bar"  ],
	[ "foo",   "foo/bar/",  "/bar/" ],
	[ "foo/",  "foo/bar",   "/bar"  ]
      ].each do |prefix, path, subpath|
	assert_equal(subpath, WWWsrv::Path::subpath(prefix, path))
      end
      assert_exception(RuntimeError) { WWWsrv::Path::subpath('/hello/world', '/hello') }
      assert_exception(RuntimeError) { WWWsrv::Path::subpath('/hello', '/detarame') }
    end

    def test_scan_path_info
      path_alist = [
	[ '/first/second/third/fourth/fifth', '' ],
	[ '/first/second/third/fourth', '/fifth' ],
	[ '/first/second/third', '/fourth/fifth' ],
	[ '/first/second', '/third/fourth/fifth' ],
	[ '/first', '/second/third/fourth/fifth' ],
	[ '', '/first/second/third/fourth/fifth' ]
      ]
      WWWsrv::Path::scan_path_info('/first/second/third/fourth/fifth') do |script_name, path_info|
	assert(! path_alist.empty?)
	assert_equal(path_alist.first[0], script_name)
	assert_equal(path_alist.first[1], path_info)
	path_alist.shift
      end
      assert(path_alist.empty?)

      path_alist = [
	[ 'first/second/third/fourth/fifth', '' ],
	[ 'first/second/third/fourth', '/fifth' ],
	[ 'first/second/third', '/fourth/fifth' ],
	[ 'first/second', '/third/fourth/fifth' ],
	[ 'first', '/second/third/fourth/fifth' ],
	[ '', 'first/second/third/fourth/fifth' ]
      ]
      WWWsrv::Path::scan_path_info('first/second/third/fourth/fifth') do |script_name, path_info|
	assert(! path_alist.empty?)
	assert_equal(path_alist.first[0], script_name)
	assert_equal(path_alist.first[1], path_info)
	path_alist.shift
      end
      assert(path_alist.empty?)
    end

    def test_expand_http_path
      assert_equal('/', WWWsrv::Path::expand_http_path(''))
      assert_equal('/foo/bar', WWWsrv::Path::expand_http_path('/foo/./bar'))
      assert_equal('/bar', WWWsrv::Path::expand_http_path('/foo/../bar'))
      assert_equal('/foo/', WWWsrv::Path::expand_http_path('/foo/.'))
      assert_equal('/foo/', WWWsrv::Path::expand_http_path('/foo/bar/..'))
      assert_equal('/foo', WWWsrv::Path::expand_http_path('/../foo'))
      assert_equal('/foo/', WWWsrv::Path::expand_http_path('/foo/.'))
      assert_equal('/foo', WWWsrv::Path::expand_http_path('/../foo'))
      assert_equal('/', WWWsrv::Path::expand_http_path('/..'))
      assert_equal('/foo bar', WWWsrv::Path::expand_http_path('/foo+bar'))
      assert_equal('/foo/~bar', WWWsrv::Path::expand_http_path('/foo/%7Ebar'))
    end

    def test_normalize_path
      assert_equal('/', WWWsrv::Path::normalize_path('')[0])
      assert_equal('/foo/bar', WWWsrv::Path::normalize_path('/foo/./bar')[0])
      assert_equal('/bar', WWWsrv::Path::normalize_path('/foo/../bar')[0])
      assert_equal('/foo/', WWWsrv::Path::normalize_path('/foo/.')[0])
      assert_equal('/foo/', WWWsrv::Path::normalize_path('/foo/bar/..')[0])
      assert_equal('/foo', WWWsrv::Path::normalize_path('/../foo')[0])
      assert_equal('/foo/', WWWsrv::Path::normalize_path('/foo/.')[0])
      assert_equal('/foo', WWWsrv::Path::normalize_path('/../foo')[0])
      assert_equal('/', WWWsrv::Path::normalize_path('/..')[0])
      assert_equal('/foo', WWWsrv::Path::normalize_path('/foo?bar')[0])
      assert_equal('bar', WWWsrv::Path::normalize_path('/foo?bar')[1])
      assert_equal('/foo bar', WWWsrv::Path::normalize_path('/foo+bar')[0])
      assert_equal('/foo/~bar', WWWsrv::Path::normalize_path('/foo/%7Ebar')[0])
    end
  end
end

if ($0 == __FILE__) then
  if (ARGV.empty?) then
    TestWWWsrv::constants.map{ |name|
      TestWWWsrv::const_get(name)
    }.select{ |const|
      (const.kind_of? Class) && (const < RUNIT::TestCase)
    }.each do |test_case|
      RUNIT::CUI::TestRunner::run(test_case::suite)
    end
  else
    for class_name in ARGV
      test_case = TestWWWsrv.const_get("Test#{class_name}")
      RUNIT::CUI::TestRunner.run(test_case.suite)
    end
  end
end
