#!/usr/local/bin/ruby
# $Id: test_process.rb,v 1.4.2.1 2003/01/05 13:06:43 toki Exp $
# -
# Test of WWWsrv::ProtectProcess module

require 'wwwsrv/process'
require 'runit/testcase'
require 'runit/cui/testrunner'

module TestWWWsrv
  class TestProtectProcess < RUNIT::TestCase
    def test_protect_environ
      tmp_env = [
	[ 'MESSAGE', 'Hello world' ],
	[ 'DEBUG', 'True' ]
      ]
      path = ENV['PATH']
      WWWsrv::ProtectProcess::protect_environ(tmp_env) do
	for name, value in ENV
	  if (name == 'PATH') then
	    assert_equal(path, value)
	  else
	    assert_not_nil(env_pair = tmp_env.assoc(name))
	    assert_equal(env_pair.last, value)
	  end
	end
      end
    end

    def test_protect_environ_not_preserved
      tmp_env = [
	[ 'MESSAGE', 'Hello world' ],
	[ 'DEBUG', 'True' ]
      ]
      WWWsrv::ProtectProcess::protect_environ(tmp_env, nil) do
	for name, value in ENV
	  assert_not_nil(env_pair = tmp_env.assoc(name))
	  assert_equal(env_pair.last, value)
	end
      end
    end

    def test_protect_chdir
      curr_work_dir = Dir::getwd
      root = File::expand_path('/')
      WWWsrv::ProtectProcess::protect_chdir(root) do
	assert_equal(root, Dir::getwd)
      end
      assert_equal(curr_work_dir, Dir::getwd)
    end
  end
end

if ($0 == __FILE__) then
  if (ARGV.empty?) then
    TestWWWsrv::constants.map{ |name|
      TestWWWsrv::const_get(name)
    }.select{ |const|
      (const.kind_of? Class) && (const < RUNIT::TestCase)
    }.each do |test_case|
      RUNIT::CUI::TestRunner::run(test_case::suite)
    end
  else
    for class_name in ARGV
      test_case = TestWWWsrv.const_get("Test#{class_name}")
      RUNIT::CUI::TestRunner.run(test_case.suite)
    end
  end
end
