#!/usr/local/bin/ruby
# $Id: test_ssi.rb,v 1.11.2.1 2003/01/05 13:06:49 toki Exp $
# -
# Test of WWWsrv::SSIdocument class

require 'docutils'
require 'pseudo_io'
require 'pseudo_doc'
require 'wwwsrv/logger'
require 'wwwsrv/request'
require 'wwwsrv/response'
require 'wwwsrv/local'
require 'wwwsrv/ssi'
require 'runit/testcase'
require 'runit/cui/testrunner'

module TestWWWsrv
  class TestSSIdocument < RUNIT::TestCase
    include DocumentUtils

    def setup
      @root = WWWsrv::DocumentFolder::new
      @local_file = WWWsrv::LocalFileDocument::new(Dir::getwd)
      @root.mount(@local_file, '/')
      @ssi = WWWsrv::SSIdocument::new(@local_file)
      @ssi_path = 'test.shtml'
      @root.mount(@ssi, '/', :page => @ssi_path)
    end

    def create_file(path, content)
      File::open(path, 'w') do |output|
	output.binmode
	output.write(content)
      end
    end
    private :create_file

    def test_respond_include_file
      create_file(@ssi_path, <<-'EOF')
<!-- #include file="hello.txt" -->
      EOF
      create_file('hello.txt', "Hello world.\n")
      begin
	srv_opt, log_output = make_srv_opt
	srv_opt.root = @root
	request, body_input = make_request('GET', '/' + @ssi_path)
	response = make_response
	respondent = try_to_respond(@root, srv_opt, '', request, response)
	pattern = Regexp::new(Regexp::quote("Hello world.\n"))
	assert_match(pattern, respondent[:body])
      ensure
	File::unlink(@ssi_path)
	File::unlink('hello.txt')
      end
    end

    def test_respond_include_virtual
      create_file(@ssi_path, <<-'EOF')
<!-- #include virtual="hello.txt" -->
      EOF
      create_file('hello.txt', "Hello world.\n")
      begin
	srv_opt, log_output = make_srv_opt
	srv_opt.root = @root
	request, body_input = make_request('GET', '/' + @ssi_path)
	response = make_response
	respondent = try_to_respond(@root, srv_opt, '', request, response)
	pattern = Regexp::new(Regexp::quote("Hello world.\n"))
	assert_match(pattern, respondent[:body])
      ensure
	File::unlink(@ssi_path)
	File::unlink('hello.txt')
      end
    end

    def test_respond_echo_var
      create_file(@ssi_path, <<-'EOF')
<!-- #echo var="PATH" -->
      EOF
      begin
	srv_opt, log_output = make_srv_opt
	srv_opt.root = @root
	request, body_input = make_request('GET', '/' + @ssi_path)
	response = make_response
	respondent = try_to_respond(@root, srv_opt, '', request, response)
	pattern = Regexp::new(Regexp::quote(ENV['PATH']))
	assert_match(pattern, respondent[:body])
      ensure
	File::unlink(@ssi_path)
      end

      create_file(@ssi_path, <<-'EOF')
<!-- #echo var="DOCUMENT_NAME" -->
      EOF
      begin
	srv_opt, log_output = make_srv_opt
	srv_opt.root = @root
	request, body_input = make_request('GET', '/' + @ssi_path)
	response = make_response
	respondent = try_to_respond(@root, srv_opt, '', request, response)
	pattern = Regexp::new(Regexp::quote(@ssi_path))
	assert_match(pattern, respondent[:body])
      ensure
	File::unlink(@ssi_path)
      end

      create_file(@ssi_path, <<-'EOF')
<!-- #echo var="DOCUMENT_URI" -->
      EOF
      begin
	srv_opt, log_output = make_srv_opt
	srv_opt.root = @root
	request, body_input = make_request('GET', '/' + @ssi_path)
	response = make_response
	respondent = try_to_respond(@root, srv_opt, '', request, response)
	pattern = Regexp::new(Regexp::quote('/' + @ssi_path))
	assert_match(pattern, respondent[:body])
      ensure
	File::unlink(@ssi_path)
      end

      create_file(@ssi_path, <<-'EOF')
<!-- #echo var="LAST_MODIFIED" -->
      EOF
      begin
	srv_opt, log_output = make_srv_opt
	srv_opt.root = @root
	request, body_input = make_request('GET', '/' + @ssi_path)
	response = make_response
	respondent = try_to_respond(@root, srv_opt, '', request, response)
	mtime = File::stat(@ssi_path).mtime.gmtime
	pattern = Regexp::new(Regexp::quote(mtime.to_s))
	assert_match(pattern, respondent[:body])
      ensure
	File::unlink(@ssi_path)
      end

      create_file(@ssi_path, <<-'EOF')
<!-- #config timefmt="%y-%m-%d %H:%M:%S" -->
<!-- #echo var="LAST_MODIFIED" -->
      EOF
      begin
	srv_opt, log_output = make_srv_opt
	srv_opt.root = @root
	request, body_input = make_request('GET', '/' + @ssi_path)
	response = make_response
	respondent = try_to_respond(@root, srv_opt, '', request, response)
	mtime = File::stat(@ssi_path).mtime.gmtime
	pattern = Regexp::new(Regexp::quote(mtime.strftime("%y-%m-%d %H:%M:%S")))
	assert_match(pattern, respondent[:body])
      ensure
	File::unlink(@ssi_path)
      end

      create_file(@ssi_path, <<-'EOF')
<!-- #echo var="SERVER_SOFTWARE" -->
      EOF
      begin
	srv_opt, log_output = make_srv_opt
	srv_opt.root = @root
	request, body_input = make_request('GET', '/' + @ssi_path)
	response = make_response
	respondent = try_to_respond(@root, srv_opt, '', request, response)
	pattern = Regexp::new(Regexp::quote(WWWsrv::product_token))
	assert_match(pattern, respondent[:body])
      ensure
	File::unlink(@ssi_path)
      end
    end

    def test_respond_fsize_file
      create_file(@ssi_path, <<-'EOF')
<!-- #fsize file="test.dat" -->
      EOF
      File::open('test.dat', 'w') do |output|
	output.binmode
	output.seek(1023, IO::SEEK_SET)
	output.write('z')
      end
      begin
	srv_opt, log_output = make_srv_opt
	srv_opt.root = @root
	request, body_input = make_request('GET', '/' + @ssi_path)
	response = make_response
	respondent = try_to_respond(@root, srv_opt, '', request, response)
	pattern = Regexp::new(Regexp::quote('1024'))
	assert_match(pattern, respondent[:body])
      ensure
	File::unlink(@ssi_path)
	File::unlink('test.dat')
      end

      create_file(@ssi_path, <<-'EOF')
<!-- #config sizefmt="bytes" -->
<!-- #fsize file="test.dat" -->
      EOF
      File::open('test.dat', 'w') do |output|
	output.binmode
	output.seek(1023, IO::SEEK_SET)
	output.write('z')
      end
      begin
	srv_opt, log_output = make_srv_opt
	srv_opt.root = @root
	request, body_input = make_request('GET', '/' + @ssi_path)
	response = make_response
	respondent = try_to_respond(@root, srv_opt, '', request, response)
	pattern = Regexp::new(Regexp::quote('1024'))
	assert_match(pattern, respondent[:body])
      ensure
	File::unlink(@ssi_path)
	File::unlink('test.dat')
      end

      create_file(@ssi_path, <<-'EOF')
<!-- #config sizefmt="abbrev" -->
<!-- #fsize file="test.dat" -->
      EOF
      File::open('test.dat', 'w') do |output|
	output.binmode
	output.seek(127, IO::SEEK_SET)
	output.write('z')
      end
      begin
	srv_opt, log_output = make_srv_opt
	srv_opt.root = @root
	request, body_input = make_request('GET', '/' + @ssi_path)
	response = make_response
	respondent = try_to_respond(@root, srv_opt, '', request, response)
	pattern = Regexp::new(Regexp::quote('128'))
	assert_match(pattern, respondent[:body])
      ensure
	File::unlink(@ssi_path)
	File::unlink('test.dat')
      end

      create_file(@ssi_path, <<-'EOF')
<!-- #config sizefmt="abbrev" -->
<!-- #fsize file="test.dat" -->
      EOF
      File::open('test.dat', 'w') do |output|
	output.binmode
	output.seek(1024, IO::SEEK_SET)
	output.write('z')
      end
      begin
	srv_opt, log_output = make_srv_opt
	srv_opt.root = @root
	request, body_input = make_request('GET', '/' + @ssi_path)
	response = make_response
	respondent = try_to_respond(@root, srv_opt, '', request, response)
	pattern = Regexp::new(Regexp::quote('1'))
	assert_match(pattern, respondent[:body])
      ensure
	File::unlink(@ssi_path)
	File::unlink('test.dat')
      end

      create_file(@ssi_path, <<-'EOF')
<!-- #config sizefmt="abbrev" -->
<!-- #fsize file="test.dat" -->
      EOF
      File::open('test.dat', 'w') do |output|
	output.binmode
	output.seek(1024**2, IO::SEEK_SET)
	output.write('z')
      end
      begin
	srv_opt, log_output = make_srv_opt
	srv_opt.root = @root
	request, body_input = make_request('GET', '/' + @ssi_path)
	response = make_response
	respondent = try_to_respond(@root, srv_opt, '', request, response)
	pattern = Regexp::new(Regexp::quote('1'))
	assert_match(pattern, respondent[:body])
      ensure
	File::unlink(@ssi_path)
	File::unlink('test.dat')
      end
    end

    def test_respond_fsize_virtual
      create_file(@ssi_path, <<-'EOF')
<!-- #fsize virtual="test.dat" -->
      EOF
      File::open('test.dat', 'w') do |output|
	output.binmode
	output.seek(1023, IO::SEEK_SET)
	output.write('z')
      end
      begin
	srv_opt, log_output = make_srv_opt
	srv_opt.root = @root
	request, body_input = make_request('GET', '/' + @ssi_path)
	response = make_response
	respondent = try_to_respond(@root, srv_opt, '', request, response)
	pattern = Regexp::new(Regexp::quote('1024'))
	assert_match(pattern, respondent[:body])
      ensure
	File::unlink(@ssi_path)
	File::unlink('test.dat')
      end

      create_file(@ssi_path, <<-'EOF')
<!-- #config sizefmt="bytes" -->
<!-- #fsize virtual="test.dat" -->
      EOF
      File::open('test.dat', 'w') do |output|
	output.binmode
	output.seek(1023, IO::SEEK_SET)
	output.write('z')
      end
      begin
	srv_opt, log_output = make_srv_opt
	srv_opt.root = @root
	request, body_input = make_request('GET', '/' + @ssi_path)
	response = make_response
	respondent = try_to_respond(@root, srv_opt, '', request, response)
	pattern = Regexp::new(Regexp::quote('1024'))
	assert_match(pattern, respondent[:body])
      ensure
	File::unlink(@ssi_path)
	File::unlink('test.dat')
      end

      create_file(@ssi_path, <<-'EOF')
<!-- #config sizefmt="abbrev" -->
<!-- #fsize virtual="test.dat" -->
      EOF
      File::open('test.dat', 'w') do |output|
	output.binmode
	output.seek(127, IO::SEEK_SET)
	output.write('z')
      end
      begin
	srv_opt, log_output = make_srv_opt
	srv_opt.root = @root
	request, body_input = make_request('GET', '/' + @ssi_path)
	response = make_response
	respondent = try_to_respond(@root, srv_opt, '', request, response)
	pattern = Regexp::new(Regexp::quote('128'))
	assert_match(pattern, respondent[:body])
      ensure
	File::unlink(@ssi_path)
	File::unlink('test.dat')
      end

      create_file(@ssi_path, <<-'EOF')
<!-- #config sizefmt="abbrev" -->
<!-- #fsize virtual="test.dat" -->
      EOF
      File::open('test.dat', 'w') do |output|
	output.binmode
	output.seek(1024, IO::SEEK_SET)
	output.write('z')
      end
      begin
	srv_opt, log_output = make_srv_opt
	srv_opt.root = @root
	request, body_input = make_request('GET', '/' + @ssi_path)
	response = make_response
	respondent = try_to_respond(@root, srv_opt, '', request, response)
	pattern = Regexp::new(Regexp::quote('1'))
	assert_match(pattern, respondent[:body])
      ensure
	File::unlink(@ssi_path)
	File::unlink('test.dat')
      end

      create_file(@ssi_path, <<-'EOF')
<!-- #config sizefmt="abbrev" -->
<!-- #fsize virtual="test.dat" -->
      EOF
      File::open('test.dat', 'w') do |output|
	output.binmode
	output.seek(1024**2, IO::SEEK_SET)
	output.write('z')
      end
      begin
	srv_opt, log_output = make_srv_opt
	srv_opt.root = @root
	request, body_input = make_request('GET', '/' + @ssi_path)
	response = make_response
	respondent = try_to_respond(@root, srv_opt, '', request, response)
	pattern = Regexp::new(Regexp::quote('1'))
	assert_match(pattern, respondent[:body])
      ensure
	File::unlink(@ssi_path)
	File::unlink('test.dat')
      end
    end

    def test_respond_flastmod_file
      create_file(@ssi_path, <<-'EOF')
<!-- #flastmod file="hello.txt" -->
      EOF
      create_file('hello.txt', "Hello world.\n")
      begin
	srv_opt, log_output = make_srv_opt
	srv_opt.root = @root
	request, body_input = make_request('GET', '/' + @ssi_path)
	response = make_response
	respondent = try_to_respond(@root, srv_opt, '', request, response)
	mtime = File::stat('hello.txt').mtime.gmtime
	pattern = Regexp::new(Regexp::quote(mtime.to_s))
	assert_match(pattern, respondent[:body])
      ensure
	File::unlink(@ssi_path)
	File::unlink('hello.txt')
      end

      create_file(@ssi_path, <<-'EOF')
<!-- #config timefmt="%y-%m-%d %H:%M:%S" -->
<!-- #flastmod file="hello.txt" -->
      EOF
      create_file('hello.txt', "Hello world.\n")
      begin
	srv_opt, log_output = make_srv_opt
	srv_opt.root = @root
	request, body_input = make_request('GET', '/' + @ssi_path)
	response = make_response
	respondent = try_to_respond(@root, srv_opt, '', request, response)
	mtime = File::stat('hello.txt').mtime.gmtime
	pattern = Regexp::new(Regexp::quote(mtime.strftime("%y-%m-%d %H:%M:%S")))
	assert_match(pattern, respondent[:body])
      ensure
	File::unlink(@ssi_path)
	File::unlink('hello.txt')
      end
    end

    def test_respond_flastmod_virtual
      create_file(@ssi_path, <<-'EOF')
<!-- #flastmod virtual="hello.txt" -->
      EOF
      create_file('hello.txt', "Hello world.\n")
      begin
	srv_opt, log_output = make_srv_opt
	srv_opt.root = @root
	request, body_input = make_request('GET', '/' + @ssi_path)
	response = make_response
	respondent = try_to_respond(@root, srv_opt, '', request, response)
	mtime = File::stat('hello.txt').mtime.gmtime
	pattern = Regexp::new(Regexp::quote(mtime.to_s))
	assert_match(pattern, respondent[:body])
      ensure
	File::unlink(@ssi_path)
	File::unlink('hello.txt')
      end

      create_file(@ssi_path, <<-'EOF')
<!-- #config timefmt="%y-%m-%d %H:%M:%S" -->
<!-- #flastmod virtual="hello.txt" -->
      EOF
      create_file('hello.txt', "Hello world.\n")
      begin
	srv_opt, log_output = make_srv_opt
	srv_opt.root = @root
	request, body_input = make_request('GET', '/' + @ssi_path)
	response = make_response
	respondent = try_to_respond(@root, srv_opt, '', request, response)
	mtime = File::stat('hello.txt').mtime.gmtime
	pattern = Regexp::new(Regexp::quote(mtime.strftime("%y-%m-%d %H:%M:%S")))
	assert_match(pattern, respondent[:body])
      ensure
	File::unlink(@ssi_path)
	File::unlink('hello.txt')
      end
    end

    def test_respond_exec_cmd
      create_file(@ssi_path, <<-'EOF')
<!-- #exec cmd="ruby -e 'puts %q{Hello world.}'" -->
      EOF
      begin
	srv_opt, log_output = make_srv_opt
	srv_opt.root = @root
	request, body_input = make_request('GET', '/' + @ssi_path)
	response = make_response
	respondent = try_to_respond(@root, srv_opt, '', request, response)
	pattern = Regexp::new(Regexp::quote(`ruby -e 'puts %q{Hello world.}'`))
	assert_match(pattern, respondent[:body])
      ensure
	File::unlink(@ssi_path)
      end
    end

    def test_respond_exec_cgi
      create_file(@ssi_path, <<-'EOF')
<!-- #exec cgi="hello.cgi" -->
      EOF
      create_file('hello.cgi', <<-'EOF')
#!/usr/local/bin/ruby
STDOUT.binmode
msg = "Hello world.\n"
print "Content-Type: text/plain\r\n"
print "Content-Length: #{msg.length}\r\n"
print "\r\n"
print msg
      EOF
      cgi = WWWsrv::CGIdocument::new(Dir::getwd, false, ENV['RUBY'] || 'ruby')
      @root.mount(cgi, '/', :page => 'hello.cgi')
      begin
	srv_opt, log_output = make_srv_opt
	srv_opt.root = @root
	request, body_input = make_request('GET', '/' + @ssi_path)
	response = make_response
	respondent = try_to_respond(@root, srv_opt, '', request, response)
	pattern = Regexp::new(Regexp::quote("Hello world.\n"))
	assert_match(pattern, respondent[:body])
      ensure
	File::unlink(@ssi_path)
	File::unlink('hello.cgi')
      end
    end
  end
end

if ($0 == __FILE__) then
  if (ARGV.empty?) then
    TestWWWsrv::constants.map{ |name|
      TestWWWsrv::const_get(name)
    }.select{ |const|
      (const.kind_of? Class) && (const < RUNIT::TestCase)
    }.each do |test_case|
      RUNIT::CUI::TestRunner::run(test_case::suite)
    end
  else
    for class_name in ARGV
      test_case = TestWWWsrv.const_get("Test#{class_name}")
      RUNIT::CUI::TestRunner.run(test_case.suite)
    end
  end
end
