#!/usr/local/bin/ruby
# $Id: test_token.rb,v 1.5.4.1 2003/01/05 13:06:50 toki Exp $
# -
# Test of WWWsrv::Token module

require 'wwwsrv/token'
require 'runit/testcase'
require 'runit/cui/testrunner'

module TestWWWsrv
  class TestToken < RUNIT::TestCase
    def test_fetch_separator
      %w! ( ) < > @ , ; : \\ " / [ ] ? = { } !.each do |sep|
	value = "#{sep}hello"
	assert_equal(sep, WWWsrv::Token::fetch_separator(value))
	assert_equal("#{sep}hello", value)

	value = " #{sep}hello"
	assert_equal(sep, WWWsrv::Token::fetch_separator(value))
	assert_equal(" #{sep}hello", value)
      end

      value = 'hello'
      assert_nil(WWWsrv::Token::fetch_separator(value))
      assert_equal('hello', value)

      value = ' hello'
      assert_nil(WWWsrv::Token::fetch_separator(value))
      assert_equal(' hello', value)
    end

    def test_fetch_separator_destructive
      %w! ( ) < > @ , ; : \\ " / [ ] ? = { } !.each do |sep|
	value = "#{sep}hello"
	assert_equal(sep, WWWsrv::Token::fetch_separator(value, true))
	assert_equal("hello", value)

	value = " #{sep}hello"
	assert_equal(sep, WWWsrv::Token::fetch_separator(value, true))
	assert_equal("hello", value)
      end

      value = 'hello'
      assert_nil(WWWsrv::Token::fetch_separator(value, true))
      assert_equal('hello', value)

      value = ' hello'
      assert_nil(WWWsrv::Token::fetch_separator(value, true))
      assert_equal('hello', value)
    end

    def test_fetch_token
      value = 'Hello ruby world'
      assert_equal('Hello', WWWsrv::Token::fetch_token(value))
      assert_equal('Hello ruby world', value)

      %w! ( ) < > @ , ; : \\ " / [ ] ? = { } !.each do |sep|
	value = "hello#{sep}"
	assert_equal('hello', WWWsrv::Token::fetch_token(value))
	assert_equal("hello#{sep}", value)
      end

      value = ' '
      assert_nil(WWWsrv::Token::fetch_token(value))
      assert_equal(' ', value)
    end

    def test_fetch_token_destructive
      value = 'Hello ruby world'
      tokens = %w[ Hello ruby world ]
      while (token = WWWsrv::Token::fetch_token(value, true))
	assert_equal(tokens.shift, token)
      end
      assert(value.empty?)
      assert(tokens.empty?)

      %w! ( ) < > @ , ; : \\ " / [ ] ? = { } !.each do |sep|
	value = "hello#{sep}"
	assert_equal('hello', WWWsrv::Token::fetch_token(value, true))
	assert_equal(sep, value)
      end

      value = ' '
      assert_nil(WWWsrv::Token::fetch_token(value, true))
      assert(value.empty?)
    end

    def test_fetch_quoted
      value = '"Hello world."'
      assert_equal('Hello world.', WWWsrv::Token::fetch_quoted(value))
      assert_equal('"Hello world."', value)

      value = ' "Hello world."'
      assert_equal('Hello world.', WWWsrv::Token::fetch_quoted(value))
      assert_equal(' "Hello world."', value)

      value = '"Hello world." '
      assert_equal('Hello world.', WWWsrv::Token::fetch_quoted(value))
      assert_equal('"Hello world." ', value)

      value = '"\"Hello world.\"'
      assert_equal('"Hello world."', WWWsrv::Token::fetch_quoted(value))
      assert_equal('"\"Hello world.\"', value)

      value = '"Hello world.'
      assert_equal('Hello world.', WWWsrv::Token::fetch_quoted(value))
      assert_equal('"Hello world.', value)

      value = 'Hello world."'
      assert_nil(WWWsrv::Token::fetch_quoted(value))
      assert_equal('Hello world."', value)

      value = ' Hello world.'
      assert_nil(WWWsrv::Token::fetch_quoted(value))
      assert_equal(' Hello world.', value)
    end

    def test_fetch_quoted_destructive
      value = '"Hello world."'
      assert_equal('Hello world.', WWWsrv::Token::fetch_quoted(value, true))
      assert(value.empty?)

      value = ' "Hello world."'
      assert_equal('Hello world.', WWWsrv::Token::fetch_quoted(value, true))
      assert(value.empty?)

      value = '"Hello world." '
      assert_equal('Hello world.', WWWsrv::Token::fetch_quoted(value, true))
      assert_equal(' ', value)

      value = '"\"Hello world.\"'
      assert_equal('"Hello world."', WWWsrv::Token::fetch_quoted(value, true))
      assert(value.empty?)

      value = '"Hello world.'
      assert_equal('Hello world.', WWWsrv::Token::fetch_quoted(value, true))
      assert(value.empty?)

      value = 'Hello world."'
      assert_nil(WWWsrv::Token::fetch_quoted(value, true))
      assert_equal('Hello world."', value)

      value = ' Hello world.'
      assert_nil(WWWsrv::Token::fetch_quoted(value, true))
      assert_equal('Hello world.', value)
    end

    def test_fetch_comment
      value = '(Hello world.)'
      assert_equal('Hello world.', WWWsrv::Token::fetch_comment(value))
      assert_equal('(Hello world.)', value)

      value = ' (Hello world.)'
      assert_equal('Hello world.', WWWsrv::Token::fetch_comment(value))
      assert_equal(' (Hello world.)', value)

      value = '(Hello world.) '
      assert_equal('Hello world.', WWWsrv::Token::fetch_comment(value))
      assert_equal('(Hello world.) ', value)

      value = '(\(Hello world.\))'
      assert_equal('(Hello world.)', WWWsrv::Token::fetch_comment(value))
      assert_equal('(\(Hello world.\))', value)

      value = '(Hello world.'
      assert_equal('Hello world.', WWWsrv::Token::fetch_comment(value))
      assert_equal('(Hello world.', value)

      value = 'Hello world.)'
      assert_nil(WWWsrv::Token::fetch_comment(value))
      assert_equal('Hello world.)', value)

      value = '(Hello (world.))'
      assert_equal('Hello (world.)', WWWsrv::Token::fetch_comment(value))
      assert_equal('(Hello (world.))', value)

      value = '(Hello ((world).'
      assert_equal('Hello ((world).', WWWsrv::Token::fetch_comment(value))
      assert_equal('(Hello ((world).', value)
    end

    def test_fetch_comment_destructive
      value = '(Hello world.)'
      assert_equal('Hello world.', WWWsrv::Token::fetch_comment(value, true))
      assert(value.empty?)

      value = ' (Hello world.)'
      assert_equal('Hello world.', WWWsrv::Token::fetch_comment(value, true))
      assert(value.empty?)

      value = '(Hello world.) '
      assert_equal('Hello world.', WWWsrv::Token::fetch_comment(value, true))
      assert_equal(' ', value)

      value = '(\(Hello world.\))'
      assert_equal('(Hello world.)', WWWsrv::Token::fetch_comment(value, true))
      assert(value.empty?)

      value = '(Hello world.'
      assert_equal('Hello world.', WWWsrv::Token::fetch_comment(value, true))
      assert(value.empty?)

      value = 'Hello world.)'
      assert_nil(WWWsrv::Token::fetch_comment(value, true))
      assert_equal('Hello world.)', value)

      value = '(Hello (world.))'
      assert_equal('Hello (world.)', WWWsrv::Token::fetch_comment(value, true))
      assert(value.empty?)

      value = '(Hello ((world).'
      assert_equal('Hello ((world).', WWWsrv::Token::fetch_comment(value, true))
      assert(value.empty?)
    end

    def test_quote_token
      assert_equal('hello', WWWsrv::Token::quote_token('hello'))
      assert_equal('"Hello world."', WWWsrv::Token::quote_token('Hello world.'))
      assert_equal('"\"Hello\\\\ world.\""', WWWsrv::Token::quote_token('"Hello\ world."'))
      value = '"Hello\ world."'
      assert_equal(value, WWWsrv::Token::fetch_quoted(WWWsrv::Token::quote_token(value)))
    end
  end
end

if ($0 == __FILE__) then
  if (ARGV.empty?) then
    TestWWWsrv::constants.map{ |name|
      TestWWWsrv::const_get(name)
    }.select{ |const|
      (const.kind_of? Class) && (const < RUNIT::TestCase)
    }.each do |test_case|
      RUNIT::CUI::TestRunner::run(test_case::suite)
    end
  else
    for class_name in ARGV
      test_case = TestWWWsrv.const_get("Test#{class_name}")
      RUNIT::CUI::TestRunner.run(test_case.suite)
    end
  end
end
