# $Id: auth.rb,v 1.32 2002/11/26 15:44:46 toki Exp $
# -
# HTTP authentication

=begin
Copyright (c) 2001-2002
	TOKI Yoshinori <toki@freedom.ne.jp>. All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  1.  Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.

  2.  Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

  3.  The name of the author may not be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
=end

require 'wwwsrv/const'
require 'wwwsrv/token'
require 'wwwsrv/htmlconv'
require 'wwwsrv/document'

module WWWsrv
  class BasicAuthentication < DocumentModifier
    include Token
    include ExportResponse

    DENIAL_PAGE = Document::new

    def DENIAL_PAGE.respond(srv_opt, prefix, request, response)
      denial_auth_user = request[:denial_auth_user]
      denial_auth_type = request[:denial_auth_type]
      yield(response)
      yield(<<-EOF)
<html>
<head><title>Failure of Basic Authentication</title></head>
<body>
<h1>Failure of Basic Authentication</h1>
<p>Your user id #{
  if (denial_auth_user) then
    'of <kbd>' + HTMLconv::quoteHTML(denial_auth_user) + '</kbd> '
  end
}is denied.</p>
</body>
</html>
      EOF

      nil
    end

    def initialize(document, realm, passwd, denial_page=DENIAL_PAGE)
      super(document)
      @realm = realm
      @passwd = passwd
      @denial_page = ResponseFilter::new(denial_page) { |srv_opt, prefix, request, response|
	case (response.status)
	when 200...300
	  # unauthorized page with status code of 401.
	  response.status = 401
	  response.erase_cond_header_fields
	else
	  raise "failed to get a authentication denial page from #{denial_page.inspect}"
	end
      }
    end

    def warm_up(log)
      super
      log.debug("#{self}: denial page -> #{@denial_page}")
      nil
    end

    def decode(credentials)
      if (credentials =~ /^Basic\s+/i) then
	base64_user_pass = credentials.split(/\s+/, 2).last
	user_pass = base64_user_pass.unpack('m').first
	userid, password = user_pass.split(/:/, 2)

	return userid, password
      end

      nil
    end
    private :decode

    def authenticate?(userid, password)
      if ((userid && ! userid.empty?) && (password && ! password.empty?)) then
	if (@passwd[userid] == password) then
	  return true
	end
      end

      false
    end
    private :authenticate?

    def deny_auth(srv_opt, prefix, request, response, userid=nil)
      srv_opt.srv_log_system('WWW Basic Authentication: deny ' +
			     ((! prefix.empty?) ? prefix : '/') + ": #{@realm}: #{request.client}")
      request[:denial_auth_type] = 'Basic'
      request[:denial_auth_user] = userid
      export_response(@denial_page, srv_opt, prefix, request, response) do |arg, opts|
	yield(arg, opts)
      end

      terminate
    end
    private :deny_auth

    def modify_request(srv_opt, context, prefix, request, response)
      response.header.www_authenticate = 'Basic realm=' + quote_token(@realm, true)
      if (credentials = request.header.authorization) then
	userid, password = decode(credentials)
	srv_opt.doc_log_internal(self, "userid #{userid.inspect}: password #{password.inspect}")
	if (authenticate? userid, password) then
	  request[:auth_type] = 'Basic'
	  request[:auth_user] = userid
	  srv_opt.srv_log_system('WWW Basic Authentication: allow ' +
				 ((! prefix.empty?) ? prefix : '/') + ": #{@realm}: #{request.client}")
	else
	  deny_auth(srv_opt, prefix, request, response, userid) do |arg, opts|
	    yield(arg, opts)
	  end
	end
      else
	deny_auth(srv_opt, prefix, request, response) do |arg, opts|
	  yield(arg, opts)
	end
      end

      nil
    end

    def internal_documents
      super + [ @denial_page ]
    end
  end

  # alias
  BasicAuth = BasicAuthentication
end

require 'wwwsrv/auth_obs' unless $WWWSRV_NOT_COMPAT
