# $Id: const.rb,v 1.20.2.4 2003/04/11 14:08:59 toki Exp $
# -
# constants

=begin
Copyright (c) 2001-2002
	TOKI Yoshinori <toki@freedom.ne.jp>. All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  1.  Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.

  2.  Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

  3.  The name of the author may not be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
=end

require 'parsedate'
require 'timeout'

unless (defined? $WWWSRV_NOT_COMPAT) then
  $WWWSRV_NOT_COMPAT = false
end

module WWWsrv
  # latest CVS commit date of core libraries for wwwsrv
  @@rcs_date = nil

  def self::update(rcs_timestamp)
    year, month, day = ParseDate::parsedate(rcs_timestamp)[0, 3]
    if (year && month && day) then
      rcs_date = format('%04d-%02d-%02d', year, month, day)
      if (! @@rcs_date || @@rcs_date < rcs_date) then
	@@rcs_date = rcs_date
      end
    end

    @@rcs_date
  end

  def self::rcs_date
    @@rcs_date
  end

  update(%q$Date: 2003/04/11 14:08:59 $)

  # Version
  SRVNAME = 'WWWsrv'.freeze
  VERSION = '0.15.3'.freeze
  CODENAME = 'Badaxe'.freeze

  def self::product_token
    "#{SRVNAME}/#{VERSION} (#{CODENAME}; #{@@rcs_date}) " +
      "Ruby/#{RUBY_VERSION} (#{RUBY_PLATFORM}; #{RUBY_RELEASE_DATE})"
  end

  # Line Separator
  CR = "\r".freeze
  LF = "\n".freeze
  CRLF = (CR + LF).freeze

  # HTTP/1.1 methods
  HTTP_OPTIONS = 'OPTIONS'.freeze
  HTTP_GET = 'GET'.freeze
  HTTP_HEAD = 'HEAD'.freeze
  HTTP_POST = 'POST'.freeze
  HTTP_PUT = 'PUT'.freeze
  HTTP_DELETE = 'DELETE'.freeze
  HTTP_TRACE = 'TRACE'.freeze
  HTTP_CONNECT = 'CONNECT'.freeze

  # Default Server Parameters
  MAX_CONNECTIONS = 64
  MAX_REQUESTS = 64
  TIMEOUT = 300
  KEEP_ALIVE = 15
  BUFFER_SIZE = 1024 * 32
  HTTP_SCHEME = 'http'.freeze

  # Basic Error
  class ParseError < StandardError; end
  class ConnectionError < StandardError; end
  TimeoutError = ::TimeoutError
end
