# $Id: delegate.rb,v 1.7.2.1 2002/12/27 15:18:09 toki Exp $
# -
# delegation support module

=begin
Copyright (c) 2001-2002
	TOKI Yoshinori <toki@freedom.ne.jp>. All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  1.  Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.

  2.  Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

  3.  The name of the author may not be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
=end

require 'wwwsrv/const'

class Object
  def __delegatable__?(method_id)
    respond_to? method_id
  end
end

module WWWsrv
  # latest CVS commit date of core libraries for wwwsrv
  update(%q$Date: 2002/12/27 15:18:09 $)

  module Delegatable
    @delegate_module = Module::new
    @delegate_module.module_eval <<-'EOF'
      def delegate_if(klass, src_obj=:__src_obj__, wrap_method=nil)
	# Note: self is a module of delegating to src_obj.

	self.module_eval "
          def __delegatable__?(method_id)
	    super or (#{src_obj}.__delegatable__? method_id)
          end
        "

	method_list = klass.instance_methods(true)
	method_list -= self.instance_methods(true)
        method_list = method_list.find_all{ |method| yield(method) }

	for deleg_method in method_list
          if (deleg_method =~ /=$/) then
	    outer_args = 'arg'
	    prior_wrap = wrap_method ? "#{wrap_method}(:#{deleg_method}, arg) do |arg2|" : ''
	    inner_args = wrap_method ? 'arg2' : 'arg'
	    after_wrap = wrap_method ? 'end' : ''
          else
	    outer_args = '*args, &block'
	    prior_wrap = wrap_method ? "#{wrap_method}(:#{deleg_method}, *args) do |args2|" : ''
	    inner_args = wrap_method ? '*args2, &block' : '*args, &block'
	    after_wrap = wrap_method ? 'end' : ''
	  end
	  self.module_eval "
	    def #{deleg_method}(#{outer_args})
	      #{prior_wrap}
	      ret_val = #{src_obj}.#{deleg_method}(#{inner_args})
	      (ret_val.__id__ == #{src_obj}.__id__) ? self : ret_val
	      #{after_wrap}
	    end
	  "
	end

        if (wrap_method) then
          self.module_eval "
	    def method_missing(id, *args, &block)
	      if (#{src_obj}.__delegatable__? id) then
		#{wrap_method}(id, *args) { |*args2|
		  ret_val = #{src_obj}.__send__(id, *args2, &block)
	          (ret_val.__id__ == #{src_obj}.__id__) ? self : ret_val
		}
	      else
	        super
	      end
	    end
          "
        else
          self.module_eval "
	    def method_missing(id, *args, &block)
	      if (#{src_obj}.__delegatable__? id) then
		ret_val = #{src_obj}.__send__(id, *args, &block)
		(ret_val.__id__ == #{src_obj}.__id__) ? self : ret_val
	      else
		super
	      end
	    end
          "
        end

	nil
      end

      def delegate(*args)
        delegate_if(*args) { |method| true }
      end
    EOF

    def self.append_features(other_module)
      other_module.extend(@delegate_module)
      super(other_module)
    end
  end
end
