# $Id: erbdoc.rb,v 1.34.2.2 2003/04/11 14:09:01 toki Exp $
# -
# eRuby document

=begin
Copyright (c) 2001-2002
	TOKI Yoshinori <toki@freedom.ne.jp>. All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  1.  Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.

  2.  Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

  3.  The name of the author may not be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
=end

require 'erb'
require 'wwwsrv/const'
require 'wwwsrv/document'

module WWWsrv
  module ERubyBind
    def eruby_bind(__srv_opt__, __request__, __response__, __env__)
      __module__ = Module::new
      __module__.const_set(:SRV_OPT,  __srv_opt__)
      __module__.const_set(:REQUEST,  __request__)
      __module__.const_set(:RESPONSE, __response__)
      __module__.const_set(:ENV,      __env__)

      unless ($WWWSRV_NOT_COMPAT) then
	# for backward compatibility
	__sid__ = __srv_opt__.sid
	__module__.const_set(:SID, __sid__)
      end

      class << __module__
	def include(mod)
	  extend(mod)
	  super(mod)
	end
	private :include
      end

      __module__.module_eval %q{ binding }
    end
    module_function :eruby_bind
  end

  class ERubyDocument < Document
    include Path
    include ERubyBind
    include PassAuth

    def initialize(base_dir, safe_level=nil, allow_method=%w[ GET HEAD POST ])
      super()
      @base_dir = File::expand_path(base_dir)
      @allow = make_local_path_pattern(@base_dir)
      @safe_level = safe_level
      @allow_method = {}
      @path_info_root = nil
      for name in allow_method
	@allow_method[name] = true
      end
      @env = {}
    end

    attr_reader :path_info_root
    attr_reader :env
    alias environ env

    def path_info_root=(new_root)
      @path_info_root = File::expand_path(new_root)
    end

    def folder?
      true
    end

    def split_erb_path(prefix, request)
      request.scan_path_info(prefix) do |script_name, path_info|
	(script_name =~ %r"/$") and next
	erb_local_path = path_http2local(script_name, @base_dir)
	erb_local_path = File::expand_path(erb_local_path)
	if (erb_local_path !~ @allow) then
	  http_error(404, "Not found a document: #{request.normalized_path}.")
	end
	if (File::file? erb_local_path) then
	  path_translated = path_http2local(path_info)
	  if (@path_info_root && ! path_info.empty?) then
	    path_translated = merge_local_path(@path_info_root, path_translated)
	    path_translated = File::expand_path(path_translated)
	  end
	  return erb_local_path, merge_path(prefix, script_name), path_info, path_translated
	end
      end
      http_error(404, "Not found a document: #{request.normalized_path}.")
    end
    private :split_erb_path

    def respond(srv_opt, prefix, request, response)
      srv_opt.doc_log_debug(self, 'enter')
      erb_local_path, script_name, path_info, path_translated = split_erb_path(prefix, request)
      unless (@allow_method.include? request.method) then
	http_error(405, "Not allowed request method: #{request.method}.", 'Allow' => @allow_method.keys.join(', '))
      end
      srv_opt.doc_log_debug(self, "allow access: #{erb_local_path}")

      erb_env = fetch_cgi_var(request, @env.dup, script_name)
      unless (path_info.empty?) then
	erb_env['PATH_INFO'] = path_info
	erb_env['PATH_TRANSLATED'] = path_translated
      end
      response[:local_path] = erb_local_path
      response.header.content_type = 'text/html'

      erb = ERB::new(File::open(erb_local_path) { |input| input.read }, @safe_level)
      content_body = erb.result(eruby_bind(srv_opt, request, response, erb_env))
      response.header.content_length = content_body.length.to_s
      srv_opt.doc_log_debug(self, "eRuby evaluated: #{content_body.length} octets")

      yield(response)
      yield(content_body)

      nil
    end
  end

  class ERubyFilterDocument < DocumentModifier
    include ERubyBind
    include PassAuth

    def initialize(document, safe_level=nil)
      super(document)
      @safe_level = safe_level
      @env = {}
    end

    attr_reader :env
    alias environ env

    def modify_request(srv_opt, context, prefix, request, response)
      request.erase_cond_header_fields
      context[:request] = request

      nil
    end

    def modify_response(srv_opt, context, prefix, request, response)
      case (response.status)
      when 200, 201, 203
	context[:state] = :filter
	context[:response] = response
	context[:message_body] = ''
      else
	context[:state] = :through
      end

      nil
    end

    def modify_message_body(srv_opt, context, body_fragment, opts=nil)
      case (context[:state])
      when :filter
	context[:message_body].concat(body_fragment)
      when :through
	yield(body_fragment, opts)
      else
	raise "unknown internal state: #{context[:state].inspect}"
      end

      nil
    end

    def modify_terminal(srv_opt, context)
      case (context[:state])
      when :filter
	# continue
      when :through
	return
      else
	raise "unknown internal state: #{context[:state].inspect}"
      end

      request = context[:request]
      response = context[:response]
      message_body = context[:message_body]

      erb = ERB::new(message_body, @safe_level)
      content_body = erb.result(eruby_bind(srv_opt, request, response, fetch_cgi_var(request, @env.dup)))
      response.erase_sticky_entity_header_fields
      response.header.content_length = content_body.length.to_s
      srv_opt.doc_log_debug(self, "eRuby evaluated: #{content_body.length} octets")

      yield(response)
      yield(content_body)

      nil
    end
  end
end
