# $Id: filter.rb,v 1.15 2002/11/26 15:44:55 toki Exp $
# -
# line oriented filter

=begin
Copyright (c) 2001-2002
	TOKI Yoshinori <toki@freedom.ne.jp>. All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  1.  Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.

  2.  Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

  3.  The name of the author may not be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
=end

require 'wwwsrv/const'
require 'wwwsrv/document'

module WWWsrv
  class LineFilterDocument < DocumentModifier
    def initialize(document, eol=$/, &filter)
      super(document)
      @line_scan = Regexp::compile('^.*?' + Regexp::quote(eol))
      @filter = filter
    end

    def modify_request(srv_opt, context, prefix, request, response)
      request.erase_cond_header_fields
      nil
    end

    def modify_response(srv_opt, context, prefix, request, response)
      case (response.status)
      when 200, 201, 203
	context[:state] = :filter
	response.erase_sticky_entity_header_fields
      else
	context[:state] = :through
      end
      yield(response)

      nil
    end

    def modify_message_body(srv_opt, context, body_fragment, opts=nil)
      case (context[:state])
      when :filter
	if (body_fragment) then
	  if (last_fragment = context[:last_fragment]) then
	    body_fragment = last_fragment.concat(body_fragment)
	  else
	    body_fragment = body_fragment.dup
	  end

	  while (body_fragment.sub!(@line_scan, ''))
	    yield(@filter.call($&))
	  end

	  if (body_fragment && ! body_fragment.empty?) then
	    context[:last_fragment] = body_fragment
	  else
	    context[:last_fragment] = nil
	  end
	end
      when :through
	yield(body_fragment, opts)
      else
	raise "unknown internral state: #{context[:state].inspect}"
      end

      nil
    end

    def modify_terminal(srv_opt, context)
      case (context[:state])
      when :filter
	if (last_fragment = context[:last_fragment]) then
	  yield(@filter.call(last_fragment))
	end
      when :through
	# nothing to do.
      else
	raise "unknown internral state: #{context[:state].inspect}"
      end

      nil
    end
  end
end

require 'wwwsrv/filter_obs.rb' unless $WWWSRV_NOT_COMPAT
