# $Id: form.rb,v 1.13.2.2 2003/04/11 14:09:02 toki Exp $
# -
# methods of parsing form data

=begin
Copyright (c) 2001-2002
	TOKI Yoshinori <toki@freedom.ne.jp>. All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  1.  Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.

  2.  Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

  3.  The name of the author may not be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
=end

require 'wwwsrv/const'
require 'wwwsrv/token'
require 'wwwsrv/htmlconv'
require 'wwwsrv/multipart'

# for debug
module WWWsrv
  class Header
    def inspect
      self.to_a.sort.inspect
    end
  end
end

module WWWsrv
  # latest CVS commit date of core libraries for wwwsrv
  update(%q$Date: 2003/04/11 14:09:02 $)

  class FormDataParseError < MultipartParseError
  end

  class FormData
    include Enumerable

    def initialize
      @form_data = Hash.new
      # NOTE: data structure
      # {@form_data}
      #   |
      #   +-"name"->[data array]
      #               |
      #               +--pos-->{data hash}
      #                          |
      #                          +--:head--><WWWsrv::Header>
      #                          |
      #                          +--:body_list-->[body array]
      #                                            |
      #                                            +--pos-->{body hash}
      #                                                       |
      #                                                       +--:filename-->"filename"
      #                                                       |
      #                                                       +--:head--><WWWsrv::Header>
      #                                                       |
      #                                                       +--:body-->"body value"
    end

    def add_form_data(name, head)
      @form_data[name] = Array.new unless (@form_data.include? name)
      @form_data[name].push({ :head => head, :body_list => Array.new })
      nil
    end

    def add_form_data_body(name, head, body, filename=nil)
      form_data_body = { :head => head, :body => body, :filename => filename }
      @form_data[name][-1][:body_list].push(form_data_body)
      nil
    end

    def get_form_data(name, data_property, pos1=0)
      if (@form_data.include? name) then
	case (@form_data[name][pos1])
	when NilClass
	  return nil
	when Array
	  return @form_data[name][pos1].map{ |form_data|
	    form_data[data_property]
	  }
	else
	  return @form_data[name][pos1][data_property]
	end
      else
	return nil
      end
    end
    private :get_form_data

    def get_form_data_body(name, body_property, pos1=0, pos2=0)
      if (@form_data.include? name) then
	case (@form_data[name][pos1])
	when NilClass
	  return nil
	when Array
	  return @form_data[name][pos1].map{ |form_data|
	    case (form_data[:body_list][pos2])
	    when NilClass
	      nil
	    when Array
	      form_data[:body_list][pos2].map{ |form_data_body|
		form_data_body[body_property]
	      }
	    else
	      form_data[:body_list][pos2][body_property]
	    end
	  }
	else
	  case (@form_data[name][pos1][:body_list][pos2])
	  when NilClass
	    return nil
	  when Array
	    return @form_data[name][pos1][:body_list][pos2].map{ |form_data_body|
	      form_data_body[body_property]
	    }
	  else
	    return @form_data[name][pos1][:body_list][pos2][body_property]
	  end
	end
      else
	return nil
      end
    end
    private :get_form_data_body

    def [](name, pos1=0, pos2=0)
      get_form_data_body(name, :body, pos1, pos2)
    end

    def head(name, pos1=0)
      get_form_data(name, :head, pos1)
    end

    def body_head(name, pos1=0, pos2=0)
      get_form_data_body(name, :head, pos1, pos2)
    end

    def filename(name, pos1=0, pos2=0)
      get_form_data_body(name, :filename, pos1, pos2)
    end

    def count_name
      @form_data.length
    end

    alias length count_name
    alias size count_name

    def count_form_data(name)
      if (@form_data.include? name) then
	return @form_data[name].length
      else
	return 0
      end
    end

    def count_form_data_body(name, pos1=0)
      if (@form_data.include? name) then
	if (0 <= pos1 && pos1 < @form_data[name].length) then
	  return @form_data[name][pos1][:body_list].length
	else
	  return 0
	end
      else
	return 0
      end
    end

    def names
      @form_data.keys
    end

    alias keys names

    def empty?
      @form_data.empty?
    end

    def include?(name)
      @form_data.include? name
    end

    alias key? include?
    alias has_key? include?
    alias member? include?

    def each
      for name, form_data_list in @form_data
	for form_data in form_data_list
	  if (form_data[:body_list].length == 1) then
	    yield(name, form_data[:body_list][0][:body])
	  else
	    body_list = form_data[:body_list].map{ |form_data_body|
	      form_data_body[:body]
	    }
	    yield(name, body_list)
	  end
	end
      end

      nil
    end

    def self.parse_query(string, separator=/&/)
      HTMLconv::decode_query(string, separator) do |param_name, param_value|
	yield(:form_data_name, param_name)
	yield(:form_data_head, Header.build(
	  'Content-Disposition' => %Q'form-data; name=#{Token::quote_token(param_name)}'
	))
	yield(:form_data_body, param_value) if param_value
	yield(:form_data_end, nil)
      end

      nil
    end

    def self.scan_query(input, separator='&')
      while (param_segment = input.gets(separator))
	param_segment.sub!(/&$/, '')
	param_name, param_value = param_segment.split(/=/, 2)
	yield(:form_data_name, HTMLconv::decodeURL(param_name))
	yield(:form_data_head, Header.build(
	  'Content-Disposition' => %Q'form-data; name=#{Token::quote_token(param_name)}'
	))
	yield(:form_data_body, HTMLconv::decodeURL(param_value)) if param_value
	yield(:form_data_end, nil)
      end

      nil
    end

    def self.get_name(content_disposition)
      case (content_disposition)
      when /name="([^"]+)"/i
	name = $1
      when /name=([^\s]+)/i
	name = $1
      else
	raise FormDataParseError, "not found a form-data name: #{content_disposition.inspect}"
      end

      name
    end
    class << self; private :get_name; end

    def self.get_filename(content_disposition)
      case (content_disposition)
      when /filename="([^"]+)"/i
	filename = $1
      when /filename=([^\s]+)"/i
	filename = $1
      else
	filename = nil
      end

      filename
    end
    class << self; private :get_filename; end

    def self.scan_form_data(input, content_type)
      depth = 0
      Multipart.scan_multipart(input, content_type) do |tag, value|
	case (tag)
	when :begin_multipart
	  depth += 1
	  case (depth)
	  when 1
	    # nothing to do.
	  when 2
	    yield(:begin_mixed, nil)
	  else
	    raise FormDataParseError, "too deep or shallow nested multipart: #{depth}"
	  end
	when :part_head
	  head = value
	  case (depth)
	  when 1
	    if (head.content_disposition =~ /^form-data/i) then
	      name = get_name(head.content_disposition)
	      yield(:form_data_name, name)
	      if (filename = get_filename(head.content_disposition)) then
		yield(:form_data_file, filename)
	      end
	    else
	      raise "not a form data: #{head.content_disposition.inspect}"
	    end
	    yield(:form_data_head, head)
	  when 2
	    if (head.content_disposition =~ /^file/i) then
	      yield(:form_data_file, get_filename(head.content_disposition))
	    else
	      raise "not a file: #{head.content_disposition.inspect}"
	    end
	    yield(:form_data_head, head)
	  else
	    raise FormDataParseError, "too deep or shallow nested multipart: #{depth}"
	  end
	when :part_body
	  body_fragment = value
	  yield(:form_data_body, body_fragment)
	when :part_end
	  yield(:form_data_end, nil)
	when :end_multipart
	  case (depth)
	  when 1
	    # nothing to do.
	  when 2
	    yield(:end_mixed, nil)
	  else
	    raise FormDataParseError, "too deep or shallow nested multipart: #{depth}"
	  end
	  depth -= 1
	else
	  raise "unknown state: #{tag.inspect}"
	end
      end
      unless (depth == 0) then # assertion!
	raise FormDataParseError, "not matched multipart nest"
      end

      nil
    end
  end
end
