# $Id: htmlconv.rb,v 1.11.2.1 2003/01/05 13:05:55 toki Exp $
# -
# HTML data conversion

=begin
Copyright (c) 2001-2002
	TOKI Yoshinori <toki@freedom.ne.jp>. All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  1.  Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.

  2.  Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

  3.  The name of the author may not be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
=end

require 'time'
require 'wwwsrv/const'

module WWWsrv
  # latest CVS commit date of core libraries for wwwsrv
  update(%q$Date: 2003/01/05 13:05:55 $)

  module HTMLconv
    def http_date(time)
      time.httpdate
    end
    module_function :http_date

    def quoteHTML(string)
      string = string.dup
      string.gsub!(/&/, '&amp;')
      string.gsub!(/</, '&lt;')
      string.gsub!(/>/, '&gt;')
      string.gsub!(/"/, '&quot;')

      string
    end
    module_function :quoteHTML

    def unquoteHTML(string)
      string = string.dup
      string.gsub!(/&quot;/, '"')
      string.gsub!(/&gt;/, '>')
      string.gsub!(/&lt;/, '<')
      string.gsub!(/&amp;/, '&')

      string
    end
    module_function :unquoteHTML

    def quoteSHELL(string)
      # for Bourne shell
      '"' + string.gsub(/["\$\`\\]/) { |special| "\\" + special } + '"'
    end
    module_function :quoteSHELL

    def encodeURL(string)
      string = string.dup
      string.gsub!(/[^ _0-9A-Za-z.-]/) { |special| format('%%%02X', special[0]) }
      string.gsub!(/ /, '+')

      string
    end
    module_function :encodeURL

    def decodeURL(string)
      string = string.dup
      string.gsub!(/\+/, ' ')
      string.gsub!(/%([0-9A-Fa-f][0-9A-Fa-f])/) { |special| $1.hex.chr }

      string
    end
    module_function :decodeURL

    def encode_query(params, separator='&')
      params.map{ |param_name, param_value|
	if (param_value) then
	  encodeURL(param_name) + '=' + encodeURL(param_value)
	else
	  encodeURL(param_name)
	end
      }.join(separator)
    end
    module_function :encode_query

    def decode_query(string, separator=/&/)
      unless (iterator?) then
	params = {}
      end

      for param_segment in string.split(separator)
	param_name, param_value = param_segment.split(/=/, 2)
	param_name = decodeURL(param_name)
	param_value = decodeURL(param_value) if param_value
	if (iterator?) then
	  yield(param_name, param_value)
	else
	  params[param_name] = [] unless (params.include? param_name)
	  params[param_name].push(param_value)
	end
      end

      if (iterator?) then
	return
      else
	params
      end
    end
    module_function :decode_query
  end
end
