# $Id: local.rb,v 1.61.2.4 2003/04/13 00:58:50 toki Exp $
# -
# local file document

=begin
Copyright (c) 2001-2002
	TOKI Yoshinori <toki@freedom.ne.jp>. All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  1.  Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.

  2.  Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

  3.  The name of the author may not be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
=end

require 'wwwsrv/const'
require 'wwwsrv/ioutil'
require 'wwwsrv/path'
require 'wwwsrv/htmlconv'
require 'wwwsrv/document'
require 'wwwsrv/server'

module WWWsrv
  class MIMEresolver
    include Enumerable

    def initialize(default='application/octet-stream')
      @default = default
      @mime_map = {}
    end

    def clone
      new_mime = super
      new_mime.instance_eval do
	@mime_map = {}
      end

      for suffix, mime_type in self
	new_mime.add_mime_type(suffix, mime_type)
      end

      new_mime
    end

    def dup
      clone
    end

    attr_accessor :default

    def add_mime_type(suffix, mime_type)
      mime_map = @mime_map
      suffix_key = suffix.downcase
      suffix_key.reverse!
      suffix_key.each_byte do |ch|
	mime_map[ch] = {} unless (mime_map.include? ch)
	mime_map = mime_map[ch]
      end
      mime_map[:mime_type] = mime_type

      self
    end

    def resolve(path)
      mime_map = @mime_map
      path_key = path.downcase
      path_key.reverse!
      path_key.each_byte do |ch|
	if (next_map = mime_map[ch]) then
	  mime_map = next_map
	elsif (mime_type = mime_map[:mime_type]) then
	  return mime_type
	else
	  break
	end
      end

      @default
    end

    def each
      traverse(@mime_map) do |suffix, mime_type|
	yield(suffix, mime_type)
      end
    end

    def traverse(mime_map, suffix_stack=[])
      if (mime_type = mime_map[:mime_type]) then
	suffix = suffix_stack.map{ |ch| ch.chr }.join
	suffix.reverse!
	yield(suffix, mime_type)
      end

      mime_map.keys.reject{ |ch| ch == :mime_type }.sort.each do |ch|
	suffix_stack.push(ch)
	traverse(mime_map[ch], suffix_stack) do |suffix, mime_type|
	  yield(suffix, mime_type)
	end
	suffix_stack.pop
      end

      self
    end
    private :traverse
  end

  MIME_TYPE = MIMEresolver::new

  # from `mime.types' in Apache 1.3.27 distribution.
  [ %w[ application/andrew-inset	ez ],
    %w[ application/mac-binhex40	hqx ],
    %w[ application/mac-compactpro	cpt ],
    %w[ application/msword		doc ],
    %w[ application/octet-stream	bin dms lha lzh exe class so dll ],
    %w[ application/oda			oda ],
    %w[ application/pdf			pdf ],
    %w[ application/postscript		ai eps ps ],
    %w[ application/smil		smi smil ],
    %w[ application/vnd.mif		mif ],
    %w[ application/vnd.ms-excel	xls ],
    %w[ application/vnd.ms-powerpoint	ppt ],
    %w[ application/vnd.ms-project      mpp ],
    %w[ application/vnd.wap.wbxml	wbxml ],
    %w[ application/vnd.wap.wmlc	wmlc ],
    %w[ application/vnd.wap.wmlscriptc	wmlsc ],
    %w[ application/x-bcpio		bcpio ],
    %w[ application/x-cdlink		vcd ],
    %w[ application/x-chess-pgn		pgn ],
    %w[ application/x-cpio		cpio ],
    %w[ application/x-csh		csh ],
    %w[ application/x-director		dcr dir dxr ],
    %w[ application/x-dvi		dvi ],
    %w[ application/x-futuresplash	spl ],
    %w[ application/x-gtar		gtar ],
    %w[ application/x-hdf		hdf ],
    %w[ application/x-javascript	js ],
    %w[ application/x-koan		skp skd skt skm ],
    %w[ application/x-latex		latex ],
    %w[ application/x-netcdf		nc cdf ],
    %w[ application/x-sh		sh ],
    %w[ application/x-shar		shar ],
    %w[ application/x-shockwave-flash	swf ],
    %w[ application/x-stuffit		sit ],
    %w[ application/x-sv4cpio		sv4cpio ],
    %w[ application/x-sv4crc		sv4crc ],
    %w[ application/x-tar		tar ],
    %w[ application/x-tcl		tcl ],
    %w[ application/x-tex		tex ],
    %w[ application/x-texinfo		texinfo texi ],
    %w[ application/x-troff		t tr roff ],
    %w[ application/x-troff-man		man ],
    %w[ application/x-troff-me		me ],
    %w[ application/x-troff-ms		ms ],
    %w[ application/x-ustar		ustar ],
    %w[ application/x-wais-source	src ],
    %w[ application/xhtml+xml		xhtml xht ],
    %w[ application/zip			zip ],
    %w[ audio/basic			au snd ],
    %w[ audio/midi			mid midi kar ],
    %w[ audio/mpeg			mpga mp2 mp3 ],
    %w[ audio/x-aiff			aif aiff aifc ],
    %w[ audio/x-mpegurl			m3u ],
    %w[ audio/x-pn-realaudio		ram rm ],
    %w[ audio/x-pn-realaudio-plugin	rpm ],
    %w[ audio/x-realaudio		ra ],
    %w[ audio/x-wav			wav ],
    %w[ chemical/x-pdb			pdb ],
    %w[ chemical/x-xyz			xyz ],
    %w[ image/bmp			bmp ],
    %w[ image/gif			gif ],
    %w[ image/ief			ief ],
    %w[ image/jpeg			jpeg jpg jpe ],
    %w[ image/png			png ],
    %w[ image/tiff			tiff tif ],
    %w[ image/vnd.djvu			djvu djv ],
    %w[ image/vnd.wap.wbmp		wbmp ],
    %w[ image/x-cmu-raster		ras ],
    %w[ image/x-portable-anymap		pnm ],
    %w[ image/x-portable-bitmap		pbm ],
    %w[ image/x-portable-graymap	pgm ],
    %w[ image/x-portable-pixmap		ppm ],
    %w[ image/x-rgb			rgb ],
    %w[ image/x-xbitmap			xbm ],
    %w[ image/x-xpixmap			xpm ],
    %w[ image/x-xwindowdump		xwd ],
    %w[ model/iges			igs iges ],
    %w[ model/mesh			msh mesh silo ],
    %w[ model/vrml			wrl vrml ],
    %w[ text/css			css ],
    %w[ text/html			html htm shtml rhtml ],
    %w[ text/plain			asc txt text log rb rd pl pm sh ],
    %w[ text/richtext			rtx ],
    %w[ text/rtf			rtf ],
    %w[ text/sgml			sgml sgm ],
    %w[ text/tab-separated-values	tsv ],
    %w[ text/vnd.wap.wml		wml ],
    %w[ text/vnd.wap.wmlscript		wmls ],
    %w[ text/x-setext			etx ],
    %w[ text/xml			xml xsl ],
    %w[ video/mpeg			mpeg mpg mpe ],
    %w[ video/quicktime			qt mov ],
    %w[ video/vnd.mpegurl		mxu ],
    %w[ video/x-msvideo			avi ],
    %w[ video/x-sgi-movie		movie ],
    %w[ x-conference/x-cooltalk		ice ]
  ].each do |mime_type, *suffix_list|
    for suffix in suffix_list
      MIME_TYPE.add_mime_type('.' + suffix, mime_type)
    end
  end

  class LocalDirIndex < Document
    include Path

    REJECT_FILTER = [
      /^RCS$/,
      /^CVS$/,
      /,v$/,
      /~$/,
      /^\#.*\#$/,
      /^\./
    ]

    LABEL_DSC = '<dsc>'
    LABEL_ASC = '<asc>'
    LABEL_REV = '<rev>'

    NM_SORT = 'sort'
    VL_SORT_NAME  = 'name'
    VL_SORT_MTIME = 'mtime'
    VL_SORT_SIZE  = 'size'

    NM_ORDER = 'order'
    VL_ORDER_DSC = 'descending'
    VL_ORDER_ASC = 'ascending'
    VL_ORDER_REV = 'reverse'

    SORT_ANCHOR = {}
    [ [ :name_dsc,  LABEL_DSC, [ NM_SORT, VL_SORT_NAME ],  [ NM_ORDER, VL_ORDER_DSC ] ],
      [ :name_asc,  LABEL_ASC, [ NM_SORT, VL_SORT_NAME ],  [ NM_ORDER, VL_ORDER_ASC ] ],
      [ :name_rev,  LABEL_REV, [ NM_SORT, VL_SORT_NAME ],  [ NM_ORDER, VL_ORDER_REV ] ],
      [ :mtime_dsc, LABEL_DSC, [ NM_SORT, VL_SORT_MTIME ], [ NM_ORDER, VL_ORDER_DSC ] ],
      [ :mtime_asc, LABEL_ASC, [ NM_SORT, VL_SORT_MTIME ], [ NM_ORDER, VL_ORDER_ASC ] ],
      [ :size_dsc,  LABEL_DSC, [ NM_SORT, VL_SORT_SIZE ],  [ NM_ORDER, VL_ORDER_DSC ] ],
      [ :size_asc,  LABEL_ASC, [ NM_SORT, VL_SORT_SIZE ],  [ NM_ORDER, VL_ORDER_ASC ] ]
    ].each do |tag, label, *params|
      SORT_ANCHOR[tag] =
	'<a href="?' +
	HTMLconv::quoteHTML(HTMLconv::encode_query(params)) +
	'">' + HTMLconv::quoteHTML(label) + '</a>'
    end

    DEFAULT_COMPARATOR = proc{ |(a_name, a_stat), (b_name, b_stat)|
      [ a_name, a_stat.mtime, a_stat.size ] <=> [ b_name, b_stat.mtime, b_stat.size ]
    }

    COMPARATOR = {}

    [ [ [ [ NM_SORT,  VL_SORT_NAME ],
	  [ NM_ORDER, VL_ORDER_DSC ]
	],
	proc{ |(a_name, a_stat), (b_name, b_stat)|
	  [ a_name, a_stat.mtime, a_stat.size ] <=> [ b_name, b_stat.mtime, b_stat.size ]
	}
      ],
      [ [ [ NM_SORT,  VL_SORT_NAME ],
	  [ NM_ORDER, VL_ORDER_ASC ]
	],
	proc{ |(a_name, a_stat), (b_name, b_stat)|
	  [ b_name, b_stat.mtime, b_stat.size ] <=> [ a_name, a_stat.mtime, a_stat.size ]
	}
      ],
      [ [ [ NM_SORT,  VL_SORT_NAME ],
	  [ NM_ORDER, VL_ORDER_REV ]
	],
	proc{ |(a_name, a_stat), (b_name, b_stat)|
	  [ a_name.reverse, a_stat.mtime, a_stat.size ] <=> [ b_name.reverse, b_stat.mtime, b_stat.size ]
	}
      ],
      [ [ [ NM_SORT,  VL_SORT_MTIME ],
	  [ NM_ORDER, VL_ORDER_DSC  ]
	],
	proc{ |(a_name, a_stat), (b_name, b_stat)|
	  [ a_stat.mtime, a_name, a_stat.size ] <=> [ b_stat.mtime, b_name, b_stat.size ]
	}
      ],
      [ [ [ NM_SORT,  VL_SORT_MTIME ],
	  [ NM_ORDER, VL_ORDER_ASC  ]
	],
	proc{ |(a_name, a_stat), (b_name, b_stat)|
	  [ b_stat.mtime, b_name, b_stat.size ] <=> [ a_stat.mtime, a_name, a_stat.size ]
	}
      ],
      [ [ [ NM_SORT,  VL_SORT_SIZE ],
	  [ NM_ORDER, VL_ORDER_DSC ]
	],
	proc{ |(a_name, a_stat), (b_name, b_stat)|
	  [ a_stat.size, a_name, a_stat.mtime ] <=> [ b_stat.size, b_name, b_stat.mtime ]
	}
      ],
      [ [ [ NM_SORT,  VL_SORT_SIZE ],
	  [ NM_ORDER, VL_ORDER_ASC ]
	],
	proc{ |(a_name, a_stat), (b_name, b_stat)|
	  [ b_stat.size, b_name, b_stat.mtime ] <=> [ a_stat.size, a_name, a_stat.mtime ]
	}
      ]
    ].each do |params, comp|
      COMPARATOR[params] = { :file => comp, :dir => comp }
    end

    [ [ [ NM_SORT,  VL_SORT_SIZE ],
	[ NM_ORDER, VL_ORDER_DSC ]
      ],
      [ [ NM_SORT,  VL_SORT_SIZE ],
	[ NM_ORDER, VL_ORDER_ASC ]
      ]
    ].each do |params|
      COMPARATOR[params][:dir] = DEFAULT_COMPARATOR
    end

    def initialize(reject_filter=REJECT_FILTER, &comp)
      @reject_filter = reject_filter
      @comp = comp || DEFAULT_COMPARATOR
    end

    def folder?
      true
    end

    def comma(decimal)
      decimal = decimal.to_s
      decimal.reverse!
      decimal = decimal.scan(/\d\d?\d?/).join(',')
      decimal.reverse!

      decimal
    end
    private :comma

    def comparator(request)
      if ((request.params.include? NM_SORT) && (request.params.include? NM_ORDER)) then
	sort_opts = [
	  [ NM_SORT,  request.params[NM_SORT]  ],
	  [ NM_ORDER, request.params[NM_ORDER] ]
	]

	if (COMPARATOR.include? sort_opts) then
	  dir_comp = COMPARATOR[sort_opts][:dir]
	  file_comp = COMPARATOR[sort_opts][:file]
	  return sort_opts, dir_comp, file_comp
	end
      end

      return [], @comp, @comp
    end
    private :comparator

    def respond(srv_opt, prefix, request, response)
      local_dir = response[:local_path]
      response.header.content_type = 'text/html'
      yield(response)

      http_dir = request.normalized_path
      http_dir = http_dir.chomp('/') if (http_dir != '/')
      http_dir = quoteHTML(http_dir)
      yield(<<-EOF)
<html
><head
><title>Directory Index of #{quoteHTML(http_dir)}</title
></head
><body
><h1>Directory Index of #{quoteHTML(http_dir)}</h1
><pre>
      EOF

      dirs = []
      files = []
      Dir::foreach(local_dir) do |name|
	catch(:next_entry) do
	  for pattern in @reject_filter
	    if (name =~ pattern) then
	      throw(:next_entry)
	    end
	  end

	  begin
	    path_stat = File::stat(merge_local_path(local_dir, name))
	  rescue SystemCallError
	    # skip vlid symbolic link.
	    next
	  end

	  if (path_stat.ftype == 'directory') then
	    dirs.push([ name, path_stat ])
	  else
	    files.push([ name, path_stat ])
	  end
	end
      end

      sort_opts, dir_comp, file_comp = comparator(request)
      dirs.sort!(&dir_comp)
      files.sort!(&file_comp)

      dirs.map!{ |name, stat|
	[ stat.mtime.to_s, '', name ]
      }
      files.map!{ |name, stat|
	[ stat.mtime.to_s, comma(stat.size), name ]
      }

      cmd_width = LABEL_DSC.length + LABEL_ASC.length + 1
      last_modified_width = (dirs + files).map{ |last_modified, size, name| last_modified.length }.max || 0
      last_modified_width = cmd_width if (cmd_width > last_modified_width)
      size_width = files.map{ |last_modified, size, name| size.length }.max || 0
      size_width = cmd_width if (cmd_width > size_width)

      head = '       '
      head << SORT_ANCHOR[:mtime_dsc] + ' ' + SORT_ANCHOR[:mtime_asc]
      head << ' ' * (last_modified_width - cmd_width)
      head << ' '
      head << SORT_ANCHOR[:size_dsc] + ' ' + SORT_ANCHOR[:size_asc]
      head << ' ' * (size_width - cmd_width)
      head << '        '
      head << SORT_ANCHOR[:name_dsc] + ' ' + SORT_ANCHOR[:name_asc] + ' ' + SORT_ANCHOR[:name_rev]
      head << "\n"
      yield(head)

      fmt = %Q'[DIR]  %-#{last_modified_width}s %#{size_width}s        <a href="%s">%s</a>\n'
      href = '../'
      href += '?' + encode_query(sort_opts) unless sort_opts.empty?
      yield(format(fmt, '', '', quoteHTML(href), '<em>Parent Directory</em>'))
      for last_modified, size, name in dirs
	href = encodeURL(name) + '/'
	href += '?' + encode_query(sort_opts) unless sort_opts.empty?
	yield(format(fmt, quoteHTML(last_modified), quoteHTML(size), quoteHTML(href), quoteHTML(name + '/')))
      end

      fmt = %Q'[FILE] %-#{last_modified_width}s %#{size_width}s bytes  <a href="%s">%s</a>\n'
      for last_modified, size, name in files
	yield(format(fmt, quoteHTML(last_modified), quoteHTML(size), encodeURL(name), quoteHTML(name)))
      end

      yield(<<-EOF)
</pre
><hr
><address
>#{quoteHTML(SRVNAME)} version #{quoteHTML(VERSION)} (#{quoteHTML(CODENAME)})<br
>Powered by <a href="http://www.ruby-lang.org/"
>Ruby</a
></address
></body
></html>
      EOF
    end
  end

  class LocalFileDocument < Document
    include Path
    include IOunit

    INDEX_LIST = [
      'index.html',
      'index.htm'
    ]

    def initialize(root, mime_type=MIME_TYPE, dir_index=LocalDirIndex::new, index_list=INDEX_LIST)
      super()
      @root = File::expand_path(root)
      @allow = make_local_path_pattern(@root)
      @mime_type = mime_type
      @dir_index = dir_index
      @index_list = index_list
      @block_size = BUFFER_SIZE
    end

    attr_accessor :block_size

    def folder?
      true
    end

    def warm_up(log)
      super
      log.debug("#{self}: local directory index -> #{@dir_index ? @dir_index : @dir_index.inspect}")
    end

    def internal_documents
      if (@dir_index) then
	super + [ @dir_index ]
      else
	super
      end
    end

    def search_index(local_dir)
      for index_name in @index_list
	index_path = merge_local_path(local_dir, index_name)
	if (File::exist? index_path) then
	  return index_path
	end
      end

      nil
    end
    private :search_index

    def expand_directory(local_path)
      if (index_path = search_index(local_path)) then
	return index_path
      elsif (! @dir_index) then
	return merge_local_path(local_path, @index_list.first)
      else
	return local_path
      end
    end
    private :expand_directory

    def entity_tag(stat, head_block, content_type)
      etag = [
	stat.size,
	head_block.hash,
	content_type.hash,
	stat.mtime.hash
      ].map{ |n| format('%x', n) }.join('-')

      '"' + etag + '"'
    end
    private :entity_tag

    def send_file(srv_opt, request, response, local_path)
      File::open(local_path) do |file|
	file.binmode
	stat = file.stat
	block_size = io_unit(stat, @block_size)
	head_block = file.read(block_size)
	content_type = @mime_type.resolve(local_path)
	etag = entity_tag(stat, head_block, content_type)

	response[:local_path] = local_path
	response.status = request.cond_if(stat.mtime, etag)
	if (response.status == 200 && request.header.range) then
	  if (req_range = PartialContent::parse_range(request.header.range)) then
	    if (request.header.if_range) then
	      response.status = request.cond_if_range(stat.mtime, etag)
	    else
	      response.status = 206
	    end
	  end
	end
	response.header.etag = etag
	response.header.last_modified = http_date(stat.mtime)
	PartialContent::set_accept_ranges(response.header)

	case (response.status)
	when 200		# OK
	  response.header.content_type = content_type
	  response.header.content_length = stat.size.to_s
	  yield(response)
	  if (request.method == HTTP_GET) then
	    if (head_block) then
	      yield(head_block)
	      while (body_fragment = file.read(block_size))
		yield(body_fragment)
	      end
	    end
	  end
	when 206		# Partial Content
	  partial_first, partial_last = PartialContent::normalize_range(req_range[0], req_range[1], stat.size)
	  srv_opt.doc_log_debug(self, "partial content: range -> #{partial_first}..#{partial_last}")
	  response.header.content_type = content_type
	  PartialContent::setup_partial_content(response.header, partial_first, partial_last, stat.size)
	  yield(response)
	  if (request.method == HTTP_GET) then
	    file.seek(partial_first)
	    partial_length = PartialContent::partial_length(partial_first, partial_last)
	    while (partial_length > block_size && (body_fragment = file.read(block_size)))
	      yield(body_fragment)
	      partial_length -= body_fragment.length
	    end
	    while (partial_length > 0 && (body_fragment = file.read(partial_length)))
	      yield(body_fragment)
	      partial_length -= body_fragment.length
	    end
	  end
	when 304	     # Not Modified
	  # only message head
	  yield(response)
	when 412	     # Precondition Failed
	  http_error(412)
	else
	  raise "unexpected status code: #{response.status}"
	end
      end
    end
    private :send_file

    def send_dir_index(srv_opt, prefix, request, response, local_path)
      if (@dir_index) then
	response[:local_path] = local_path
	@dir_index.respond(srv_opt, prefix, request, response) do |arg, opts|
	  yield(arg, opts)
	end
      else
	http_error(404, "Not found a document: #{request.normalized_path}.")
      end
    end
    private :send_dir_index

    def redirect_directory(srv_opt, request, response)
      response.status = 301
      response.header.location = srv_opt.expand_location(request, request.normalized_path + '/')
      response.header.content_type = 'text/plain'
      location = response.header.location
      yield(response)
      yield("Moved to #{location}.\n")
    end
    private :redirect_directory

    def respond(srv_opt, prefix, request, response)
      srv_opt.doc_log_debug(self, 'enter')
      local_path = path_http2local(request.subpath(prefix), @root)
      srv_opt.doc_log_debug(self, "local document path: #{local_path}")
      if (local_path !~ @allow) then
	http_error(404, "Not found a document: #{request.normalized_path}.")
      end
      srv_opt.doc_log_debug(self, "allow access: #{local_path}")

      case (request.method)
      when HTTP_GET, HTTP_HEAD
	local_path = expand_directory(local_path) if (request.normalized_path =~ %r"/$")
	if ((File::file? local_path) && (File::readable? local_path)) then
	  send_file(srv_opt, request, response, local_path) do |arg, opts|
	    yield(arg, opts)
	  end
	elsif ((File::directory? local_path) && (File::readable? local_path)) then
	  if (request.normalized_path =~ %r"/$") then
	    send_dir_index(srv_opt, prefix, request, response, local_path) do |arg, opts|
	      yield(arg, opts)
	    end
	  else
	    redirect_directory(srv_opt, request, response) do |arg, opts|
	      yield(arg, opts)
	    end
	  end
	else
	  http_error(404, "Not found a document: #{request.normalized_path}.")
	end
      else
	http_error(405, "Not allowed request method: #{request.method}.", 'Allow' => 'GET, HEAD')
      end

      self
    end
  end
end
