# $Id: process.rb,v 1.8 2002/06/13 15:40:35 toki Exp $
# -
# process resources

=begin
Copyright (c) 2001-2002
	TOKI Yoshinori <toki@freedom.ne.jp>. All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  1.  Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.

  2.  Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

  3.  The name of the author may not be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
=end

require 'thread'
require 'wwwsrv/const'

module WWWsrv
  # latest CVS commit date of core libraries for wwwsrv
  update(%q$Date: 2002/06/13 15:40:35 $)

  PROCESS_LOCK = Mutex::new
  # lock for process global resources
  # ex.) ENV, Dir::chdir, ...

  module ProtectProcess
    def protect_environ(temp_var_pairs, preserved_var_names=[ 'PATH' ])
      if (preserved_var_names) then
	preserved_var_pairs = preserved_var_names.map{ |name| [ name, ENV[name] ] }
      else
	preserved_var_pairs = []
      end
      orig_var_pairs = ENV.to_a
      begin
	for name, value in orig_var_pairs
	  ENV.delete(name)
	end
	for modified_var_pairs in [ preserved_var_pairs, temp_var_pairs ]
	  for name, value in modified_var_pairs
	    ENV[name] = value
	  end
	end
	return yield
      ensure
	for name, value in ENV.to_a
	  ENV.delete(name)
	end
	for name, value in orig_var_pairs
	  ENV[name] = value
	end
      end
    end
    module_function :protect_environ

    def protect_chdir(dirname)
      cwd = Dir::getwd
      begin
	Dir::chdir(dirname)
	return yield
      ensure
	Dir::chdir(cwd)
      end
    end
    module_function :protect_chdir
  end
end
