# $Id: srvopt.rb,v 1.26 2002/11/17 04:33:37 toki Exp $
# -
# server option

=begin
Copyright (c) 2001-2002
	TOKI Yoshinori <toki@freedom.ne.jp>. All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  1.  Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.

  2.  Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

  3.  The name of the author may not be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
=end

require 'wwwsrv/const'
require 'wwwsrv/cache'
require 'wwwsrv/path'

module WWWsrv
  # latest CVS commit date of core libraries for wwwsrv
  update(%q$Date: 2002/11/17 04:33:37 $)

  module Attributes
    include Enumerable

    def [](attr_key)
      option[attr_key]
    end

    def []=(attr_key, attr_value)
      option[attr_key] = attr_value
    end

    def keys
      option.keys
    end

    def values
      option.values
    end

    def key?(attr_key)
      option.key? attr_key
    end

    alias has_key? key?
    alias include? key?

    def each_pair
      option.each do |attr_key, attr_value|
	yield(attr_key, attr_value)
      end

      nil
    end

    alias each each_pair

    def each_key
      option.each_key do |attr_key|
	yield(attr_key)
      end

      nil
    end

    def each_value
      option.each_value do |attr_value|
	yield(attr_value)
      end

      nil
    end

    def each_symbol
      option.each do |attr_key, attr_value|
	case (attr_key)
	when Symbol
	  yield(attr_key, attr_value)
	when String
	  yield(attr_key.intern, attr_value)
	end
      end

      nil
    end
  end

  class ServerOption
    include Attributes

    def initialize(sid=nil, root=nil, log=nil, scheme=HTTP_SCHEME)
      @sid = sid
      @root = root
      @log = log
      @scheme = scheme
      @option = {}
    end

    def clear
      @sid = nil
      @root = nil
      @log = nil
      @scheme = HTTP_SCHEME
      @option.clear

      self
    end

    attr_accessor :sid
    attr_accessor :root
    attr_accessor :log
    attr_accessor :scheme

    # for hook of WWWsrv::Attribute module.
    attr_reader :option; private :option

    def expand_location(request, location_path)
      Path::expand_location(request.host, request.normalized_path, location_path, @scheme)
    end

    def srv_log_print(priority, msg=nil)
      if (msg) then
	if (iterator?) then
	  raise Arguments, 'conflict arguments vs. block'
	end
	log.print(priority, Logger::srv_format(@sid, msg))
      else
	log.print(priority) { Logger::srv_format(@sid, yield) }
      end

      nil
    end

    def srv_log_internal(msg=nil, &block)
      srv_log_print(Logger::INTERNAL, msg, &block)
    end

    def srv_log_debug(msg=nil, &block)
      srv_log_print(Logger::DEBUG, msg, &block)
    end

    def srv_log_system(msg=nil, &block)
      srv_log_print(Logger::SYSTEM, msg, &block)
    end

    def srv_log_message(msg=nil, &block)
      srv_log_print(Logger::MESSAGE, msg, &block)
    end

    def srv_log_warning(msg=nil, &block)
      srv_log_print(Logger::WARNING, msg, &block)
    end

    def srv_log_notice(msg=nil, &block)
      srv_log_print(Logger::NOTICE, msg, &block)
    end

    def srv_log_alert(msg=nil, &block)
      srv_log_print(Logger::ALERT, msg, &block)
    end

    def srv_log_error(msg=nil, &block)
      srv_log_print(Logger::ERROR, msg, &block)
    end

    def doc_log_print(priority, doc, msg=nil)
      if (msg) then
	if (iterator?) then
	  raise Arguments, 'conflict arguments vs. block'
	end
	log.print(priority, Logger::doc_format(@sid, doc, msg))
      else
	log.print(priority) { Logger::doc_format(@sid, doc, yield) }
      end

      nil
    end

    def doc_log_internal(doc, msg=nil, &block)
      doc_log_print(Logger::INTERNAL, doc, msg, &block)
    end

    def doc_log_debug(doc, msg=nil, &block)
      doc_log_print(Logger::DEBUG, doc, msg, &block)
    end

    def doc_log_system(doc, msg=nil, &block)
      doc_log_print(Logger::SYSTEM, doc, msg, &block)
    end

    def doc_log_message(doc, msg=nil, &block)
      doc_log_print(Logger::MESSAGE, doc, msg, &block)
    end

    def doc_log_warning(doc, msg=nil, &block)
      doc_log_print(Logger::WARNING, doc, msg, &block)
    end

    def doc_log_notice(doc, msg=nil, &block)
      doc_log_print(Logger::NOTICE, doc, msg, &block)
    end

    def doc_log_alert(doc, msg=nil, &block)
      doc_log_print(Logger::ALERT, doc, msg, &block)
    end

    def doc_log_error(doc, msg=nil, &block)
      doc_log_print(Logger::ERROR, doc, msg, &block)
    end

    CACHE = RecycleObjectCache::new(self)

    def self::fetch_cache(&block)
      CACHE.fetch(&block)
    end

    def self::store_cache(old_obj)
      CACHE.store(old_obj)
    end
  end
end
