# $Id: token.rb,v 1.12.2.1 2003/04/11 14:09:11 toki Exp $
# -
# token parser

=begin
Copyright (c) 2001-2002
	TOKI Yoshinori <toki@freedom.ne.jp>. All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  1.  Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.

  2.  Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

  3.  The name of the author may not be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
=end

require 'wwwsrv/const'

module WWWsrv
  # latest CVS commit date of core libraries for wwwsrv
  update(%q$Date: 2003/04/11 14:09:11 $)

  module Token
    def fetch_separator(value, destructive=false)
      value = value.dup unless destructive
      value.sub!(/^\s+/, '')
      value.sub!(%r'^[\s\(\)<>@,;:\\"/\[\]\?=\{\}]', '')
      $&
    end
    module_function :fetch_separator

    def fetch_token(value, destructive=false)
      value = value.dup unless destructive
      value.sub!(/^\s+/, '')
      value.sub!(%r'^[^\s\(\)<>@,;:\\"/\[\]\?=\{\}]+', '')
      $&
    end
    module_function :fetch_token

    def fetch_quoted(value, destructive=false)
      value = value.dup unless destructive
      value.sub!(/^\s+/, '')
      if (value !~ /^"/) then
	return nil
      end

      state = :begin
      quoted_string = nil
      while (value.sub!(/^["\\]/, '') || value.sub!(/^[^"\\]+/, ''))
	fragment = $&
	case (state)
	when :begin
	  if (fragment != '"') then
	    raise 'internal error'
	  end
	  quoted_string = ''
	  state = :string
	when :string
	  case (fragment)
	  when '"'
	    break
	  when '\\'
	    state = :quoted_pair
	  else
	    quoted_string.concat(fragment)
	  end
	when :quoted_pair
	  quoted_string.concat(fragment)
	  state = :string
	else
	  raise 'internal error'
	end
      end

      quoted_string
    end
    module_function :fetch_quoted

    def fetch_comment(value, destructive=false)
      value = value.dup unless destructive
      value.sub!(/^\s+/, '')
      if (value !~ /^\(/) then
	return nil
      end

      state = :begin
      depth = 0
      comment = nil
      while (value.sub!(/^[\(\)\\]/, '') || value.sub!(/^[^\(\)\\]+/, ''))
	fragment = $&
	case (state)
	when :begin
	  if (fragment != '(') then
	    raise 'internal error'
	  end
	  comment = ''
	  state = :comment
	when :comment
	  case (fragment)
	  when '('
	    comment.concat('(')
	    depth += 1
	  when ')'
	    if (depth > 0) then
	      comment.concat(')')
	      depth -= 1
	    else
	      break
	    end
	  when '\\'
	    state = :quoted_pair
	  else
	    comment.concat(fragment)
	  end
	when :quoted_pair
	  comment.concat(fragment)
	  state = :comment
	else
	  raise 'internal error'
	end
      end

      comment
    end
    module_function :fetch_comment

    def quote_token(word, enforce=false)
      if (enforce || word =~ %r'[\s\(\)<>@,;:\\"/\[\]\?=\{\}]') then
	return '"' + word.gsub(/["\\]/, '\\\\\&') + '"'
      end

      word
    end
    module_function :quote_token
  end
end
