# $Id: version.rb,v 1.10.2.1 2003/04/11 14:09:12 toki Exp $
# -
# HTTP version

=begin
Copyright (c) 2001-2002
	TOKI Yoshinori <toki@freedom.ne.jp>. All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  1.  Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.

  2.  Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

  3.  The name of the author may not be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
=end

require 'wwwsrv/const'
require 'wwwsrv/cache'

module WWWsrv
  # latest CVS commit date of core libraries for wwwsrv
  update(%q$Date: 2003/04/11 14:09:12 $)

  class HTTPVersionFormatError < ParseError; end

  class HTTPversion
    include Comparable

    def initialize(major, minor)
      @major = major
      @minor = minor
    end

    attr_reader :major
    attr_reader :minor

    def <=>(other)
      [ major, minor ] <=> [ other.major, other.minor ]
    end

    def to_s
      "HTTP/#{@major}.#{@minor}"
    end

    CACHE = ImmutableObjectCache::new{ |*args| self::new(*args) }

    def self::fetch_cache(*args)
      CACHE.fetch(*args)
    end

    PARSE_CACHE = ImmutableObjectCache::new{ |version|
      if (version !~ %r"^HTTP/(\d+)\.(\d+)$") then
	raise HTTPVersionFormatError, "invalid HTTP version format: #{version.inspect}"
      end
      self::fetch_cache($1.to_i, $2.to_i)
    }

    def self::parse(version)
      PARSE_CACHE.fetch(version)
    end
  end

  # alias
  HTTPVersion = HTTPversion

  HTTP_0_9 = HTTPversion::fetch_cache(0, 9)
  HTTP_1_0 = HTTPversion::fetch_cache(1, 0)
  HTTP_1_1 = HTTPversion::fetch_cache(1, 1)
  HTTP_2_0 = HTTPversion::fetch_cache(2, 0)
end
